@@ -61,6 +61,10 @@
 #undef __EXTENSIONS__
 #undef _POSIX_C_SOURCE
 
+/* Solaris 2.6 */
+
+#undef __BIT_TYPES_DEFINED__
+
 /* LOCK and LOG directories */
 
 #undef LOCKFILEDIR
