@@ -33,7 +33,7 @@ extern int errno;
 /* Write LEN bytes at PTR to descriptor DESC, retrying if interrupted.
    Return LEN upon success, write's (negative) error code otherwise.  */
 
-full_write (desc, ptr, len)
+cf_full_write (desc, ptr, len)
 
 int desc;
 char *ptr;
