@@ -254,7 +254,7 @@ Debug1("AppendTwoDimItem(itemlist, sep=%c)\n",sep);
 
 if ((ip = (struct TwoDimList *)malloc(sizeof(struct TwoDimList))) == NULL)
    {
-   perror("Can't allocate memory in AppendTwoDimItem()");
+   CfLog(cferror,"AppendTwoDimItem""malloc");
    FatalError("");
    }
 
