@@ -59,6 +59,10 @@ struct utsname
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#ifdef HAVE_SYS_SYSTEMINFO_H
+# include <sys/systeminfo.h>
+#endif
+
 #ifdef HAVE_SYS_PARAM_H
 # include <sys/param.h>
 #endif
@@ -103,7 +107,7 @@ struct utsname
 
 #include <signal.h>
 
-
+#include <syslog.h>
 extern int errno;
 
 /* Do this for ease of configuration from the Makefile */
@@ -200,6 +204,10 @@ extern int errno;
 # include <linux/in.h>
 #endif
 
+#ifndef HAVE_STRDUP
+char *strdup();
+#endif
+
 
 #include "../pub/regex.h"
 
@@ -232,9 +240,9 @@ extern int errno;
 /* This is the only place you ever need to edit anything           */
 /*******************************************************************/
 
-#define clssattr 28         /* increase this for each new class added */
+#define clssattr 29         /* increase this for each new class added */
                             /* It defines the array size for class data */
-#define attr 3              /* Only used in CLASSATTRUBUTES[][] defn */
+#define attrdim 3           /* Only used in CLASSATTRUBUTES[][] defn */
 
    /* end class array limits */
 
@@ -260,7 +268,7 @@ extern int errno;
 /* Output control defines */
 
 #define Verbose if (VERBOSE || DEBUG || D2) printf
-#define EditVerbose  if (EDITVERBOSE || DEBUG) printf
+#define EditVerbose  if (EDITVERBOSE || DEBUG || D2) printf
 #define Debug3  if (D3 || DEBUG || D2) printf
 #define Debug2  if (DEBUG || D2) printf
 #define Debug1  if (DEBUG || D1) printf
@@ -428,7 +436,7 @@ enum actions
    unmounta,
    admit,
    deny,
-   acla
+   acls
    };
 
 /*******************************************************************/
@@ -512,6 +520,10 @@ enum fileattr
    cfdefine,
    cftimeout,
    cffree,
+   cfnofile,
+   cfacl,
+   cfpurge,
+   cfuseshell,
    cfbad
    };
 
@@ -698,7 +710,8 @@ enum editnames
    ReplaceLinesMatchingField,
    SplitOn,
    AppendToLineIfNotContains,
-   DeleteLinesAfterThisMatching
+   DeleteLinesAfterThisMatching,
+   DefineClasses
    };
 
 enum editnames EditActionsToCode();
@@ -752,6 +765,18 @@ enum SignalNames
 
 /*******************************************************************/
 
+enum cfoutputlevel
+   {
+   cfsilent,
+   cfinform,
+   cfverbose,
+   cfeditverbose,
+   cferror,
+   cflogonly
+   };
+
+/*******************************************************************/
+
 enum actions ActionStringToCode();
 enum classes ClassStringToCode();
 enum vnames ScanVariable();
@@ -807,9 +832,11 @@ struct Process
    char           *restart;       /* shell comm to be done after */
    short          matches;
    char           comp;
+   char           *defines;
    short          signal;
    char           action;
    char           *classes;
+   char           useshell;
    struct Process *next;
    };
 
@@ -840,6 +867,7 @@ struct Tidy
          int                size;              /* in bytes */
          char               *pattern;
          char               *classes;
+	 char               *defines;
          char               travlinks;
          char               dirlinks;          /* k=keep, t=tidy */
          char               rmdirs;            /* t=true, f=false */
@@ -874,6 +902,7 @@ struct MiscMount
 struct File
    {
    char   *path;
+   char   *defines;
    enum   fileactions action;
    mode_t plus;
    mode_t minus;
@@ -885,6 +914,7 @@ struct File
    struct UidList *uid;
    struct GidList *gid;
    struct File *next;
+   struct Item *acl_aliases;
    };
 
 /*******************************************************************/
@@ -908,6 +938,7 @@ struct Disable
    short  rotate;
    int    size;
    char   comp;
+   char   *defines;
    struct Disable *next;
    };
 
@@ -933,6 +964,7 @@ struct Image
    int    makeholes;
    int    size;
    char   comp;
+   char   purge;
    struct Item *exclusions;
    struct Item *inclusions;
    struct Item *symlink;
@@ -940,9 +972,10 @@ struct Image
    struct cfstat *cache;                              /* stat cache */
    struct CompressedArray *inode_cache;              /* inode cache */
 
-   struct hostent *dns;                       /* Cache gethostbyname */
+   struct in_addr *dns;                      /* Cache gethostbyname */
    
    struct Image *next;
+   struct Item *acl_aliases;
    };
 
 /*******************************************************************/
@@ -967,12 +1000,49 @@ struct GidList *MakeGidList();
 
 /*******************************************************************/
 
+enum cffstype
+   {
+   posixfs,
+   solarisfs,
+   dfsfs,
+   afsfs,
+   hpuxfs,
+   ntfs,
+   badfs
+   };
+
+/*******************************************************************/
+
+struct CFACL
+   {
+   char * acl_alias;
+   enum   cffstype type;
+   char   method;            /* a = append, o = overwrite */
+   struct CFACE *aces;
+   struct CFACL *next;
+   };
+
+/*******************************************************************/
+
+struct CFACE
+   {
+   char *mode;        /* permission flags*/
+   char *name;        /* id name */
+   char *acltype;     /* user / group / other */
+   char *classes;
+   struct CFACE *next;
+   };
+
+/*******************************************************************/
+
 struct Link
    {
    char   *from;
    char   *to;
    char   *classes;
+   char   *defines;
    short  force;
+   short  nofile;
    short  silent;
    char   type;
    char   copytype;
@@ -988,6 +1058,7 @@ struct Link
 struct Edit
    {
    char *fname;
+   char *defines;
    struct Edlist *actions;
    struct Edit *next;
    };
@@ -1010,6 +1081,7 @@ struct ShellComm
    char              *name;
    char              *classes;
    int               timeout;
+   char              useshell;
    struct ShellComm  *next;
    };
 
@@ -1106,3 +1178,6 @@ char ToLower();
 char *ToUpperStr();
 char *ToLowerStr();
 char *ReadLastNode();
+char *AbsLinkPath();
+
+FILE *cfpopen();
