@@ -43,7 +43,7 @@ extern char VARCH[];
 extern char *ACTIONTEXT[]; 
 extern char *ACTIONID[];
 extern char *CLASSTEXT[];
-extern char *CLASSATTRIBUTES[clssattr][attr];
+extern char *CLASSATTRIBUTES[clssattr][attrdim];
 extern char *FILEACTIONTEXT[];
 extern char *COMMATTRIBUTES[];
 extern char VINPUTFILE[];
@@ -56,6 +56,8 @@ extern char VEDITABORT[];
 extern char LISTSEPARATOR;
 extern char REPOSCHAR;
 extern char DISCOMP;
+extern char USESHELL;
+extern char PURGE;
 extern int  DISABLESIZE;
 
 extern char VLOGDIR[];
@@ -106,6 +108,7 @@ extern char   IMAGEACTION[bufsize];
 extern char *HASH[hashtablesize];
 
 extern char VBUFF[bufsize];
+extern char OUTPUT[bufsize];
 
 extern char VFACULTY[maxvarsize];
 extern char VDOMAIN[maxvarsize];
@@ -130,6 +133,7 @@ extern struct Item *VLINKCOPIES;
 extern struct Item *VEXCLUDEPARSE;
 extern struct Item *VCPLNPARSE;
 extern struct Item *VINCLUDEPARSE;
+extern struct Item *VACLBUILD;
 
 extern struct Item *VMOUNTLIST;
 extern struct Item *VHEAP;      /* Points to the base of the attribute heap */
@@ -165,6 +169,8 @@ extern struct Item *VADDCLASSES;
 extern struct Item *VALLADDCLASSES;
 extern struct Edit *VEDITLIST;
 extern struct Edit *VEDITLISTTOP;
+extern struct CFACL  *VACLLIST;
+extern struct CFACL  *VACLLISTTOP;
 extern struct Item *VUNMOUNT;
 extern struct Item *VCLASSDEFINE;
 extern struct Image *VIMAGE;
@@ -197,8 +203,11 @@ extern short D3;
 extern short ISCFENGINE;
 
 extern short VERBOSE;
+extern short INFORM;
+extern short LOGGING;
 extern short TIDYDIRS;
 extern short TRAVLINKS;
+extern short DEADLINKS;
 extern short PTRAVLINKS;
 extern short DONTDO;
 extern short IFCONF;
@@ -278,6 +287,7 @@ extern int VIFELAPSED;
 extern int VEXPIREAFTER;
 extern int VDEFAULTIFELAPSED;
 extern int VDEFAULTEXPIREAFTER;
+extern int AUTOCREATED;
 
 extern enum fileactions FILEACTION;
 
