@@ -35,29 +35,35 @@
 
 #define INET 1
 
+
 #include "cf.defs.h"
 #include "cf.extern.h"
 #include "../pub/getopt.h"
 #include "cfd.h"
 
-#if HAVE_SYS_IPC_H
-#include <sys/ipc.h>
-#endif
-
-#if HAVE_SYS_SHM_H
-#include <sys/shm.h>
-#endif
-
-#if HAVE_TCPD_H
+#ifdef HAVE_TCPD_H
 # include <tcpd.h>
 #endif
 
-#if HAVE_DB_H
-#include <db.h>
+#ifdef HAVE_DB_H
+# include <db.h>
 #endif
 
 #if HAVE_DB_DB_H
-#include <db/db.h>
+# include <db/db.h>
+#endif
+
+#ifdef HAVE_PTHREAD_H
+# include <pthread.h>
+#endif
+
+#ifdef HAVE_SCHED_H
+# include <sched.h>
+#endif
+
+#ifdef HAVE_PTHREAD_H
+# define MAXTHREADS 10
+pthread_t THREADS[MAXTHREADS];
 #endif
 
 /*******************************************************************/
@@ -82,6 +88,8 @@ if (PARSEONLY)
 CheckVariables();
 SummarizeParsing();
 StartServer(argc,argv);
+
+/* Never exit here */
 }
 
 /********************************************************************/
@@ -126,6 +134,9 @@ while ((c=getopt_long(argc,argv,"d:f:vhpFV",CFDOPTIONS,&optindex)) != EOF)
                    default:  DEBUG = true;
                              break;
                    }
+		
+		NO_FORK = true;
+		printf("cfd: Debug mode: running in foreground\n");
                 break;
 
       case 'v': VERBOSE = true;
@@ -150,7 +161,10 @@ while ((c=getopt_long(argc,argv,"d:f:vhpFV",CFDOPTIONS,&optindex)) != EOF)
    }
 
 
+LOGGING = true;                    /* Do output to syslog */
+ 
 sprintf(VBUFF,"%s/test",LOCKFILEDIR);
+sprintf(VPREFIX,"cfd:%s:",VUQNAME); 
 
 MakeDirectoriesFor(VBUFF);
 strcpy(VLOCKDIR,LOCKFILEDIR);
@@ -318,15 +332,26 @@ char **argv;
   struct cfd_connection conn;
   int portnumber, yes=1, shmid;
   void ExitCleanly();
+
+
+openlog(VPREFIX,LOG_PID|LOG_NOWAIT|LOG_ODELAY,LOG_DAEMON);
   
 if ((!NO_FORK) && (fork() != 0))
    {
+   sprintf(OUTPUT,"cfd: starting %s\n",ctime(&CFSTARTTIME));
+   CfLog(cfinform,OUTPUT,"");
    return;
    }
 
 if (!NO_FORK)
   {
-  setpgrp();
+#ifdef HAVE_SETSID
+  setsid();  /* BSD has arguments, SV doesn't */
+#endif 
+  fclose (stdin);
+  fclose (stdout);
+  fclose (stderr);
+  closelog();
   }
 
 signal(SIGINT,ExitCleanly);
@@ -334,29 +359,22 @@ signal(SIGTERM,ExitCleanly);
 signal(SIGHUP,SIG_IGN);
 signal(SIGPIPE,SIG_IGN);
 
-if ((shmid = shmget(IPC_PRIVATE,sizeof(int),IPC_CREAT)) == -1)
-    {
-    printf("cfd: Couldn't get shared memory object\n");
-    perror("shmget");
-    exit(0);
-    }
+#ifdef HAVE_PTHREAD_T 
 
-if ((SHM_DAEMON_COUNT = (int *) shmat(shmid,0,0)) == (void *) -1)
+for (i = 0; i < MAXTHREADS; i++)
    {
-   printf("cfd: Couldn't attach shared memory object\n");
-   perror("shmat");
-   exit(0);
+   THREADS[i] = NULL;
    }
-
-*SHM_DAEMON_COUNT = 0;
-
+ 
+#endif
+ 
 if ((server = getservbyname(CFENGINE_SERVICE,"tcp")) == NULL)
    {
-   perror("getservbyname");
+   CfLog(cferror,"Couldn't get cfengine service","getservbyname");
    exit (1);
    }
 
-bzero(&cin,sizeof(cin));
+ bzero(&cin,sizeof(cin));
 
 sin.sin_port = (unsigned short)(server->s_port); /*  Service returns network byte order */
 sin.sin_addr.s_addr = INADDR_ANY;
@@ -364,25 +382,25 @@ sin.sin_family = AF_INET;
 
 if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
    {
-   perror("socket");
+   CfLog(cferror,"Couldn't open socket","socket");
    exit (1);
    }
 
 if (setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, (char *) &yes, sizeof (int)) == -1)
    {
-   perror ("setsockopt");
+   CfLog(cferror,"Couldn't set socket options","sockopt");
    exit (1);
    }
 
 if (bind(sd,(struct sockaddr *)&sin,sizeof(sin)) == -1)  /* Must have this on server */
    {
-   perror("bind");
+   CfLog(cferror,"Couldn't bind to socket","bind");
    exit(1);
    }
 
 if (listen(sd,queuesize) == -1)
    {
-   perror("listen");
+   CfLog(cferror,"listen failed","listen");
    exit(1);
    }
 
@@ -411,7 +429,6 @@ while (true)
    }
 }
 
-
 /*********************************************************************/
 /* Level 2                                                           */
 /*********************************************************************/
@@ -513,7 +530,7 @@ switch (GetCommand(recvbuffer))
 		     thr_args.replybuff = sendbuffer;
 		     thr_args.replyfile = filename;
 		     
-		     SpawnGetFile(&thr_args);
+		     SpawnCfGetFile(&thr_args);
 		     return true;
                      break;
 		     
@@ -559,7 +576,8 @@ switch (GetCommand(recvbuffer))
 
                      if (tloc - trem > CLOCK_DRIFT)
 			{
-			Verbose("Clocks are too far unsynchronized %ld/%ld\n",(long)tloc,(long)trem);
+			sprintf(OUTPUT,"Clocks are too far unsynchronized %ld/%ld\n",(long)tloc,(long)trem);
+			CfLog(cfinform,OUTPUT,"");
 			}
 		     else
 			{
@@ -586,10 +604,11 @@ switch (GetCommand(recvbuffer))
    }
 
 sprintf (sendbuffer,"BAD: Malformed protocol request\n%c",EOF);
+CfLog(cfinform,"Malformed protocol request\n",""); 
 
 if (send(conn->sd_reply,sendbuffer,strlen(sendbuffer)+1,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    exit(1);
    }
 
@@ -606,7 +625,7 @@ char **argv;
 int sd;
 
 { struct stat newstat;
-  char filename[bufsize], line[bufsize], *sp;
+  char filename[bufsize],line[bufsize], *sp, **argvtest;
   FILE *pp;
 
 bzero(&newstat,sizeof(struct stat));
@@ -618,68 +637,40 @@ if ((sp=getenv(CFINPUTSVAR)) != NULL)
    if (! IsAbsoluteFileName(VINPUTFILE))     /* Don't prepend to absolute names */
       { 
       strcpy(filename,sp);
-      if (filename[strlen(filename)-1] != '/')
-	 {
-	 strcat(filename,"/");
-	 }
+      AddSlash(filename);
       }
    }
 
 strcat(filename,VINPUTFILE);
 
-stat(filename,&newstat);
+if (stat(filename,&newstat) == -1)
+   {
+   sprintf(OUTPUT,"Input file %s missing!\n",filename);
+   return;
+   }
 
 Debug("Checking file updates on %s \n",filename);
 
 if (CFSTARTTIME < newstat.st_ctime)
    {
-   Verbose("cfd: restarting and rereading config files %s..\n",filename);
+   sprintf(OUTPUT,"Restarting and rereading config files %s..\n",filename);
+   CfLog(cfinform,OUTPUT,"");
 
-   sprintf(filename,"%s -p 2>&1\n",argv[0]);
-
-   Verbose("cfd: checking ok to restart %s\n",filename);
-
-   if ((pp=popen(filename,"r")) == NULL)
+   argvtest[0] = argv[0];
+   argvtest[1] = "-p";
+   argvtest[2] = NULL;
+   
+   if (execv(argv[0],argvtest) == -1)
       {
-      printf("cfd: unable to restart process\n");
+      CfLog(cferror,"Unable to restart cfd process\n","execv");
       return;
       }
 
-   while (!feof(pp))
-      {
-      bzero(line,bufsize);
-	 
-      if (ferror(pp))  /* abortable */
-	 {
-         pclose(pp);
-	 printf("cfd: unable to restart process\n");
-	 return;
-	 }
-	 
-      fgets(line,bufsize,pp);
-
-      if (strlen(line) > 0)
-	 {
-	 pclose(pp);
-	 printf("cfd: unable to restart process\n");     
-         return;
-	 }
-
-      if (ferror(pp))  /* abortable */
-	 {
-	 pclose(pp);
- 	 printf("cfd: unable to restart process\n");     
-         return;
-	 }	 
-      }
-
-   pclose(pp);
-   
    close(sd);
 
    if (execv(argv[0],argv) == -1)
       {
-      printf("cfd: unable to restart process\n");
+      CfLog(cferror,"Unable to restart cfd process\n","execv");
       }
    }
 }
@@ -689,8 +680,21 @@ if (CFSTARTTIME < newstat.st_ctime)
 
 void ExitCleanly()
 
-{
-Verbose("cfd: Exit %d\n",getpid());
+{ int i;
+sprintf(OUTPUT,"Exit -- (waiting for threads), pid = %d\n",getpid());
+CfLog(cfinform,OUTPUT,"");
+closelog();
+
+#ifdef HAVE_PTHREAD_H 
+for (i = 0; i < MAXTHREADS; i++)
+   {
+   if (THREADS[i] != NULL)
+      {
+      pthread_join(THREADS[i],(void **)NULL);
+      }
+   }
+#endif 
+ 
 exit(0);
 }
 
@@ -787,7 +791,7 @@ if (GetMacroValue("cfrunCommand") == NULL)
    
    if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
       }
    return;
    }
@@ -814,13 +818,13 @@ if (!GetLock("cfd","exec",VIFELAPSED,VEXPIREAFTER,VUQNAME,CFSTARTTIME))
 
    if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
       }
    
    return;
    }
 
-ExpandVarstring("$(cfrunCommand) --no-splay ",buffer,"");
+ExpandVarstring("$(cfrunCommand) --no-splay --inform",buffer,"");
 
 if (strlen(buffer)+strlen(args)+6 > bufsize)
    {
@@ -828,7 +832,7 @@ if (strlen(buffer)+strlen(args)+6 > bufsize)
 
    if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
       }
 
    ReleaseCurrentLock();
@@ -845,25 +849,26 @@ else
 
       if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
          {
-         perror("send");
+         CfLog(cferror,"","send");
          }
       }
    }
 
-strcat(buffer," 2>&1");
-
-Verbose("cfd: Executing command %s\n",buffer);
+sprintf(OUTPUT,"Executing command %s\n",buffer);
+CfLog(cfinform,OUTPUT,""); 
 
 bzero(sendbuffer,bufsize);
 
-if ((pp = popen(buffer,"r")) == NULL)
+if ((pp = cfpopen(buffer,"r")) == NULL)
    {
-   printf("cfd: Couldn't open pipe to command %s\n",buffer);
-   perror("popen");
+   sprintf(OUTPUT,"Couldn't open pipe to command %s\n",buffer);
+   CfLog(cferror,OUTPUT,"pipe");
+
    sprintf(sendbuffer,"Unable to run %s\n",buffer);
+
    if (send(conn->sd_reply,sendbuffer,strlen(sendbuffer),0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
       }
    ReleaseCurrentLock();
    return;
@@ -871,24 +876,20 @@ if ((pp = popen(buffer,"r")) == NULL)
 
 while (!feof(pp))
    {
-   bzero(line,bufsize);
-	 
    if (ferror(pp))  /* abortable */
       {
-      Verbose("cfd: shell command %s\n",buffer);
       fflush(pp);
       break;
       }
-	 
-   fgets(line,bufsize,pp);
 
+   ReadLine(line,bufsize,pp);
+   
    if (ferror(pp))  /* abortable */
       {
       fflush(pp);
       break;
       }	 
 
-   Chop(line);
    print = false;
 	 
    for (sp = line; *sp != '\0'; sp++)
@@ -906,14 +907,14 @@ while (!feof(pp))
 
       if (send(conn->sd_reply,sendbuffer,strlen(sendbuffer),0) == -1)
          {
-         perror("send");
+         CfLog(cferror,"","send");
 	 fflush(pp);
 	 break;
          }
       }
   }
       
-pclose(pp);
+cfpclose(pp);
 ReleaseCurrentLock();
 }
 
@@ -948,7 +949,8 @@ struct cfd_connection *conn;
 char buf[bufsize];
 
 { struct sockaddr_in saddr,raddr;
-  char ipstring[maxvarsize], fqname[maxvarsize];
+  char ipstring[maxvarsize], fqname[bufsize];
+  char name1[bufsize], name2[bufsize], name3[bufsize];
   struct hostent *hp;
   int len;
    
@@ -962,16 +964,20 @@ len = sizeof(struct sockaddr_in);
 
 if (getpeername(conn->sd_reply,(struct sockaddr *)&saddr,&len) == -1)
    {
-   printf("cfd: couldn't get socket address\n");
-   perror("getsockname");
+   CfLog(cferror,"Couldn't get socket address\n","getsockname");
    return false;
    }
 
-Verbose("Socket originates from %s=%s\n",inet_ntoa(saddr.sin_addr),fqname);
+strcpy(name1,ToLowerStr(fqname));
+strcpy(name2,ToLowerStr(inet_ntoa(saddr.sin_addr))); 
+strcpy(name3,ToLowerStr(ipstring));
+ 
+sprintf(OUTPUT,"Socket originates from %s=%s\n",name2,name1);
+CfLog(cfinform,OUTPUT,""); 
 
 #ifdef HAVE_LIBWRAP
 
-if (!hosts_ctl("cfd",fqname,inet_ntoa(saddr.sin_addr),STRING_UNKNOWN))
+if (!hosts_ctl("cfd",name1,name2,STRING_UNKNOWN))
    {
    Verbose("TCPwrappers rejects this packet.");
    return false;
@@ -981,13 +987,14 @@ if (!hosts_ctl("cfd",fqname,inet_ntoa(saddr.sin_addr),STRING_UNKNOWN))
 
 /* Do our own checking ... */
 
-if (strcmp(ipstring,inet_ntoa(saddr.sin_addr)) != 0)
+if (strcmp(name3,name2) != 0)
    {
-   Debug("Host identification failure - attempt at DNS spoofing? - got %s != %s\n",inet_ntoa(saddr.sin_addr),ipstring);
+   sprintf(OUTPUT,"Host identification failure - attempt at DNS spoofing? - got %s != %s\n",name2,name3);
+   CfLog(cfinform,OUTPUT,"");
    return false;
    }
   
-if ((hp = gethostbyname(fqname)) == NULL)
+if ((hp = gethostbyname(name1)) == NULL)
    {
    Verbose("cfd: Couldn't look up name %s\n",fqname);
    Verbose("     Make sure that fully qualified names can be looked up at your site!\n");
@@ -998,18 +1005,19 @@ if ((hp = gethostbyname(fqname)) == NULL)
 
 raddr.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
 
-if (strcmp(inet_ntoa(saddr.sin_addr),inet_ntoa(raddr.sin_addr)) != 0)
+if (strcmp(name2,ToLowerStr(inet_ntoa(raddr.sin_addr))) != 0)
    {
-   Debug("Reverse hostname lookup failed, host claiming to be %s was %s\n",buf,inet_ntoa(raddr.sin_addr));
+   sprintf(OUTPUT,"Reverse hostname lookup failed, host claiming to be %s was %s\n",buf,inet_ntoa(raddr.sin_addr));
+   CfLog(cfinform,OUTPUT,"");
    return false;
    }
 
-conn->hostname = (char *) malloc(strlen(fqname)+1);
+conn->hostname = strdup(name1);
 
 if (conn->hostname != NULL)
    {
-   Debug("Host ID verified as %s\n",fqname);
-   strcpy(conn->hostname,fqname);
+   Debug("Host ID verified as %s\n",name1);
+   strcpy(conn->hostname,name1);
    return true;   
    }
 
@@ -1025,11 +1033,11 @@ char *sendbuffer;
 
 {
 sprintf(sendbuffer,"BAD: Host authentication failed - did you set the correct domainname?\n");
-Verbose("BAD: Host authentication failed - did you set the correct domainname?\n");
+CfLog(cfinform,"Host authentication failed\n","");
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    }
 }
 
@@ -1048,7 +1056,8 @@ bzero(realname,bufsize);
 #ifdef HAVE_REALPATH
 if (realpath(filename,realname) == NULL)
    {
-   Verbose("cfd: couldn't resolve filename %s from host %s\n",filename,hostname);
+   sprintf(OUTPUT,"Couldn't resolve filename %s from host %s\n",filename,hostname);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 #else
@@ -1086,11 +1095,13 @@ for (ap = VDENY; ap != NULL; ap=ap->next)
 
 if (access)
    {
-   Verbose("Host %s granted access to %s\n",hostname,realname);
+   sprintf(OUTPUT,"Host %s granted access to %s\n",hostname,realname);
+   CfLog(cfverbose,OUTPUT,"");
    }
 else
    {
-   Verbose("Host %s denied access to %s\n",hostname,realname);
+   sprintf(OUTPUT,"Host %s denied access to %s\n",hostname,realname);
+   CfLog(cfinform,OUTPUT,"");
    }
 
 return access;
@@ -1098,41 +1109,76 @@ return access;
 
 /**************************************************************/
 
-SpawnGetFile(args)
+SpawnCfGetFile(args)
 
 struct cfd_thread_arg *args;
 
-{ int pid = -1, i;
+{ int tid, count = 0;
+  void *CfGetFile();
+  
+#if defined HAVE_LIBPTHREAD && defined NOTBROKEN
+/*  
+  pthread_mutex_t mutex;
 
-/* semaphores ! Probably unnecessary in practice ... but still */
-
-/* 
-*SHM_DAEMON_COUNT++;
-
-if (*SHM_DAEMON_COUNT < CFD_MAXPROCESSES)
+if (pthread_mutex_lock(&mutex) != 0)
    {
-   if ((pid = fork()) != 0)
+   CfLog(cferror,"pthread_mutex_lock failed","pthread_mutex_lock");
+   CfGetFile(args);
+   return;
+   }
+
+for (tid = 0; tid < MAXTHREADS; tid++)
+   {
+   if (THREADS[tid] != NULL)
       {
-      Verbose("cfd: Spawning child pid %d to handle %s, current total = %d\n",pid,args->replyfile,*SHM_DAEMON_COUNT);
-      return;
+      count++;
       }
    }
-else
+ 
+if ((count >= CFD_MAXPROCESSES) || (count >= MAXTHREADS))
    {
-   Verbose("cfd: Reached maximum limit on number of concurrent daemons\n");
+   CfLog(cfinform,"Too many threads","");
+   if (pthread_mutex_unlock(&mutex) != 0)
+      {
+      CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+      }
+   return;
    }
 
-   */
 
-GetFile(args);
-
-if (pid == 0) /* Child thread */
+for (tid = 0; tid < MAXTHREADS; tid++)
    {
-   *SHM_DAEMON_COUNT--;
+   if (THREADS[tid] == NULL)
+      {
+      break;
+      }
+   }
+
+Note the threads share the stack, so args shouldn't be local vars which can deallocate 
+ 
+if (pthread_create(&(THREADS[tid]),NULL,CfGetFile,args) != 0)
+   {
+   CfLog(cferror,"pthread_create failed","create");
+   CfGetFile(args);
+   return;
+   }
+
+if (pthread_mutex_unlock(&mutex) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+   }
+
+Careful! This will close the file descriptor before it cam be used...
    
-   Verbose("cfd: done with child %d\n",getpid());
-   exit(0);
-   }
+ */
+ 
+CfGetFile(args);
+ 
+#else
+
+CfGetFile(args);
+
+#endif 
 }
 
 /**************************************************************/
@@ -1150,14 +1196,27 @@ char *sendbuffer, *filename;
   struct stat statbuf;
   char linkbuf[bufsize];
 
-if (lstat(filename,&statbuf) == -1)
+if (strlen(filename) > maxlinksize)
    {
-   perror("lstat");
-   sprintf(sendbuffer,"BAD: unable to stat file\n");
+   sprintf(sendbuffer,"BAD: Filename suspiciously long\n");
+   CfLog(cferror,sendbuffer,"");
 
    if (send(sd,sendbuffer,bufsize,0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
+      }
+
+   return -1;
+   }
+
+if (lstat(filename,&statbuf) == -1)
+   {
+   sprintf(sendbuffer,"BAD: unable to stat file %s\n",filename);
+   CfLog(cferror,sendbuffer,"lstat");
+
+   if (send(sd,sendbuffer,bufsize,0) == -1)
+      {
+      CfLog(cferror,"","send");
       }
 
    return -1;
@@ -1177,12 +1236,12 @@ if (S_ISLNK(statbuf.st_mode))
        
    if (readlink(filename,linkbuf,bufsize) == -1)
       {
-      perror("readlink");
       sprintf(sendbuffer,"BAD: unable to read link\n");
-
+      CfLog(cferror,sendbuffer,"readlink");
+      
       if (send(sd,sendbuffer,bufsize,0) == -1)
 	 {
-	 perror("send");
+	 CfLog(cferror,"","send");
 	 }
       
       return -1;
@@ -1195,13 +1254,12 @@ if (S_ISLNK(statbuf.st_mode))
 
 if (stat(filename,&statbuf) == -1)
    {
-   perror("stat");
-   printf("Can't stat %s\n",filename);
-   sprintf(sendbuffer,"BAD: unable to stat file\n");
+   sprintf(sendbuffer,"BAD: unable to stat file %s\n",filename);
+   CfLog(cferror,OUTPUT,"stat");
 
    if (send(sd,sendbuffer,bufsize,0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
       }
          
    return -1;
@@ -1285,7 +1343,7 @@ sprintf(sendbuffer,"OK: %d %d %d %d %d %d %d %d %d %d %d %d",
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    exit(1);
    }
 
@@ -1304,7 +1362,7 @@ else
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    exit(1);
    }
 
@@ -1319,7 +1377,7 @@ CompareLocalChecksum(conn,sendbuffer,recvbuffer)
 struct cfd_connection *conn;
 char *sendbuffer, *recvbuffer;
 
-{ unsigned char digest1[16],digest2[17], num[5],filename[bufsize];
+{ unsigned char digest1[17],digest2[17], num[5],filename[bufsize];
   char *sp;
   int i;
 
@@ -1359,7 +1417,7 @@ for (i = 0; i < 16; i++)
 
       if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
          {
-         perror("send");
+         CfLog(cferror,"","send");
          }
       
       return;
@@ -1372,7 +1430,7 @@ Debug("Checksums matched ok\n");
 
 if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    return;
    }
 }
@@ -1402,7 +1460,7 @@ if ((dirh = opendir(dirname)) == NULL)
    sprintf(sendbuffer,"BAD: cfengine, couldn't open dir %s\n",dirname);
    if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
       {
-      perror("send");
+      CfLog(cferror,"","send");
       exit(1);
       }
    return -1;
@@ -1420,7 +1478,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
       if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
          {
-         perror("send");
+         CfLog(cferror,"","send");
          exit(1);
 	 }
 
@@ -1436,7 +1494,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
 if (send(conn->sd_reply,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    exit(1);
    }
 
@@ -1458,7 +1516,7 @@ strcpy(buffer,CFD_TERMINATOR);
 
 if (send(sd,buffer,strlen(buffer)+1,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    Verbose("Unable to reply with terminator...\n");
    }
 }
@@ -1485,7 +1543,7 @@ if (ap != NULL)
 /* Level 4                                                     */
 /***************************************************************/
 
-GetFile(args)
+void *CfGetFile(args)
 
 struct cfd_thread_arg *args;
 
@@ -1496,14 +1554,13 @@ sd         = args->sd_reply;
 sendbuffer = args->replybuff;
 filename   = args->replyfile;
 
-Debug("GetFile(%s on sd=%d)\n",filename,sd);
+Debug("CfGetFile(%s on sd=%d)\n",filename,sd);
 
 if ((fd = open(filename,O_RDONLY)) == -1)
    {
-   printf("cfd: can't copy %s!\n",filename);
    sprintf(sendbuffer,"BAD: can't access %s on server!\n",filename);
-   perror("open");
-   return false;
+   CfLog(cferror,sendbuffer,"open");
+   return NULL;
    }
 
 while(true)
@@ -1513,9 +1570,9 @@ while(true)
    if ((n_read = read(fd,sendbuffer,bufsize)) == -1)
       {
       close(fd);
-      perror("read");
-      sprintf(sendbuffer,"BAD: GetFile, read failed\n");
-      return false;
+      sprintf(sendbuffer,"BAD: CfGetFile, read failed\n");
+      CfLog(cferror,sendbuffer,"read");
+      return NULL;
       }
 
    if (n_read == 0)
@@ -1526,13 +1583,13 @@ while(true)
    if (send(sd,sendbuffer,n_read,0) == -1)
       {
       close(fd);
-      perror("send");
-      return false;
+      CfLog(cferror,"","send");
+      return NULL;
       }
    }
 
 close(fd);
-return true;
+return NULL;
 }
 
 /***************************************************************/
@@ -1547,7 +1604,7 @@ sprintf(buffer,"Hello %s, nothing to be done on %s\n",conn->hostname,VFQNAME);
 
 if (send(conn->sd_reply,buffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    }
 }
 
@@ -1556,7 +1613,7 @@ if (send(conn->sd_reply,buffer,bufsize,0) == -1)
 InChecksumDB(filename,digest)
 
 char *filename;
-char digest[16];
+char digest[17];
 
 {     
 #if defined HAVE_LIBDB && (defined HAVE_DB_H || defined HAVE_DB_DB_H)
@@ -1590,7 +1647,7 @@ if (CHECKSUMDB[0] == '\0')
 if ((db = dbopen(CHECKSUMDB, O_RDWR | O_CREAT, 0644, DB_BTREE,NULL)) == NULL)
    {
    printf("cfd: couldn't open checksum database %s\n",CHECKSUMDB);
-   perror("db_open");
+   CfLog(cferror,"","db_open");
    return false;
    }
 
@@ -1620,7 +1677,7 @@ if ((result = db->get(db,&key,&value,0)) == 0)
       
       if (db->del(db,&key,0) != 0)
 	 {
-	 perror("db_store");
+	 CfLog(cferror,"","db_store");
 	 }
 
       key.data = filename;
@@ -1629,7 +1686,7 @@ if ((result = db->get(db,&key,&value,0)) == 0)
       if ((result = db->put(db,&key,&value,0)) != 0)
 	 {
 	 Debug("retrun val = %d\n",result);
-	 perror("db_store");
+	 CfLog(cferror,"","db_store");
 	 }
 
       db->close(db);
@@ -1639,7 +1696,7 @@ if ((result = db->get(db,&key,&value,0)) == 0)
 else
    {
    Debug("result= %d\n",result);
-   perror("get");
+   CfLog(cferror,"","get");
    cfMDFile(filename,digest);
    digest[16] = '\0';
 
@@ -1653,7 +1710,7 @@ else
    if ((result = db->put(db,&key,&value,0)) != 0)
       {
       printf("return value = %d\n",result);
-      perror("db_store");
+      CfLog(cferror,"","db_store");
       }
    
    db->close(db);
