@@ -81,8 +81,8 @@ struct cfd_thread_arg
 /*******************************************************************/
 
 int CLOCK_DRIFT = 100;
-
-int *SHM_DAEMON_COUNT; /* Shared memory pointer */
+int CFD_MAXPROCESSES = 0;
+int NO_FORK = false;
 
 struct option CFDOPTIONS[] =
    {
@@ -102,9 +102,22 @@ char CURRENTPATH[bufsize];
 char CFRUNCOMMAND[bufsize];
 char CHECKSUMDB[bufsize];
 
-int CFD_MAXPROCESSES = 0;
-int NO_FORK = false;
-
 #ifdef RE_DUP_MAX
 # undef RE_DUP_MAX
 #endif
+
+/*******************************************************************
+ * Sunos4.1.4 need these prototypes
+ *******************************************************************/
+
+#if defined(SUN4)
+extern char *realpath(/* char *path; char resolved_path[MAXPATHLEN] */);
+#endif
+
+/* Need this to to avoid conflict with solaris 2.6 and db.h */
+
+#ifdef SOLARIS
+# define u_int32_t uint32_t
+# define u_int16_t uint16_t
+# define u_int8_t uint8_t
+#endif
