@@ -58,7 +58,7 @@ CheckOptsAndInit(argc,argv);
 
 for (ip = VCFRUNHOSTLIST; ip != NULL; ip=ip->next)
    {
-   printf("cfrun: talking to %s\n",ip->name);
+   printf("cfrun: negotiating with %s\n",ip->name);
    ConnectToServer(ip->name,ip->classes);
    }
 }
@@ -103,7 +103,7 @@ while (!feof(fp))
    bzero(options,bufsize);
    bzero(line,bufsize);
 
-   fgets(line,bufsize,fp);
+   ReadLine(line,bufsize,fp);
 
    if (strncmp(line,"domain",6) == 0)
       {
@@ -196,11 +196,11 @@ if (uname(&VSYSNAME) == -1)
 
 if ((strlen(VDOMAIN) > 0) && !strchr (VSYSNAME.nodename, '.'))
    {
-   sprintf(VFQNAME,"%s.%s",VSYSNAME.nodename,VDOMAIN);
+   sprintf(VFQNAME,"%s.%s\n",VSYSNAME.nodename,VDOMAIN);
    }
 else
    {
-   sprintf(VFQNAME,"%s",VSYSNAME.nodename);
+   sprintf(VFQNAME,"%s\n",VSYSNAME.nodename);
    }
 }
 
@@ -255,7 +255,7 @@ alarm(CF_TIMEOUT);
 
 if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
    {
-   printf("Host %s didn't answer poll\n",host);
+   printf("Host %s isn't talking to anyone\n",host);
    return false;
    }
 
@@ -319,6 +319,11 @@ while (true)
       continue;
       }
 
+   if (strstr(recvbuffer,"too soon"))
+      {
+      printf("%s",recvbuffer);
+      }
+
    if (strstr(recvbuffer,"cfd:")) /* Don't print messages fom cfd unless -v */
       {
       Verbose("%s",recvbuffer);
