@@ -64,6 +64,7 @@ char *CLASSTEXT[] =   /* If you change here change enum classes too! */
    "nextstep",
    "cray",
    "gnu",
+   "nt",
    "unused1",
    "unused2",
    "unused3",
@@ -74,8 +75,8 @@ char *CLASSTEXT[] =   /* If you change here change enum classes too! */
 
   /* remember to change cf.defs.h !!  */
 
-char *CLASSATTRIBUTES[clssattr][attr] = /* Used to match the softclass */ 
-   {                            /* from uname. Only as many characters */
+char *CLASSATTRIBUTES[clssattr][attrdim] =
+   {                            /* Only as many characters */
    "-","-","-",                 /* as appear here are matched. The     */
    "-","-","-",                 /* fields are sysname and machine */
    "sunos","*","4*",           /* sun 4  */
@@ -100,6 +101,7 @@ char *CLASSATTRIBUTES[clssattr][attr] = /* Used to match the softclass */
    "nextstep","*","*",         /* nextstep */
    "sn*","cray*","*",          /* cray */
    "gnu*","*","*",             /* gnu */
+   "Windows_NT","*","*",       /* NT */
    "unused1","blah","blah",
    "unused2","blah","blah",
    "unused3","blah","blah",
@@ -134,6 +136,7 @@ char *VPSCOMM[clssattr] =
    "/bin/ps",       /* nextstep */
    "/bin/ps",       /* cray */
    "/bin/ps",       /* gnu */
+   "/C/usr/bin/ps", /* NT */
    "/bin/ps",
    "/bin/ps",
    "/bin/ps",
@@ -168,6 +171,7 @@ char *VPSOPTS[clssattr] =
    "-ax",    /* nextstep */
    "-ef",    /* cray */
    "-ax",    /* gnu */
+   "-a",     /* NT */
    "-",
    "-",
    "-",
@@ -180,28 +184,29 @@ char *VMOUNTCOMM[clssattr] =
    {
    "",                                              /* see cf.defs.h */
    "",
-   "/etc/mount -va 2<&1",      /* sun4 */
-   "/etc/mount -va 2<&1",      /* ultrix */
-   "/sbin/mount -va 2<&1",     /* hpux10 */
-   "/etc/mount -va 2<&1",      /* hpux */
-   "/etc/mount -t nfs 2<&1",   /* aix */
-   "/bin/mount -va 2<&1",      /* linux */
-   "/usr/sbin/mount -a 2>&1",  /* solaris */
-   "/usr/sbin/mount -va 2<&1", /* osf1 */
-   "/etc/mount -va 2<&1",      /* sun3 */
-   "/sbin/mount -va 2<&1",     /* irix4 */
-   "/sbin/mount -va 2<&1",     /* irix */
-   "/sbin/mount -va 2<&1",     /* irix64 */
-   "/sbin/mount -va 2<&1" ,    /* freebsd */
-   "/usr/sbin/mount -a 2>&1",  /* solarisx86 */
-   "/etc/mount -a 2>&1",       /* bsd 4.3 */
-   "/etc/mount -a 2>&1",       /* newsos4 */
-   "/sbin/mount -a 2>&1",      /* netbsd */
-   "/etc/mount -a 2>&1",       /* AOS */
-   "/sbin/mount -a 2>&1",      /* BSDI */
-   "/usr/etc/mount -a 2>&1",   /* nextstep */
-   "/etc/mount -va  2>&1",     /* cray */
-   "/bin/mount -va 2<&1",      /* gnu */
+   "/etc/mount -va",      /* sun4 */
+   "/etc/mount -va",      /* ultrix */
+   "/sbin/mount -va",     /* hpux10 */
+   "/etc/mount -va",      /* hpux */
+   "/etc/mount -t nfs",   /* aix */
+   "/bin/mount -va",      /* linux */
+   "/usr/sbin/mount -a",  /* solaris */
+   "/usr/sbin/mount -va", /* osf1 */
+   "/etc/mount -va",      /* sun3 */
+   "/sbin/mount -va",     /* irix4 */
+   "/sbin/mount -va",     /* irix */
+   "/sbin/mount -va",     /* irix64 */
+   "/sbin/mount -va" ,    /* freebsd */
+   "/usr/sbin/mount -a",  /* solarisx86 */
+   "/etc/mount -a",       /* bsd 4.3 */
+   "/etc/mount -a",       /* newsos4 */
+   "/sbin/mount -a",      /* netbsd */
+   "/etc/mount -a",       /* AOS */
+   "/sbin/mount -a",      /* BSDI */
+   "/usr/etc/mount -a",   /* nextstep */
+   "/etc/mount -va",      /* cray */
+   "/bin/mount -va",      /* gnu */
+   "N/A",                 /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -236,6 +241,7 @@ char *VUNMOUNTCOMM[clssattr] =
    "/usr/etc/umount",  /* nextstep */
    "/etc/umount",      /* cray */
    "/sbin/umount",     /* gnu */
+   "N/A",              /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -272,6 +278,7 @@ char *VMOUNTOPTS[clssattr] =
    "bg,hard,intr",    /* nextstep */
    "bg,hard,intr",    /* cray */
    "defaults",        /* gnu */
+   "N/A",             /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -305,7 +312,8 @@ char *VIFDEV[clssattr] =
    "le0",    /* BSDI */
    "en0",    /* nextstep -default */
    "en0",    /* cray */
-   "eth0",   /* gnu */ 
+   "eth0",   /* gnu */
+   "N/A",    /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -341,6 +349,7 @@ char *VRESOLVCONF[clssattr] =
    "/etc/resolv.conf",     /* nextstep */
    "/etc/resolv.conf",     /* cray */
    "/etc/resolv.conf",     /* gnu */
+   "/C/resolv.conf",       /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -377,6 +386,7 @@ char *VFSTAB[clssattr] =
    "/etc/fstab",       /* nextstep */
    "/etc/fstab",       /* cray */
    "/etc/fstab",       /* gnu */
+   "N/A",              /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -411,6 +421,7 @@ char *VMAILDIR[clssattr] =
    "/usr/spool/mail",    /* nextstep */
    "/usr/mail",          /* cray */
    "/var/spool/mail",    /* gnu */
+   "N/A",                /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
@@ -445,6 +456,7 @@ char *VNETSTAT[clssattr] =
    "/usr/ucb/netstat -rn",   /* nextstep */
    "/usr/ucb/netstat -rn",   /* cray */
    "/bin/netstat -rn",       /* gnu */
+   "/C/WINNT/System32/netstat" /* NT */
    "unused-blah",
    "unused-blah",
    "unused-blah",
