@@ -102,14 +102,14 @@ if (ret != 0)
 bzero(&cin,sizeof(cin));
 
 cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = ((struct in_addr *)((ip->dns)->h_addr))->s_addr;
+cin.sin_addr.s_addr = (ip->dns)->s_addr;
 cin.sin_family = AF_INET; 
 
 Debug("Trying to connect to %s = %s, port h=%d\n",ip->server,inet_ntoa(cin.sin_addr),PORTNUMBER);
 
 if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
    {
-   perror("socket");
+   CfLog(cferror,"","socket");
    exit(1);
    }
 
@@ -118,11 +118,8 @@ alarm(CF_TIMEOUT);
 
 if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
    {
-   if (VERBOSE || DEBUG || D2)
-      {
-      printf("%s: Couldn't connect to host %s\n",VPREFIX,ip->server);
-      perror("connect");
-      }
+   sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
+   CfLog(cfinform,OUTPUT,"connect");
    close(sd);
    return -1;
    }
@@ -134,7 +131,7 @@ Debug("Sending on socket...\n");
 
 if (! IdentifyForVerification(sd,VIPADDRESS,VFQNAME))
    {
-   printf("%s: server registration procedure failed",VPREFIX);
+   CfLog(cferror,"Server registration procedure failed","");
    errno = EPERM;
    close(sd);
    return -1;
@@ -142,14 +139,14 @@ if (! IdentifyForVerification(sd,VIPADDRESS,VFQNAME))
 
 if ((tloc = time((time_t *)NULL)) == -1)
    {
-   printf("Couldn't read system clock\n");
+   CfLog(cferror,"Couldn't read system clock\n","");
    }
 
 sprintf(sendbuffer,"SYNCH %d STAT %s",tloc,file);
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    close(sd);
    return -1;
    }
@@ -210,8 +207,7 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
 
    if (strlen(recvbuffer) > 3)
       {
-      cfst.cf_readlink = malloc(strlen(recvbuffer));
-      strcpy(cfst.cf_readlink,recvbuffer+3);
+      cfst.cf_readlink = strdup(recvbuffer+3);
       }
    else
       {
@@ -235,12 +231,14 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
       }
 
 
-   cfst.cf_filename = malloc(strlen(file)+1);
-   strcpy(cfst.cf_filename,file);
-
-   cfst.cf_server = malloc(strlen(ip->server)+1);
-   strcpy(cfst.cf_server,ip->server);
+   cfst.cf_filename = strdup(file);
+   cfst.cf_server =  strdup(ip->server);
 
+   if ((cfst.cf_filename == NULL) ||(cfst.cf_server) == NULL)
+      {
+      FatalError("Memory allocation in cf_rstat");
+      }
+   
    cfst.cf_failed = false;
 
    if (cfst.cf_lmode != 0)
@@ -275,7 +273,7 @@ if (strncmp(recvbuffer,"OK:",3) == 0)
    }
 
 
-Verbose("cfengine: server didn't respond correctly (Got: %s)\n",recvbuffer);
+CfLog(cfinform,"Server didn't respond correctly. Got:",recvbuffer);
 errno = EPERM;
 
 close(sd);
@@ -301,12 +299,12 @@ Debug("CfOpenDir(%s:%s)\n",ip->server,dirname);
 bzero(&cin,sizeof(cin));
 
 cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = ((struct in_addr *)((ip->dns)->h_addr))->s_addr;
+cin.sin_addr.s_addr = (ip->dns)->s_addr;
 cin.sin_family = AF_INET; 
 
 if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
    {
-   perror("socket");
+   CfLog(cferror,"","socket");
    exit(1);
    }
 
@@ -315,11 +313,8 @@ alarm(CF_TIMEOUT);
 
 if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
    {
-   if (VERBOSE || DEBUG || D2)
-      {
-      printf("%s: Couldn't connect to host %s\n",VPREFIX,ip->server);
-      perror("connect");
-      }
+   sprintf(OUTPUT,"Couldn't connect to host %s\n",ip->server);
+   CfLog(cfinform,OUTPUT,"connect");
    close(sd);
    return NULL;
    }
@@ -327,11 +322,9 @@ if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
 alarm(0);
 signal(SIGALRM,SIG_DFL);
 
-Debug("Sending on socket...\n");
-
 if (! IdentifyForVerification(sd,VIPADDRESS,VFQNAME))
    {
-   printf("%s: server registration procedure failed",VPREFIX);
+   CfLog(cfinform,"Server registration procedure failed\n","");
    errno = EPERM;
    close(sd);
    return NULL;
@@ -339,7 +332,7 @@ if (! IdentifyForVerification(sd,VIPADDRESS,VFQNAME))
 
 if ((cfdirh = (CFDIR *)malloc(sizeof(CFDIR))) == NULL)
    {
-   printf("%s: couldn't allocate memory in cf_ropendir\n",VPREFIX);
+   CfLog(cferror,"Couldn't allocate memory in cf_ropendir\n","");
    exit(1);
    }
 
@@ -351,7 +344,7 @@ sprintf(sendbuffer,"OPENDIR %s",dirname);
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    close(sd);
    return NULL;
    }
@@ -421,12 +414,12 @@ struct Image *ip;
 bzero(&cin,sizeof(cin));
 
 cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = ((struct in_addr *)((ip->dns)->h_addr))->s_addr;
+cin.sin_addr.s_addr = (ip->dns)->s_addr;
 cin.sin_family = AF_INET; 
 
 if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
    {
-   perror("socket");
+   CfLog(cferror,"","socket");
    Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
    return false;
    }
@@ -436,7 +429,7 @@ alarm(CF_TIMEOUT);
 
 if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
    {
-   Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
+   CfLog(cfinform,"No answer from host, assuming checksum ok to avoid remote copy for now...\n","");
    close(sd);
    return false;
    }
@@ -444,12 +437,9 @@ if (err=connect(sd,(void *) &cin,sizeof(cin)) == -1)
 alarm(0);
 signal(SIGALRM,SIG_DFL);
 
-Debug("Sending on socket...\n");
-
 if (! IdentifyForVerification(sd,VIPADDRESS,VFQNAME))
    {
-   Verbose("%s: server registration procedure failed\n",VPREFIX);
-   Verbose("%s: assuming checksum ok to avoid remote copy for now...\n",VPREFIX);
+   CfLog(cfinform,"Server registration procedure failed\n","");
    errno = EPERM;
    close(sd);
    return true;
@@ -462,7 +452,7 @@ bzero(sendbuffer,bufsize);
 
 if (20 + 3*16 + strlen(file1) > bufsize-1)
    {
-   Verbose("%s: MD5 transfer overflow...updating for safety\n",VPREFIX);
+   CfLog(cferror,"MD5 transfer overflow...updating for safety\n","");
    close(sd);
    return true;
    }
@@ -472,9 +462,7 @@ sprintf(sendbuffer,"MD5 %s %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d \0",f
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
-   close(sd);
-   Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
+   CfLog(cferror,"","send");
    close(sd);
    return false;
    }
@@ -483,7 +471,7 @@ if (RecvSocketStream(sd, recvbuffer, bufsize,0) == -1)
    {
    close(sd);
    Verbose("No answer from host, assuming checksum ok to avoid remote copy for now...\n");
-   return false;  /* Do this to avoid starting a copy if remote host is down */
+   return false;
    }
 
 if (strcmp(CFD_TRUE,recvbuffer) == 0)
@@ -523,20 +511,20 @@ Debug("CopyRegNet(%s,%s)\n",source,new);
 bzero(&cin,sizeof(cin));
 
 cin.sin_port = PORTNUMBER;
-cin.sin_addr.s_addr = ((struct in_addr *)((ip->dns)->h_addr))->s_addr;
+cin.sin_addr.s_addr = (ip->dns)->s_addr;
 cin.sin_family = AF_INET; 
 
 if ((dd = open(new,O_WRONLY | O_CREAT | O_TRUNC, 0600)) == -1)
    {
-   printf("%s: can't copy to destination %s!\n",VPREFIX,new);
-   perror("open");
+   sprintf(OUTPUT,"Can't copy to destination %s!\n",new);
+   CfLog(cferror,OUTPUT,"open");
    unlink(new);
    return false;
    }
 
 if ((sd = socket(AF_INET,SOCK_STREAM,0)) == -1)
    {
-   perror("socket");
+   CfLog(cferror,"","socket");
    exit(1);
    }
 
@@ -545,9 +533,10 @@ alarm(CF_TIMEOUT);
 
 if (err = connect(sd,(void *) &cin,sizeof(cin)) == -1)
    {
-   printf("%s: Couldn't connect to host %s\n",VPREFIX,ip->server);
-   perror("connect");
+   sprintf("Couldn't connect to host %s\n",ip->server);
+   CfLog(cferror,OUTPUT,"connect");
    close(sd);
+   close(dd);
    return false;
    }
 
@@ -556,8 +545,9 @@ signal(SIGALRM,SIG_DFL);
 
 if (! IdentifyForVerification(sd,VIPADDRESS,VFQNAME))
    {
-   printf("%s: server registration procedure failed",VPREFIX);
+   CfLog("Server registration procedure failed\n","");
    close(sd);
+   close(dd);
    return false;
    }
 
@@ -565,8 +555,9 @@ sprintf(sendbuffer,"GET %s",source);
 
 if (send(sd,sendbuffer,bufsize,0) == -1)
    {
-   perror("send");
+   CfLog(cferror,"","send");
    close(sd);
+   close(dd);
    return false;
    }
 
@@ -590,7 +581,8 @@ while (true)
 
    if (strncmp(buf,"BAD: Host authentication failed",strlen("BAD: Host authentication failed")) == 0)
       {
-      printf("%s: network access to %s:%s denied\n",VPREFIX,ip->server,source);
+      sprintf(OUTPUT,"Network access to %s:%s denied\n",ip->server,source);
+      CfLog(cfinform,OUTPUT,"");
       close(sd);
       close(dd);
       free(buf);
@@ -634,11 +626,12 @@ while (true)
          /* Make a hole.  */
          if (lseek (dd, (off_t) n_read, SEEK_CUR) < 0L)
             {
-            printf ("cfengine: lseek in CopyReg, dest=%s\n", new);
-            perror("lseek");
+            sprintf (OUTPUT,"lseek in CopyReg, dest=%s\n", new);
+            CfLog(cferror,OUTPUT,"lseek");
             free(buf);
 	    unlink(new);
             close(sd);
+	    close(dd);
             return false;
             }
          last_write_made_hole = 1;
@@ -654,7 +647,7 @@ while (true)
       {
       if (cf_full_write (dd, buf, n_read) < 0)
          {
-         printf ("cfengine: full_write failed in CopyReg()\n");
+         CfLog(cferror,"cfengine: full_write failed in CopyReg()\n","");
          close(sd);
          close(dd);
          free(buf);
@@ -675,10 +668,11 @@ while (true)
 
     if (cf_full_write (dd, "", 1) < 0 || ftruncate (dd, n_read_total) < 0)
        {
-       printf("cfengine: full_write or ftruncate error in CopyReg\n");
+       CfLog(cferror,"cfengine: full_write or ftruncate error in CopyReg\n","");
        free(buf);
        unlink(new);
        close(sd);
+       close(dd);
        return false;
        }
     }
@@ -751,7 +745,7 @@ struct Image *ip;
 
 if ((sp = (struct cfstat *) malloc(sizeof(struct cfstat))) == NULL)
    {
-   printf("cfengine: Memory allocation faliure in CacheData()\n");
+   CfLog(cferror,"Memory allocation faliure in CacheData()\n","");
    return;
    }
 
