@@ -75,16 +75,16 @@ struct Image *ip;
   
 if ((sd = open(source,O_RDONLY)) == -1)
    {
-   printf("%s: can't copy %s!\n",VPREFIX,source);
-   perror("open");
+   sprintf(OUTPUT,"Can't copy %s!\n",source);
+   CfLog(cfinform,OUTPUT,"open");
    unlink(new);
    return false;
    }
 
 if ((dd = open(new,O_WRONLY | O_CREAT | O_TRUNC, 0600)) == -1)
    {
-   printf("%s: can't copy to destination %s!\n",VPREFIX,new);
-   perror("open");
+   sprintf(OUTPUT,"Can't copy to destination %s!\n",new);
+   CfLog(cfinform,OUTPUT,"open");
    close(sd);
    unlink(new);
    return false;
@@ -145,8 +145,8 @@ while (true)
          /* Make a hole.  */
          if (lseek (dd, (off_t) n_read, SEEK_CUR) < 0L)
             {
-            printf ("cfengine: lseek in CopyReg, dest=%s\n", new);
-            perror("lseek");
+            sprintf (OUTPUT,"cfengine: lseek in CopyReg, dest=%s\n", new);
+            CfLog(cferror,OUTPUT,"lseek");
             free(buf);
 	    unlink(new);
 	    close(dd);
@@ -166,7 +166,7 @@ while (true)
       {
       if (cf_full_write (dd, buf, n_read) < 0)
          {
-         printf ("cfengine: full_write failed in CopyReg()\n");
+         CfLog(cferror,"cfengine: full_write failed in CopyReg()\n","");
          close(sd);
          close(dd);
          free(buf);
@@ -187,7 +187,7 @@ while (true)
 
     if (cf_full_write (dd, "", 1) < 0 || ftruncate (dd, n_read_total) < 0)
        {
-       printf("cfengine: full_write or ftruncate error in CopyReg\n");
+       CfLog(cferror,"cfengine: full_write or ftruncate error in CopyReg\n","");
        free(buf);
        unlink(new);
        close(sd);
