@@ -48,6 +48,8 @@
 #include <sys/mount.h>
 #endif
 
+/************************************************************************/
+
 GetDiskUsage (file)
 
 char *file;
@@ -68,25 +70,29 @@ char *file;
 #if defined ULTRIX
     if (getmnt (NULL, &buf, sizeof (struct fs_data), STAT_ONE, file) == -1)
        {
-       printf("%s: Couldn't get filesystem info for %s\n",VPREFIX,file);
+       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #elif defined SOLARIS || defined OSF1
     if (statvfs (file, &buf) != 0)
        {
-       printf("%s: Couldn't get filesystem info for %s\n",VPREFIX,file);
+       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #elif defined IRIX || defined CFCRAY
     if (statfs (file, &buf, sizeof (struct statfs), 0) != 0)
        {
-       printf("%s: Couldn't get filesystem info for %s\n",VPREFIX,file);
+       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #else
     if (statfs (file, &buf) != 0)
        {
-       printf("%s: Couldn't get filesystem info for %s\n",VPREFIX,file);
+       sprintf(OUTPUT,"Couldn't get filesystem info for %s\n",file);
+       CfLog(cferror,OUTPUT,"");
        return cfinfinity;
        }
 #endif
@@ -110,9 +116,9 @@ char *file;
 #endif
 
 #if defined OSF1
-    total = buf.f_blocks / (blocksize / buf.f_frsize);
-    used = (buf.f_blocks - buf.f_bfree) / (blocksize / buf.f_frsize);
-    avail = buf.f_bavail / (blocksize / buf.f_frsize);
+    total = (buf.f_blocks *  buf.f_frsize) / blocksize;
+    used = ((buf.f_blocks - buf.f_bfree)* buf.f_frsize) / blocksize;
+    avail = (buf.f_bavail * buf.f_frsize) / blocksize;
 #endif
 
 #if defined AIX
@@ -128,9 +134,9 @@ char *file;
 #endif
 
 #if defined IRIX
-    total = buf.f_blocks / (blocksize / buf.f_bsize);
-    used = (buf.f_blocks - buf.f_bfree) / (blocksize / buf.f_bsize);
-    avail = buf.f_bfree / (blocksize / buf.f_bsize);
+    total = (buf.f_blocks * buf.f_bsize) / blocksize;
+    used = (buf.f_blocks - buf.f_bfree)*  buf.f_bsize / blocksize;
+    avail = buf.f_bfree *  buf.f_bsize/ blocksize;
 #endif
 
 #if defined CFCRAY
