@@ -53,7 +53,7 @@ struct Edit *ptr;
   
 if (!MountPathDefined())
    {
-   printf("%s: mountpattern is undefined\n",VPREFIX);
+   CfLog(cfinform,"Mountpattern is undefined\n","");
    return;
    }
 
@@ -66,8 +66,8 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
    
    if ((dirh = opendir(ip->name)) == NULL)
       {
-      printf("%s: Can't open directory %s\n",VPREFIX,ip->name);
-      perror("opendir");
+      sprintf(OUTPUT,"Can't open directory %s\n",ip->name);
+      CfLog(cferror,OUTPUT,"opendir");
       return;
       }
 
@@ -89,8 +89,8 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
 
       if ((dirh2 = opendir(homedir)) == NULL)
          {
-         printf("%s: Can't open directory%s\n",VPREFIX,homedir);
-         perror("opendir");
+         sprintf(OUTPUT,"Can't open directory%s\n",homedir);
+         CfLog(cferror,OUTPUT,"opendir");
          return;
          }
 
@@ -161,7 +161,8 @@ filestart = NULL;
 currenteditscript = NULL;
 searchstr = NULL;
 bzero(ACTIONBUFF,bufsize);
-
+AUTOCREATED = false;
+ 
 for (ep = ptr->actions; ep != NULL; ep=ep->next)
    {
    if (!IsExcluded(ep->classes))
@@ -184,7 +185,7 @@ CheckCreate(filename,ptr->actions);
 
 if (! LoadItemList(&filestart,filename))
    {
-   printf("%s: File was marked for editing\n",VPREFIX);
+   CfLog(cfinform,"File was marked for editing\n","");
    ReleaseCurrentLock();
    return;
    }
@@ -200,7 +201,7 @@ SEARCHREPLACELEVEL = 0;
 FOREACHLEVEL = 0;
 loopstart = NULL;
 
-Verbose("Edit: BEGIN %s\n",filename);
+Verbose("%s: Begin editing %s\n",VPREFIX,filename);
 ep = ptr->actions;
 
 while (ep != NULL)
@@ -454,7 +455,6 @@ while (ep != NULL)
                   EditVerbose("%s: LocateLineMatchingRegexp failed in %s, aborting editing\n",VPREFIX,filename);
                   goto abort;
                   }
-               CURRENTLINEPTR = newlineptr;
                break;
 
       case InsertLine:
@@ -592,7 +592,7 @@ while (ep != NULL)
                break;
 
       case EmptyEntireFilePlease:
-               EditVerbose("Edit: Emptying entire file\n");
+               EditVerbose("Emptying entire file\n");
                DeleteItemList(filestart);
 	       filestart = NULL;
 	       CURRENTLINEPTR = NULL;
@@ -607,7 +607,7 @@ while (ep != NULL)
       case BreakIfLineMatches:
                if (LineMatches(CURRENTLINEPTR->name,ep->data))
                   {
-                  EditVerbose("Edit: Break! %s\n",ep->data);
+                  EditVerbose("Break! %s\n",ep->data);
                   goto abort;
                   }
                break;
@@ -615,13 +615,13 @@ while (ep != NULL)
       case BeginGroupIfNoMatch:
 	       if (CURRENTLINEPTR == NULL || CURRENTLINEPTR->name == NULL )
 		  {
-		  EditVerbose("Edit: (Begin Group - no match for %s - file empty)\n",ep->data);
+		  EditVerbose("(Begin Group - no match for %s - file empty)\n",ep->data);
 		  break;
 		  }
 	       
                if (LineMatches(CURRENTLINEPTR->name,ep->data))
                   {
-                  EditVerbose("Edit: (Begin Group - skipping %s)\n",ep->data);
+                  EditVerbose("(Begin Group - skipping %s)\n",ep->data);
                   while(ep->code != EndGroup)
                      {
                      ep=ep->next;
@@ -629,14 +629,14 @@ while (ep != NULL)
                   }
                else
                   {
-                  EditVerbose("Edit: (Begin Group - no match for %s)\n",ep->data);
+                  EditVerbose("(Begin Group - no match for %s)\n",ep->data);
                   }
                break;
 
      case BeginGroupIfNoLineMatching:
                if (LocateItemMatchingRegExp(filestart,ep->data) != 0)
                   {
-                  EditVerbose("Edit: (Begin Group - skipping)\n");
+                  EditVerbose("(Begin Group - skipping)\n");
                   while(ep->code != EndGroup)
                      {
                      ep=ep->next;
@@ -644,14 +644,14 @@ while (ep != NULL)
                   }
                else
                   {
-                  EditVerbose("Edit: (Begin Group - no line matching %s)\n",ep->data);
+                  EditVerbose("(Begin Group - no line matching %s)\n",ep->data);
                   }
                break;
 
       case BeginGroupIfNoLineContaining:
                if (LocateNextItemContaining(filestart,ep->data) != 0)
                   {
-                  EditVerbose("Edit: (Begin Group - skipping, string matched)\n");
+                  EditVerbose("(Begin Group - skipping, string matched)\n");
                   while(ep->code != EndGroup)
                      {
                      ep=ep->next;
@@ -659,14 +659,14 @@ while (ep != NULL)
                   }
                else
                   {
-                  EditVerbose("Edit: (Begin Group - no line containing %s)\n",ep->data);
+                  EditVerbose("(Begin Group - no line containing %s)\n",ep->data);
                   }
                break;
 
       case BeginGroupIfNoSuchLine:
                if (IsItemIn(filestart,ep->data))
                   {
-                  EditVerbose("Edit: (Begin Group - skipping, line exists)\n");
+                  EditVerbose("(Begin Group - skipping, line exists)\n");
                   while(ep->code != EndGroup)
                      {
                      ep=ep->next;
@@ -674,15 +674,15 @@ while (ep != NULL)
                   }
                else
                   {
-                  EditVerbose("Edit: (Begin Group - no line %s)\n",ep->data);
+                  EditVerbose("(Begin Group - no line %s)\n",ep->data);
                   }
                break;
 
 
      case BeginGroupIfFileIsNewer:
-               if (! FileIsNewer(filename,ep->data))
+               if ((!AUTOCREATED) && (!FileIsNewer(filename,ep->data)))
                   {
-                  EditVerbose("Edit: (Begin Group - skipping, file is older)\n");
+                  EditVerbose("(Begin Group - skipping, file is older)\n");
                   while(ep->code != EndGroup)
                      {
                      ep=ep->next;
@@ -690,14 +690,14 @@ while (ep != NULL)
                   }
                else
                   {
-                  EditVerbose("Edit: (Begin Group - new file %s)\n",ep->data);
+                  EditVerbose("(Begin Group - new file %s)\n",ep->data);
                   }
                break;
 
      case BeginGroupIfFileExists:
                if (stat(ep->data,&tmpstat) == -1)
                   {
-                  EditVerbose("Edit: (Begin Group - file unreadable/no such file - skipping)\n");
+                  EditVerbose("(Begin Group - file unreadable/no such file - skipping)\n");
                   while(ep->code != EndGroup)
                      {
                      ep=ep->next;
@@ -705,11 +705,11 @@ while (ep != NULL)
                   }
                else
                   {
-                  EditVerbose("Edit: (Begin Group - found file %s)\n",ep->data);
+                  EditVerbose("(Begin Group - found file %s)\n",ep->data);
                   }
                break;
       case EndGroup:
-               EditVerbose("Edit: (End Group)\n");
+               EditVerbose("(End Group)\n");
                break;
 
       case ReplaceAll:
@@ -744,7 +744,7 @@ while (ep != NULL)
 
 		  if ((loop_fp = fopen(ep->data,"r")) == NULL)
 		     {
-		     EditVerbose("Edit: Couldn't open %s\n",ep->data);
+		     EditVerbose("Couldn't open %s\n",ep->data);
 		     while(ep->code != EndLoop) /* skip over loop */
 		        {
 		        ep = ep->next;
@@ -752,7 +752,7 @@ while (ep != NULL)
 		     break;
 		     }
 		  
-		  EditVerbose("Edit: Starting ForEach loop with %s\n",ep->data);
+		  EditVerbose("Starting ForEach loop with %s\n",ep->data);
 		  continue;
 		  }
 	       else
@@ -760,12 +760,11 @@ while (ep != NULL)
 	          if (!feof(loop_fp))
 		     {
 		     bzero(ACTIONBUFF,bufsize);
-		     fgets(ACTIONBUFF,bufsize,loop_fp); /* Like SetLine */
-		     Chop(ACTIONBUFF);
+		     ReadLine(ACTIONBUFF,bufsize,loop_fp); /* Like SetLine */
 		     }
 		  else
 		     {
-		     EditVerbose("Edit: EndForeach\n");
+		     EditVerbose("EndForeach\n");
 		     
 		     fclose(loop_fp);
 		     loopstart = NULL;
@@ -791,6 +790,9 @@ while (ep != NULL)
       case AppendToLineIfNotContains:
 	       AppendToLine(CURRENTLINEPTR,ep->data,filename);
 	       break;
+
+      case DefineClasses:
+	       break;
 	       
       default: printf("%s: Unknown action in editing of file %s\n",VPREFIX,filename);
                break;
@@ -801,13 +803,15 @@ while (ep != NULL)
 
 abort :  
 
-EditVerbose("Edit: END %s\n",filename);
+EditVerbose("%s: End editing %s\n",VPREFIX,filename);
+EditVerbose(".....................................................................\n");
 
 EDITVERBOSE = false;
 
 if ((! DONTDO) && (NUMBEROFEDITS > 0))
    {
    SaveItemList(filestart,filename);
+   AddEditfileClasses(ptr);
    }
 
 ReleaseCurrentLock();
@@ -839,7 +843,7 @@ if (CURRENTLINEPTR == NULL)  /* is prev undefined, set to line 1 */
    {
    if (liststart == NULL)
       {
-      EditVerbose("Edit: cannot increment line pointer in empty file\n");
+      EditVerbose("cannot increment line pointer in empty file\n");
       return true;
       }
    else
@@ -883,11 +887,11 @@ for (i = 0; i < n; i++)
       CURRENTLINEPTR = CURRENTLINEPTR->next;
       CURRENTLINENUMBER++;
 
-      EditVerbose("Edit: incrementing line pointer to line %d\n",CURRENTLINENUMBER);
+      EditVerbose("incrementing line pointer to line %d\n",CURRENTLINENUMBER);
       }
    else
       {
-      EditVerbose("Edit: inc pointer failed, still at %d\n",CURRENTLINENUMBER);
+      EditVerbose("inc pointer failed, still at %d\n",CURRENTLINENUMBER);
       }
    }
 
@@ -924,7 +928,7 @@ if (i < n || ip == NULL)
    return false;
    }
 
-EditVerbose("Edit: resetting pointers to line %d\n",n);
+EditVerbose("resetting pointers to line %d\n",n);
 
 CURRENTLINENUMBER = n;
 CURRENTLINEPTR = ip;
@@ -942,7 +946,7 @@ struct Edlist *editptr;
 
 if (strcmp(editptr->data,CURRENTLINEPTR->name) == 0)
    {
-   EditVerbose("Edit: ReplaceLineWith - line does not need correction.\n");
+   EditVerbose("ReplaceLineWith - line does not need correction.\n");
    return true;
    }
 
@@ -952,7 +956,7 @@ if ((sp = malloc(strlen(editptr->data)+1)) == NULL)
    return false;
    }
 
-EditVerbose("Edit: Replacing line %d with %10s...\n",CURRENTLINENUMBER,editptr->data);
+EditVerbose("Replacing line %d with %10s...\n",CURRENTLINENUMBER,editptr->data);
 strcpy(sp,editptr->data);
 free (CURRENTLINEPTR->name);
 CURRENTLINEPTR->name = sp;
@@ -990,7 +994,7 @@ DeleteItemList(*filestart);
 
 sprintf (buffer,"%s %s %s  2>&1",script,fname,CLASSTEXT[VSYSTEMHARDCLASS]);
 
-EditVerbose("Edit: Running command: %s\n",buffer);
+EditVerbose("Running command: %s\n",buffer);
 
 if ((pp = popen(buffer,"r")) == NULL)
    {
@@ -1000,7 +1004,7 @@ if ((pp = popen(buffer,"r")) == NULL)
 
 while (!feof(pp))   
    {
-   fgets(CURRENTITEM,bufsize,pp);
+   ReadLine(CURRENTITEM,bufsize,pp);
 
    if (!feof(pp))
       {
@@ -1038,7 +1042,7 @@ char *type;
   char *sp;
   int gotCR;
 
-EditVerbose("Edit: Checking end of line conventions: type = %s\n",type);
+EditVerbose("Checking end of line conventions: type = %s\n",type);
 
 if (strcmp("unix",type) == 0 || strcmp("UNIX",type) == 0)
    {
@@ -1110,7 +1114,7 @@ for (ip = VMOUNTABLES; ip != NULL; ip=ip->next)
       }
    else
       {
-      EditVerbose("Edit: have a server for %s\n",sp);
+      EditVerbose("have a server for %s\n",sp);
       }
    }
 }
@@ -1139,14 +1143,16 @@ for (ep = actions; ep != NULL; ep=ep->next)
 	                  {
 			  if ((fd = creat(filename,0644)) == -1)
 			     {
-			     printf("%s: unable to create file %s\n",VPREFIX,filename);
-			     perror("creat");
+			     sprintf(OUTPUT,"Unable to create file %s\n",filename);
+			     CfLog(cfinform,OUTPUT,"creat");
 			     }
 			  else
 			     {
+			     AUTOCREATED = true;
 			     close(fd);
 			     }
-			  EditVerbose("%s: creating file %s, mode 644\n",VPREFIX,filename);
+			  sprintf(OUTPUT,"Creating file %s, mode 644\n",filename);
+			  CfLog(cfinform,OUTPUT,"");
 			  return;
 	                  }
       }
@@ -1157,3 +1163,37 @@ for (ep = actions; ep != NULL; ep=ep->next)
 /**************************************************************/
 /* Level 3                                                    */
 /**************************************************************/
+
+AddEditfileClasses (list)
+
+struct Edit *list;
+
+{ char *sp, currentitem[maxvarsize];
+  struct Edlist *ep;
+
+for (ep = list->actions; ep != NULL; ep=ep->next)
+   {
+   if (ep->code == DefineClasses)
+      {
+      break;
+      }
+   }
+
+if (ep == NULL)
+   {
+   return;
+   }
+
+Debug("Entering AddEditfileClasses(%s)\n",ep->data);
+ 
+for (sp = ep->data; *sp != '\0'; sp++)
+   {
+   currentitem[0] = '\0';
+
+   sscanf(sp,"%[^,:.]",currentitem);
+
+   sp += strlen(currentitem);
+
+   AddClassToHeap(currentitem);
+   }
+}
