@@ -40,7 +40,9 @@ char *s;
 
 {
 fprintf (stderr,"%s:%s:%s\n",VPREFIX,VCURRENTFILE,s);
+SILENT = true;
 ReleaseCurrentLock();
+closelog(); 
 exit(1);
 }
 
