@@ -74,8 +74,8 @@ Debug2("cfengine: EmptyDir(%s)\n",path);
 
 if ((dirh = opendir(path)) == NULL)
    {
-   printf("%s: Can't open directory %s\n",VPREFIX,path);
-   perror("opendir");
+   sprintf(OUTPUT,"Can't open directory %s\n",path);
+   CfLog(cferror,OUTPUT,"opendir");
    return true;
    }
 
@@ -121,8 +121,8 @@ Debug("RecursiveCheck(%s)\n",name);
 
 if ((dirh = opendir(name)) == NULL)
    {
-   printf("%s: Can't open directory %s\n",VPREFIX,name);
-   perror("opendir");
+   sprintf(OUTPUT,"Can't open directory %s\n",name);
+   CfLog(cferror,OUTPUT,"opendir");
    return;
    }
 
@@ -140,22 +140,21 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
    strcpy(pcwd,name);                                   /* Assemble pathname */
    AddSlash(pcwd);
-   strcat(pcwd,dirp->d_name);
 
-   if (strlen(pcwd) > bufsize - buffer_margin)
+   if (BufferOverflow(pcwd,dirp->d_name))
       {
-      printf("%s: buffer overflow constructing string in RecursiveCheck\n",VPREFIX);
-      printf(" culprit: %s\n",pcwd);
       return;
       }
 
+   strcat(pcwd,dirp->d_name);
 
+   
    if (TRAVLINKS)
       {
       if (stat(pcwd,&statbuf) == -1)
          {
-         printf("%s: was working in %s when this happened:\n",VPREFIX,name);
-         printf("          RecursiveCheck(): Can't stat %s\n",dirp->d_name);
+         sprintf(OUTPUT,"RecursiveCheck was working on %s when this happened:\n",pcwd);
+	 CfLog(cferror,OUTPUT,"stat");
          continue;
          }
       }
@@ -163,15 +162,15 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
       if (lstat(pcwd,&statbuf) == -1)
          {
-         printf("%s: was working in %s when this happened:\n",VPREFIX,name);
-         printf("          RecursiveCheck(): Can't stat %s\n",dirp->d_name);
+         sprintf(OUTPUT,"RecursiveCheck was working in %s when this happened:\n",pcwd);
+         CfLog(cferror,OUTPUT,"lstat");
          continue;
          }
       }
 
    if (S_ISLNK(statbuf.st_mode))            /* should we ignore links? */
       {
-      CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr);
+      CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
       continue;
       }
 
@@ -185,18 +184,18 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
          {
          if (strcmp(dirp->d_name,".") != 0)  /* avoid infinite regression */
             {
-            CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr);
+            CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
             RecursiveCheck(pcwd,plus,minus,action,uidlist,gidlist,recurse-1,rlevel+1,ptr);
             }
          else
             {
-            CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr);
+            CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
             }
          }
       }
    else
       {
-      CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr);
+      CheckExistingFile(pcwd,plus,minus,action,uidlist,gidlist,&statbuf,ptr,ptr->acl_aliases);
       }
    }
 
@@ -206,7 +205,7 @@ closedir(dirh);
 
 /*********************************************************************/
 
-CheckExistingFile(file,plus,minus,action,uidlist,gidlist,dstat,ptr)
+CheckExistingFile(file,plus,minus,action,uidlist,gidlist,dstat,ptr,acl_aliases)
 
 char *file;
 mode_t plus,minus;
@@ -215,11 +214,12 @@ struct GidList *gidlist;
 enum fileactions action;
 struct stat *dstat;
 struct File *ptr;
+struct Item *acl_aliases;
 
 { mode_t newperm = dstat->st_mode;
   int amroot = true;
 
-Debug("Checking file/dir %s\n",file);
+Verbose("%s: Checking fs-object %s\n",VPREFIX,file);
 
 if (ptr != NULL)
    {
@@ -242,7 +242,6 @@ newperm = (dstat->st_mode & 07777) ;
 newperm |= plus;
 newperm &= ~minus;
 
-
 if (S_ISREG(dstat->st_mode) && (action == fixdirs || action == warndirs)) 
    {
    return;
@@ -277,7 +276,11 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISUID))
       {
       if (! IsItemIn(VSETUIDLIST,file))
          {
-         if (amroot) printf("%s: NEW SETUID root PROGRAM %s\n",VPREFIX,file);
+         if (amroot)
+	     {
+	     sprintf(OUTPUT,"NEW SETUID root PROGRAM %s\n",file);
+	     CfLog(cfinform,OUTPUT,"");
+	     }
          PrependItem(&VSETUIDLIST,file,NULL);
          }
       }
@@ -287,14 +290,24 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISUID))
          {
          case fixall:
          case fixdirs:
-         case fixplain: printf("%s: removing setuid (root) flag from %s...\n\n",VPREFIX,file);
-                        break;
+         case fixplain:  sprintf(OUTPUT,"Removing setuid (root) flag from %s...\n\n",file);
+	                 CfLog(cfinform,OUTPUT,"");
+			 
+	                 if (ptr != NULL)
+                            {
+                            AddMultipleClasses(ptr->defines);
+                            }
+                         break;
          case warnall:
          case warndirs:
-         case warnplain: if (amroot) printf("%s: WARNING setuid (root) flag on %s...\n\n",VPREFIX,file);
+         case warnplain: if (amroot)
+	                    {
+      	                    sprintf(OUTPUT,"WARNING setuid (root) flag on %s...\n\n",file);
+			    CfLog(cferror,OUTPUT,"");
+	                    }
                          break;
 
-         default:       break;
+         default:        break;
          }
       }
    }
@@ -311,7 +324,11 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISGID))
             }
          else
             {
-            if (amroot) printf("%s: NEW SETGID root PROGRAM %s\n",VPREFIX,file);
+            if (amroot)
+		{
+		sprintf(OUTPUT,"NEW SETGID root PROGRAM %s\n",file);
+		CfLog(cferror,OUTPUT,"");
+		}
             PrependItem(&VSETUIDLIST,file,NULL);
             }
          }
@@ -322,11 +339,17 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISGID))
          {
          case fixall:
          case fixdirs:
-         case fixplain: printf("%s: removing setgid (root) flag from %s...\n\n",VPREFIX,file);
-                        break;
+         case fixplain: sprintf(OUTPUT,"Removing setgid (root) flag from %s...\n\n",file);
+	                CfLog(cfinform,OUTPUT,"");
+	                if (ptr != NULL)
+			   {
+                           AddMultipleClasses(ptr->defines);
+			   }
+	                break;
          case warnall:
          case warndirs:
-         case warnplain: printf("%s: WARNING setgid (root) flag on %s...\n\n",VPREFIX,file);
+         case warnplain: sprintf(OUTPUT,"WARNING setgid (root) flag on %s...\n\n",file);
+	                 CfLog(cferror,OUTPUT,"");
                          break;
 
          default:        break;
@@ -334,17 +357,38 @@ if (dstat->st_uid == 0 && (dstat->st_mode & S_ISGID))
       }
    }
 
-CheckOwner(file,action,uidlist,gidlist,dstat);
+if (CheckOwner(file,action,uidlist,gidlist,dstat))
+   {
+   if (ptr != NULL)
+      {
+      AddMultipleClasses(ptr->defines);
+      }
+   }
 
 if (S_ISLNK(dstat->st_mode))             /* No point in checking permission on a link */
    {
    KillOldLink(file);
+
+   if (ptr != NULL)
+      {
+      AddMultipleClasses(ptr->defines);
+      }
    return;
    }
 
 if (stat(file,dstat) == -1)
+   {
    perror("stat");
+   }
 
+if (CheckACLs(file,action,acl_aliases))
+   {
+   if (ptr != NULL)
+      {
+      AddMultipleClasses(ptr->defines);
+      }
+   }
+ 
 if (((newperm & 07777) == (dstat->st_mode & 07777)) && (action != touch))    /* file okay */
    {
    return;
@@ -357,20 +401,26 @@ switch (action)
    case warnplain:
                 if (S_ISREG(dstat->st_mode))
                    {
-                   printf("%s: %s has permission %o\n",VPREFIX,file,dstat->st_mode & 07777);
-                   printf("%s: [should be %o]\n",VPREFIX,newperm & 07777);
+                   sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+		   CfLog(cferror,OUTPUT,"");
+                   sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+		   CfLog(cferror,OUTPUT,"");
                    }
                 break;
    case warndirs:
                 if (S_ISDIR(dstat->st_mode))
                    {
-                   printf("%s: %s has permission %o\n",VPREFIX,file,dstat->st_mode & 07777);
-                   printf("%s: [should be %o]\n",VPREFIX,newperm & 07777);
+                   sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+		   CfLog(cferror,OUTPUT,"");
+                   sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+		   CfLog(cferror,OUTPUT,"");
                    }
                 break;
    case warnall:   
-                printf("%s: %s has permission %o\n",VPREFIX,file,dstat->st_mode & 07777);
-                printf("%s: [should be %o]\n",VPREFIX,newperm & 07777);
+                sprintf(OUTPUT,"%s has permission %o\n",file,dstat->st_mode & 07777);
+		CfLog(cferror,OUTPUT,"");
+                sprintf(OUTPUT,"[should be %o]\n",newperm & 07777);
+		CfLog(cferror,OUTPUT,"");
                 break;
 
    case fixplain:
@@ -381,14 +431,20 @@ switch (action)
                       {
                       if (chmod (file,newperm & 07777) == -1)
 			 {
-                         printf("%s: chmod failed on %s\n",VPREFIX,file);
-                         perror("chmod");
+                         sprintf(OUTPUT,"chmod failed on %s\n",file);
+                         CfLog(cferror,OUTPUT,"chmod");
                          break;
                          }
                       }
 
-                   Verbose("%s had permission %o\n",file,dstat->st_mode & 07777);
-                   Verbose("[changed to %o]\n",newperm & 07777);
+                   sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+			   file,dstat->st_mode & 07777,newperm & 07777);
+		   CfLog(cfinform,OUTPUT,"");
+		   
+                   if (ptr != NULL)
+		      {
+		      AddMultipleClasses(ptr->defines);
+		      }
                    }
                 break;
 
@@ -399,14 +455,20 @@ switch (action)
                       {
                       if (chmod (file,newperm & 07777) == -1)
                          {
-                         printf("cfengine: chmod failed on %s\n",file);
-                         perror("chmod");
+                         sprintf(OUTPUT,"chmod failed on %s\n",file);
+			 CfLog(cferror,OUTPUT,"chmod");
                          break;
                          }
                       }
 
-		   Verbose("%s had permission %o\n",file,dstat->st_mode & 07777);
-		   Verbose("[changed to %o]\n",newperm & 07777);
+                   sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+			   file,dstat->st_mode & 07777,newperm & 07777);
+		   CfLog(cfinform,OUTPUT,"");
+		   
+                   if (ptr != NULL)
+		      {
+		      AddMultipleClasses(ptr->defines);
+		      }
                    }
                 break;
 
@@ -414,27 +476,36 @@ switch (action)
                    {
                    if (chmod (file,newperm & 07777) == -1)
 		      {
-                      printf("%s: chmod failed on %s\n",VPREFIX,file);
-                      perror("chmod");
+                      sprintf("chmod failed on %s\n",file);
+                      CfLog(cferror,OUTPUT,"chmod");
                       break;
                       }
                    }
 
-                Verbose("%s had permission %o\n",file,dstat->st_mode & 07777);
-                Verbose("[changed to %o]\n",newperm & 07777);
+                   sprintf(OUTPUT,"%s had permission %o, changed it to %o\n",
+			   file,dstat->st_mode & 07777,newperm & 07777);
+		   CfLog(cfinform,OUTPUT,"");
 
+                   if (ptr != NULL)
+		      {
+                      AddMultipleClasses(ptr->defines);
+		      }
                 break;
 
    case touch:  if (! DONTDO)
                    {
                    if (chmod (file,newperm & 07777) == -1)
 		      {
-                      printf("cfengine: chmod failed on %s\n",file);
-                      perror("chmod");
+                      sprintf("chmod failed on %s\n",file);
+                      CfLog(cferror,OUTPUT,"chmod");
                       break;
 		      }
                    utime (file,NULL);
                    }
+                if (ptr != NULL)
+		   {
+                   AddMultipleClasses(ptr->defines);
+		   }
                 break;
 
    default:     FatalError("cfengine: internal error CheckExistingFile(): illegal file action\n");
@@ -443,6 +514,34 @@ switch (action)
 
 /*********************************************************************/
 
+CheckCopiedFile(file,plus,minus,action,uidlist,gidlist,dstat,sstat,ptr,acl_aliases)
+
+char *file;
+mode_t plus,minus;
+struct UidList *uidlist;
+struct GidList *gidlist;
+enum fileactions action;
+struct stat *dstat;
+struct stat *sstat;
+struct File *ptr;
+struct Item *acl_aliases;
+
+{ mode_t newplus,newminus;
+
+if ((plus == 0) && (minus == 0))
+   {
+   newplus = sstat->st_mode & 07777;
+   newminus = ~(sstat->st_mode & 07777) & 07777;
+   CheckExistingFile(file,newplus,newminus,fixall,uidlist,gidlist,dstat,NULL,acl_aliases);
+   }
+else
+   {
+   CheckExistingFile(file,plus,minus,fixall,uidlist,gidlist,dstat,NULL,acl_aliases);
+   }
+}
+
+/*********************************************************************/
+
 CheckOwner(file,action,uidlist,gidlist,statbuf)
 
 char *file;
@@ -486,7 +585,7 @@ for (glp = gidlist; glp != NULL; glp=glp->next)
 
 if (uidmatch && gidmatch)
    {
-   return;
+   return false;
    }
 else
    {
@@ -503,13 +602,13 @@ else
    if (S_ISLNK(statbuf->st_mode) && (action == fixdirs || action == fixplain))
       {
       Debug2("File %s incorrect type (link), skipping...\n",file);
-      return;
+      return false;
       }
 
    if ((S_ISREG(statbuf->st_mode) && action == fixdirs) || (S_ISDIR(statbuf->st_mode) && action == fixplain))
       {
       Debug2("File %s incorrect type, skipping...\n",file);
-      return;
+      return false;
       }
 
    switch (action)
@@ -522,18 +621,18 @@ else
                      {
                      if (uid == -1 && gid == -1)
                         {
-                        printf("%s: touching %s\n",VPREFIX,file);
+                        printf("%s:   touching %s\n",VPREFIX,file);
                         }
                      else
                         {
                         if (uid != -1)
                            {
-                           printf("%s: changing owner of %s to uid %d\n",VPREFIX,file,uid);
+                           sprintf(OUTPUT,"  (Changing owner to uid %d)\n",uid);
                            }
 
                         if (gid != -1)
                            {
-                           printf("%s: changing owner of %s to gid %d\n",VPREFIX,file,gid);
+                           sprintf(OUTPUT,"  (Changing group to gid %d)\n",gid);
                            }
                         }
                      }
@@ -543,18 +642,26 @@ else
 #ifdef HAVE_LCHOWN
                      if (lchown(file,uid,gid) == -1)
                         {
-                        printf("%s: Cannot set ownership on link %s!\n",VPREFIX,file);
-                        perror("lchown");
+                        sprintf(OUTPUT,"Cannot set ownership on link %s!\n",file);
+                        CfLog(cferror,OUTPUT,"lchown");
                         }
+		     else
+			{
+			return true;
+			}
 #endif
                      }
                   else if (! DONTDO)
                      {
                      if (chown(file,uid,gid) == -1)
                         {
-                        printf("%s: Cannot set ownership on file %s!\n",VPREFIX,file);
-                        perror("chown");
+                        sprintf(OUTPUT,"Cannot set ownership on file %s!\n",file);
+                        CfLog(cferror,OUTPUT,"chown");
                         }
+		     else
+			{
+			return true;
+			}
                      }
                   break;
 
@@ -564,21 +671,26 @@ else
       case warnplain:
                   if ((pw = getpwuid(statbuf->st_uid)) == NULL)
                      {
-                     printf ("%s: File %s is not owned by anybody in the passwd database\n",VPREFIX,file);
-                     printf ("%s: (uid = %d,gid = %d)\n",VPREFIX,statbuf->st_uid,statbuf->st_gid);
+                     sprintf(OUTPUT,"File %s is not owned by anybody in the passwd database\n",file);
+		     CfLog(cferror,OUTPUT,"");
+                     sprintf(OUTPUT,"(uid = %d,gid = %d)\n",statbuf->st_uid,statbuf->st_gid);
+		     CfLog(cferror,OUTPUT,"");
                      break;
                      }
 
                   if ((gp = getgrgid(statbuf->st_gid)) == NULL)
                      {
-                     printf ("%s: File %s is not owned by any group in group database\n",VPREFIX,file);
+                     sprintf(OUTPUT,"File %s is not owned by any group in group database\n",file);
+		     CfLog(cferror,OUTPUT,"");
                      break;
                      }
 
-                  printf ("%s: File %s is owned by [%s], group [%s]\n",VPREFIX,file,pw->pw_name,gp->gr_name);
+                  sprintf(OUTPUT,"File %s is owned by [%s], group [%s]\n",file,pw->pw_name,gp->gr_name);
+		  CfLog(cferror,OUTPUT,"");
                   break;
       }
    }
+return false; 
 }
 
 
@@ -649,13 +761,13 @@ char *file1, *file2;
 
 if (stat(file2,&statbuf2) == -1)
    {
-   perror("stat");
+   CfLog(cferror,"","stat");
    return false;
    }
 
 if (stat(file1,&statbuf1) == -1)
    {
-   perror("stat");
+   CfLog(cferror,"","stat");
    return true;
    }
 
