@@ -70,6 +70,11 @@ AddSlash(str)
 char *str;
 
 {
+if ((strlen(str)== 0) || (str == NULL))
+   {
+   return;
+   }
+ 
 if (str[strlen(str)-1] != '/')
    {
    strcat(str,"/");
@@ -83,7 +88,30 @@ DeleteSlash(str)
 char *str;
 
 {
-if (str[strlen(str)-1] == '/')
+if ((strlen(str)== 0) || (str == NULL))
+   {
+   return;
+   }
+
+ if (str[strlen(str)-1] == '/')
+   {
+   str[strlen(str)-1] = '\0';
+   }
+}
+
+/*********************************************************************/
+
+DeleteNewline(str)
+
+char *str;
+
+{
+if ((strlen(str)== 0) || (str == NULL))
+   {
+   return;
+   }
+
+ if (str[strlen(str)-1] == '\n')
    {
    str[strlen(str)-1] = '\0';
    }
@@ -101,29 +129,16 @@ ChopLastNode(str)
 char *str;
 
 { char *sp;
-  int hasslash = false;
 
-for (sp = str; *sp != '\0'; sp++)
-   {
-   if (*sp == '/')
-      {
-      hasslash = true;
-      break;
-      }
-   }
-
-if ( ! hasslash)
+if ((sp = strrchr(str, '/')) == NULL)
    {
    return false;
    }
-
-for (sp = str+strlen(str); *sp != '/'; sp--)        /* skip link name */
+else
    {
    *sp = '\0';
+   return true;
    }
-
-*sp = '\0';
-return true;
 }
 
 /*********************************************************************/
@@ -188,12 +203,15 @@ char *ReadLastNode(str)
 char *str;
 
 { char *sp;
-
-for (sp = str+strlen(str); *sp != '/'; sp--)        /* skip link name */
+  
+if ((sp = strrchr(str, '/')) == NULL)
    {
+   return str;
+   }
+else
+   {
+   return sp + 1;
    }
-
-return sp+1;
 }
 
 /*********************************************************************/
@@ -216,18 +234,16 @@ for (sp = pathbuf+strlen(file); *sp != '/'; sp--)        /* skip link name */
 
 if (lstat(pathbuf,&statbuf) != -1)
    {
-   if (VERBOSE || DEBUG || D2)
+   if (S_ISLNK(statbuf.st_mode))
       {
-      if (S_ISLNK(statbuf.st_mode))
-         {
-         printf("%s: INFO: %s is a symbolic link, not a true directory!\n",VPREFIX,pathbuf);
-         }
+      Verbose("%s: INFO: %s is a symbolic link, not a true directory!\n",VPREFIX,pathbuf);
       }
 
    if (! S_ISLNK(statbuf.st_mode) && ! S_ISDIR(statbuf.st_mode))
       {
-      printf("%s: Warning. The object %s is not a directory.\n",VPREFIX,pathbuf);
-      printf("%s: Cannot make a new directory without deleting it!\n\n",VPREFIX);
+      sprintf(OUTPUT,"Warning. The object %s is not a directory.\n",pathbuf);
+      CfLog(cfinform,OUTPUT,"");
+      CfLog(cfinform,"Cannot make a new directory without deleting it!\n\n","");
       return(false);
       }
    }
@@ -254,8 +270,8 @@ for (sp = file, spc = currentpath; *sp != '\0'; sp++)
             {
             if (mkdir(currentpath,DEFAULTMODE) == -1)
                {
-               printf("%s: Unable to make directories to %s\n",VPREFIX,file);
-               perror("          ");
+               sprintf(OUTPUT,"Unable to make directories to %s\n",file);
+               CfLog(cferror,OUTPUT,"mkdir");
                return(false);
                }
             }
@@ -264,7 +280,8 @@ for (sp = file, spc = currentpath; *sp != '\0'; sp++)
          {
          if (! S_ISDIR(statbuf.st_mode))
             {
-            printf("%s: cannot make %s. %s is not a directory!\n",VPREFIX,file,currentpath);
+            sprintf(OUTPUT,"Cannot make %s. %s is not a directory!\n",file,currentpath);
+	    CfLog(cferror,OUTPUT,"");
             return(false);
             }
          }
@@ -288,8 +305,9 @@ char *str1, *str2;
 
 if ((strlen(str1)+len) > (bufsize - buffer_margin))
    {
-   printf("%s: Buffer overflow constructing string. Increase bufsize macro.\n",VPREFIX);
-   printf("%s: tried to add %s to %s\n",VPREFIX,str2,str1);
+   sprintf(OUTPUT,"Buffer overflow constructing string. Increase bufsize macro.\n");
+   CfLog(cferror,OUTPUT,"");
+   printf("%s: Tried to add %s to %s\n",VPREFIX,str2,str1);
    return true;
    }
 
@@ -319,6 +337,7 @@ SensibleFile(nodename,path)
 char *nodename, *path;
 
 { int i, suspicous = true;
+  struct stat statbuf; 
   char *sp;
 
 for (i = 0; VSKIPFILES[i] != NULL; i++)
@@ -338,8 +357,35 @@ for (sp = nodename; *sp != '\0'; sp++) /* Check for files like ".. ." */
       }
    }
 
-Silent("%s: Suspicous looking file object \"%s\" masquerading as hidden file in %s\n",VPREFIX,nodename,path);
+sprintf(OUTPUT,"Suspicous looking file object \"%s\" masquerading as hidden file in %s\n",nodename,path);
+CfLog(cfsilent,OUTPUT,"");
 
+strcpy(VBUFF,path);
+AddSlash(VBUFF);
+strcat(VBUFF,nodename); 
+ 
+if (lstat(VBUFF,&statbuf) == -1)
+   {
+   sprintf(OUTPUT,"Couldn't stat %s",VBUFF);
+   CfLog(cfsilent,OUTPUT,"lstat");
+   }
+else
+   {
+   if (S_ISLNK(statbuf.st_mode))
+      {
+      sprintf(OUTPUT,"   %s is a symbolic link\n",nodename);
+      CfLog(cfsilent,OUTPUT,"");
+      }
+   else if (S_ISDIR(statbuf.st_mode))
+      {
+      sprintf(OUTPUT,"   %s is a directory\n",nodename);
+      CfLog(cfsilent,OUTPUT,"");
+      }
+
+   sprintf(OUTPUT,"%s has size %d and full mode %o\n",nodename,statbuf.st_size,statbuf.st_mode);
+   CfLog(cfsilent,OUTPUT,"");
+   }
+ 
 return true;
 }
 
