@@ -48,7 +48,7 @@
 /*******************************************************************/
 
 char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
-
+char OUTPUT[bufsize*2];
 
 /*******************************************************************/
 /*                                                                 */
@@ -90,6 +90,8 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
   PUBLIC short D2 = false;
   PUBLIC short D3 = false;
   PUBLIC short VERBOSE = false;
+  PUBLIC short INFORM = false;
+  PUBLIC short LOGGING = false;
 
   PUBLIC char VINPUTFILE[bufsize];
   PUBLIC char VCURRENTFILE[bufsize];
@@ -177,6 +179,8 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
   PROTECTED  struct Item *VUNMOUNT=NULL;
   PROTECTED  struct Edit *VEDITLIST=NULL;
   PROTECTED  struct Edit *VEDITLISTTOP=NULL;
+  PROTECTED  struct CFACL  *VACLLIST=NULL;
+  PROTECTED  struct CFACL  *VACLLISTTOP=NULL;
   PROTECTED  struct Item *VCLASSDEFINE=NULL;
   PROTECTED  struct Process *VPROCLIST=NULL;
   PROTECTED  struct Process *VPROCTOP=NULL;
@@ -266,7 +270,8 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
       { "no-hard-classes",no_argument,0,'H'},
       { "no-splay",no_argument,0,'q'},
       { "no-lock",no_argument,0,'K'},
-      { "auto",no_argument,0,'A'},      
+      { "auto",no_argument,0,'A'},
+      { "inform",no_argument,0,'I'},
       { NULL,0,0,0 }
       };
 
@@ -400,6 +405,7 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
   PRIVATE short TIDYDIRS = false;
   PRIVATE short TRAVLINKS = false;
   PRIVATE short PTRAVLINKS = false;
+  PRIVATE short DEADLINKS = true;
   PRIVATE short DONTDO = false;
   PRIVATE short IFCONF = true;
   PRIVATE short PARSEONLY = false;
@@ -434,6 +440,8 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
   PRIVATE short NOHARDCLASSES=false;
   PRIVATE short NOSPLAY = false;
 
+  PROTECTED  struct Item *VACLBUILD = NULL;
+
   PRIVATE char LINKTYPE = 's';
   PRIVATE char AGETYPE = 'a';
   PRIVATE char COPYTYPE = 't';
@@ -441,6 +449,8 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
   PRIVATE char LISTSEPARATOR = ':';
   PRIVATE char LINKDIRS = 'k';
   PRIVATE char DISCOMP = '=';
+  PRIVATE char USESHELL = 'y';  /* yes or no */
+  PRIVATE char PURGE = 'n';
 
   PRIVATE char VUIDNAME[maxvarsize];
   PRIVATE char VGIDNAME[maxvarsize];
@@ -517,6 +527,10 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
      "define",
      "timeout",
      "freespace",
+     "nofile",
+     "acl",
+     "purge",
+     "useshell",
      NULL
      };
 
@@ -541,6 +555,8 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
   PRIVATE int SEARCHREPLACELEVEL=0;
   PRIVATE int FOREACHLEVEL = 0;
 
+  PRIVATE int AUTOCREATED = 0;
+
   PRIVATE char *COMMENTSTART;
   PRIVATE char *COMMENTEND;
 
@@ -621,6 +637,7 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
      "SplitOn",
      "AppendToLineIfNotContains",
      "DeleteLinesAfterThisMatching",
+     "DefineClasses",
      NULL
      };
 
@@ -693,6 +710,5 @@ char VBUFF[bufsize]; /* General workspace, contents not guaranteed */
 
   PUBLIC FILE *VLOGFP;
 
-
 /* EOF */
 
