@@ -113,17 +113,17 @@ if (VBROADCAST[0] == '\0')
 strcpy(IFR.ifr_name,VIFDEV[class]);
 IFR.ifr_addr.sa_family = AF_INET;
 
-Verbose("Interface name : %s\n",VIFDEV[class]);
+Verbose("Interface name: %s\n",VIFDEV[class]);
 
 if ((sk = socket(AF_INET,SOCK_DGRAM,IPPROTO_IP)) == -1)
    {
-   perror("cfengine: socket");
+   CfLog(cferror,"","socket");
    FatalError("Error in IfConfig()");
    }
 
 if (ioctl(sk,SIOCGIFFLAGS, (caddr_t) &IFR) == -1)   /* Get the device status flags */
    {
-   perror ("ioctl #1:");
+   CfLog(cferror,"","ioctl");
    FatalError("Software error: no such ethernet device");
    }
 
@@ -132,7 +132,7 @@ strcpy(IFR.ifr_name,VIFDEV[class]);                   /* copy this each time */
  
 if (ioctl(sk,SIOCGIFMETRIC, (caddr_t) &IFR) == -1)   /* Get the routing priority */
    {
-   perror ("ioctl #2:");
+   CfLog(cferror,"","ioctl");
    FatalError("Software error: error getting metric");
    }
 
@@ -165,7 +165,7 @@ enum classes class;
 
 if ((hp = gethostbyname(VSYSNAME.nodename)) == NULL)
    {
-   perror("GetIfStatus(): gethostbyname: ");
+   CfLog(cferror,"","gethostbyname");
    return false;
    }
 else
@@ -194,9 +194,9 @@ Verbose("IP address on %s: %s\n",VIFDEV[class],inet_ntoa(sin->sin_addr));
 
 if (strcmp((char *)inet_ntoa(*(struct in_addr *)(hp->h_addr)),(char *)inet_ntoa(sin->sin_addr)) != 0)
    {
-   printf("%s: This machine is configured with an address which differs from\n",VPREFIX);
-   printf("        :%s: the nameserver's information! (Insane!)\n",VPREFIX);
-   printf("        :%s: Don't quite know what to do...\n",VPREFIX);
+   CfLog(cferror,"This machine is configured with an address which differs from\n","");
+   CfLog(cferror,"the nameserver's information! (Insane!)\n","");
+   CfLog(cferror,"Don't quite know what to do...\n","");
    insane = true;
    }
 
@@ -213,7 +213,7 @@ strcpy(VBUFF,inet_ntoa(netmask.sin_addr));
 
 if (strcmp(VBUFF,VNETMASK))
    {
-   printf("%s: The netmask is incorrectly configured, resetting...\n",VPREFIX);
+   CfLog(cferror,"The netmask is incorrectly configured, resetting...\n","");
    insane = true;
    }
 
@@ -231,7 +231,8 @@ GetBroadcastAddr(inet_ntoa(inaddr));
 
 if (strcmp(VBUFF,VBROADCAST) != 0)
    {
-   printf("%s: broadcast address was %s (should be %s)\n",VPREFIX,VBUFF,VBROADCAST);
+   sprintf(OUTPUT,"Broadcast address was %s (should be %s)\n",VBUFF,VBROADCAST);
+   CfLog(cferror,OUTPUT,"");
    insane = true;
    }
 
@@ -269,12 +270,12 @@ sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
 if (ioctl(sk,SIOCSIFNETMASK, (caddr_t) &IFR)) 
    {
-   printf("%s: Permission to reconfigure netmask denied.\n",VPREFIX);
-   perror("ioctl");
+   CfLog(cferror,"Permission to reconfigure netmask denied.\n","ioctl");
    }
 else
    {
-   printf("%s: Set Netmask to: %s\n",VPREFIX,inet_ntoa(netmask.sin_addr));
+   sprintf(OUTPUT,"Set Netmask to: %s\n",inet_ntoa(netmask.sin_addr));
+   CfLog(cfinform,OUTPUT,"");
    }
 
 /* broadcast addr */
@@ -286,18 +287,18 @@ sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
 if (ioctl(sk,SIOCSIFBRDADDR, (caddr_t) &IFR) == -1) 
    {
-   printf("%s: Permission to reconfigure broadcast denied.\n",VPREFIX);
-   perror("ioctl");
+   CfLog(cferror,"Permission to reconfigure broadcast denied.\n","ioctl");
    return;
    } 
 
 if ((char *) sin->sin_addr.s_addr == NULL)
    {
-   printf("%s: No broadcast address on socket after configuration!!\n",VPREFIX);
+   CfLog(cferror,"No broadcast address on socket after configuration!!\n","");
    }
 else
    {
-   printf("%s: Set Broadcast address to: %s\n",VPREFIX,inet_ntoa(sin->sin_addr));
+   sprintf(OUTPUT,"Set Broadcast address to: %s\n",inet_ntoa(sin->sin_addr));
+   CfLog(cfinform,OUTPUT,"");
    }
 }
 
@@ -376,15 +377,17 @@ if (VDEFAULTROUTE[0] == '\0')
    return;
    }
 
-if ((pp = popen(VNETSTAT[VSYSTEMHARDCLASS],"r")) == NULL)
+if ((pp = cfpopen(VNETSTAT[VSYSTEMHARDCLASS],"r")) == NULL)
    {
-   printf("cfengine: failed to open pipe from %s\n",VNETSTAT[VSYSTEMHARDCLASS]);
+   sprintf(OUTPUT,"Failed to open pipe from %s\n",VNETSTAT[VSYSTEMHARDCLASS]);
+   CfLog(cferror,OUTPUT,"popen");
    return;
    }
 
 while (!feof(pp))
    {
-   fgets(VBUFF,bufsize,pp);
+   ReadLine(VBUFF,bufsize,pp);
+   
    if (strncmp(VBUFF,"default",7) == 0)
       {
       if (strstr(VBUFF,VDEFAULTROUTE))
@@ -395,15 +398,15 @@ while (!feof(pp))
          }
       else
          {
-         printf("%s: the default packet-route is incorrectly set\n",VPREFIX);
-         printf("%s: Please correct this manually using route(1).\n",VPREFIX);
+         CfLog(cferror,"The default packet-route is incorrectly set\n","");
+         CfLog(cferror,"Please correct this manually using route(1).\n","");
          break;
          }
       }
    defaultokay = 0;
    }
 
-pclose(pp);
+cfpclose(pp);
 
 if (defaultokay)
    {
@@ -437,11 +440,12 @@ if (! DONTDO)
    {
    if (ioctl(sk,SIOCADDRT, (caddr_t) &route) == -1)   /* Get the device status flags */
       {
-      perror ("ioctl SIOCADDRT:");
+      CfLog(cferror,"","ioctl SIOCADDRT:");
       FatalError("Software error: set default route");
       }
 
-   printf("%s: The routing table did not contain a default route.\n",VPREFIX);
-   printf("%s: I'm setting it to %s\n",VPREFIX,VDEFAULTROUTE);
+   CfLog(cferror,"The routing table did not contain a default route.\n","");
+   sprintf(OUTPUT,"I'm setting it to %s\n",VDEFAULTROUTE);
+   CfLog(cferror,OUTPUT,"");
    }
 }
