@@ -46,14 +46,14 @@ char *from, *to;
 int maxrecurse;
 
 { struct stat statbuf, deststatbuf;
-  CFDIR *dirh;
-  struct cfdirent *dirp;
   char newfrom[bufsize];
   char newto[bufsize];
-  void *bug_check;
   int save_uid, save_gid;
+  struct Item *namecache = NULL;
+  struct cfdirent *dirp;
+  CFDIR *dirh;
 
-if (maxrecurse == -1)  /* reached depth limit */
+if (maxrecurse == 0)  /* reached depth limit */
    {
    Debug2("MAXRECURSE ran out, quitting at level %s with endlist = %d\n",from,ip->next);
    return;
@@ -63,7 +63,7 @@ Debug2("RecursiveImage(%s,lev=%d,next=%d)\n",from,maxrecurse,ip->next);
 
 if (IgnoreFile(from,""))
    {
-   Verbose("cfengine: Ignoring directory %s\n",from);
+   Verbose("Ignoring directory %s\n",from);
    return;
    }
 
@@ -72,8 +72,6 @@ if (strlen(from) == 0)     /* Check for root dir */
    from = "/";
    }
 
-bug_check = ip->next;
-
   /* Check that dest dir exists before starting */
 
 strcpy(newto,to);
@@ -81,23 +79,18 @@ AddSlash(newto);
 
 if (! MakeDirectoriesFor(newto))
    {
-   printf("%s: Unable to make directory %s\n",VPREFIX,from);
+   sprintf(OUTPUT,"Unable to make directory %s\n",from);
+   CfLog(cferror,OUTPUT,"");
    return;
    }
 
 if ((dirh = cfopendir(from,ip)) == NULL)
    {
-   printf("%s: RecursiveImage(): Can't open directory [%s]\n",VPREFIX,from);
+   sprintf(OUTPUT,"copy can't open directory [%s]\n",from);
+   CfLog(cferror,OUTPUT,"");
    return;
    }
 
-if (ip->next != bug_check)
-   {
-   printf("%s: solaris BSD compat bug: opendir wrecked the heap memory!!",VPREFIX);
-   printf("%s: in copy to %s, using workaround...\n",VPREFIX,from);
-   ip->next = bug_check;
-   }
-
 for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
    {
    if (!SensibleFile(dirp->d_name,from))
@@ -115,6 +108,11 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       continue;
       }
 
+   if (ip->purge == 'y')
+      {
+      AppendItem(&namecache,dirp->d_name,NULL);
+      }
+   
    strcpy(newfrom,from);                                   /* Assemble pathname */
    AddSlash(newfrom);
    strcpy(newto,to);
@@ -142,7 +140,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       {
       if (cfstat(newfrom,&statbuf,ip) == -1)
          {
-         Verbose("cfengine: RecursiveImage(): Can't stat %s\n",newfrom);
+         Verbose("%s: (Can't stat %s)\n",VPREFIX,newfrom);
          continue;
          }
       }
@@ -150,7 +148,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       {
       if (cflstat(newfrom,&statbuf,ip) == -1)
          {
-         Verbose("cfengine: RecursiveImage(): Can't stat %s\n",newfrom);
+         Verbose("%s: (Can't stat %s)\n",VPREFIX,newfrom);
          if (cfreadlink(newfrom,VBUFF,bufsize,ip) != -1)
             {
             Verbose("          File is link to -> %s\n",VBUFF);
@@ -160,7 +158,7 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       }
 
    if (S_ISDIR(statbuf.st_mode))
-      {
+      {	
       save_uid = (ip->uid)->uid;
       save_gid = (ip->gid)->gid;
       
@@ -174,13 +172,12 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
 	 (ip->gid)->gid = statbuf.st_gid;
 	 }
 
-      if (stat(newto,&statbuf) == -1)
+      if (stat(newto,&deststatbuf) == -1)
 	 {
 	 mkdir(newto,statbuf.st_mode);
-	 bzero(&statbuf,sizeof(statbuf));
 	 }
 
-      CheckExistingFile(newto,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&statbuf,NULL);
+      CheckCopiedFile(newto,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&statbuf,&statbuf,NULL,ip->acl_aliases);
 
       (ip->uid)->uid = save_uid;
       (ip->gid)->gid = save_gid;
@@ -193,6 +190,13 @@ for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       }
    }
 
+
+if (ip->purge == 'y')
+   {
+   PurgeFiles(namecache,to);
+   DeleteItemList(namecache);
+   }
+ 
 DeleteCompressedArray(ip->inode_cache);
 
 ip->inode_cache = NULL;
@@ -223,7 +227,8 @@ if (!MountPathDefined())
 
 if (cfstat(ip->path,&statbuf,ip))
    {
-   printf("%s: Master file %s doesn't exist for copying\n",VPREFIX,ip->path);
+   sprintf(OUTPUT,"Master file %s doesn't exist for copying\n",ip->path);
+   CfLog(cferror,OUTPUT,"");
    return;
    }
 
@@ -236,8 +241,8 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
    
    if ((dirh = cfopendir(itp->name,ip)) == NULL)
       {
-      printf("%s: Can't open directory %s\n",VPREFIX,itp->name);
-      perror("cfopendir");
+      sprintf(OUTPUT,"Can't open directory %s\n",itp->name);
+      CfLog(cferror,OUTPUT,"cfopendir");
       return;
       }
 
@@ -259,8 +264,8 @@ for (itp = VMOUNTLIST; itp != NULL; itp=itp->next)
 
       if ((dirh2 = cfopendir(homedir,ip)) == NULL)
          {
-         printf("%s: Can't open directory %s\n",VPREFIX,homedir);
-         perror("cfopendir");
+	 sprintf(OUTPUT,"Can't open directory %s\n",homedir);
+	 CfLog(cferror,OUTPUT,"cfopendir");
          return;
          }
 
@@ -328,12 +333,12 @@ Debug2("CheckImage (source=%s destination=%s)\n",source,destination);
 
 if (cflstat(source,&sourcestatbuf,ip) == -1)
    {
-   printf("%s: Can't stat %s\n",VPREFIX,source);
+   sprintf(OUTPUT,"Can't stat %s\n",source);
+   CfLog(cferror,OUTPUT,"");
    FlushClientCache(ip);
    return;
    }
 
-
 if (sourcestatbuf.st_nlink > 1)    /* Preserve hard link structure when copying */
    {
    RegisterHardLink(sourcestatbuf.st_ino,destination,ip);
@@ -369,14 +374,9 @@ if (S_ISDIR(sourcestatbuf.st_mode))
       return;
       }
 
-   if (chmod (destdir,sourcestatbuf.st_mode  & 07777) == -1)
-      {
-      perror("chmod");
-      }
+   /* Now check any overrides */
 
-   stat(destdir,&deststatbuf);
-
-   CheckExistingFile(destdir,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,NULL);
+   CheckCopiedFile(destdir,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
    
    for (dirp = cfreaddir(dirh,ip); dirp != NULL; dirp = cfreaddir(dirh,ip))
       {
@@ -430,6 +430,65 @@ ImageCopy(sourcefile,destfile,sourcestatbuf,ip);
 FlushClientCache(ip);
 }
 
+/*********************************************************************/
+
+PurgeFiles(filelist,directory)
+
+struct Item *filelist;
+char *directory;
+
+{ DIR *dirh;
+  struct dirent *dirp;
+  char filename[bufsize];
+
+Debug("PurgeFiles(%s)\n",directory);
+ 
+ if ((dirh = opendir(directory)) == NULL)
+    {
+    sprintf(OUTPUT,"Can't open directory %s\n",directory);
+    CfLog(cferror,OUTPUT,"cfopendir");
+    return;
+    }
+
+ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
+    {
+    if (!SensibleFile(dirp->d_name,directory))
+       {
+       continue;
+       }
+
+    if (! IsItemIn(filelist,dirp->d_name))
+       {
+       strcpy(filename,directory);
+       AddSlash(filename);
+       strcat(filename,dirp->d_name);
+       
+       if (DONTDO)
+	  {
+	  printf("%s: Need to purge %s from copy dest directory\n",filename);
+	  }
+       else
+	  {
+	  sprintf(OUTPUT,"Purging %s in copy dest directory\n",filename);
+	  CfLog(cfsilent,OUTPUT,"");
+	  
+	  if (unlink(filename) == -1)
+	     {
+	     sprintf(OUTPUT,"Couldn't unlink %s\n",filename);
+	     CfLog(cfverbose,OUTPUT,"unlink");
+	     if (rmdir(filename) == -1)
+		{
+		sprintf(OUTPUT,"Couldn't unlink %s\n",filename);
+		CfLog(cfverbose,OUTPUT,"unlink");
+		}
+	     }
+	  }
+       }
+    }
+ 
+ closedir(dirh);
+}
+
 
 /*********************************************************************/
 /* Level 3                                                           */
@@ -444,12 +503,15 @@ struct Image *ip;
 
 { char linkbuf[bufsize], *lastnode;
   struct stat deststatbuf;
+  struct Link empty;
   int i, silent = false;
   mode_t srcmode = sourcestatbuf.st_mode;
   int ok_to_copy;
   
 Debug2("ImageCopy(%s,%s,+%o,-%o)\n",sourcefile,destfile,ip->plus,ip->minus);
 
+empty.defines = NULL;
+ 
 if (IgnoredOrExcluded(image,sourcefile,ip->inclusions,ip->exclusions))
    {
    return;
@@ -461,13 +523,13 @@ if (IsWildItemIn(VLINKCOPIES,lastnode) || IsWildItemIn(ip->symlink,lastnode))
    switch (ip->linktype)
       {
       case 's':
-                LinkFiles(destfile,sourcefile,NULL,NULL,NULL);
+                LinkFiles(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
                 break;
       case 'r':
-                RelativeLink(destfile,sourcefile,NULL,NULL,NULL);
+                RelativeLink(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
 	        break;
       case 'a':
-                AbsoluteLink(destfile,sourcefile,NULL,NULL,NULL);
+                AbsoluteLink(destfile,sourcefile,NULL,NULL,NULL,0,&empty);
                 break;
       default:
                 printf("%s: internal error, link type was [%c] in ImageCopy\n",VPREFIX,ip->linktype);
@@ -487,20 +549,23 @@ if (lstat(destfile,&deststatbuf) == -1)
    {
    if (strcmp(ip->action,"warn") == 0)
       {
-      Silent("%s: image file %s is non-existent\n",VPREFIX,destfile);
-      Silent("%s: (should be copy of %s)\n",VPREFIX, sourcefile);
+      sprintf(OUTPUT,"Image file %s is non-existent\n",destfile);
+      CfLog(cfsilent,OUTPUT,"");
+      sprintf(OUTPUT,"(should be copy of %s)\n",sourcefile);
+      CfLog(cfsilent,OUTPUT,"");
       return;
       }
 
    if (S_ISREG(srcmode))
       {
-      Silent("Image doesn't exist: copying %s:%s to %s\n",ip->server,sourcefile,destfile);      
+      sprintf(OUTPUT,"%s wasn't at destination",destfile);
+      CfLog(cfverbose,OUTPUT,"");
+      sprintf(OUTPUT,"Copying from %s:%s\n",ip->server,sourcefile);
+      CfLog(cfinform,OUTPUT,"");
 
       if (CopyReg(sourcefile,destfile,sourcestatbuf,deststatbuf,ip))
          {
-         /*chmod(destfile,sourcestatbuf.st_mode & 07777);*/
-         stat(destfile,&deststatbuf);
-         CheckExistingFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,NULL);
+         CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
 	 AddCopyClasses(ip);
          }
 
@@ -517,8 +582,8 @@ if (lstat(destfile,&deststatbuf) == -1)
          }
       else if (mkfifo (destfile,srcmode))
          {
-         printf ("%s: cannot create fifo `%s'",VPREFIX, destfile);
-	 Debug2("Leaving ImageCopy\n");
+         sprintf(OUTPUT,"Cannot create fifo `%s'", destfile);
+	 CfLog(cferror,OUTPUT,"mkfifo");
          return;
          }
       
@@ -531,12 +596,12 @@ if (lstat(destfile,&deststatbuf) == -1)
          {
          if (DONTDO)
             {
-            printf("%s: Make BLK/CHR/SOCK %s\n",VPREFIX,destfile);
+            Silent("%s: Make BLK/CHR/SOCK %s\n",VPREFIX,destfile);
             }
          else if (mknod (destfile, srcmode, sourcestatbuf.st_rdev))
             {
-            printf ("%s: cannot create special file `%s'",VPREFIX, destfile);
-	    Debug2("Leaving ImageCopy\n");
+            sprintf(OUTPUT,"Cannot create special file `%s'",destfile);
+	    CfLog(cferror,OUTPUT,"mknod");
             return;
             }
 	 
@@ -548,12 +613,14 @@ if (lstat(destfile,&deststatbuf) == -1)
       {
       if (cfreadlink(sourcefile,linkbuf,bufsize,ip) == -1)
          {
-         printf("%s: Can't readlink %s\n",VPREFIX,sourcefile);
+         sprintf(OUTPUT,"Can't readlink %s\n",sourcefile);
+	 CfLog(cferror,OUTPUT,"");
 	 Debug2("Leaving ImageCopy\n");
          return;
          }
 
-      Verbose("cfengine: imaging link from %s to %s\n",destfile,linkbuf);
+      sprintf(OUTPUT,"Checking link from %s to %s\n",destfile,linkbuf);
+      CfLog(cfverbose,OUTPUT,"");
 
       if (ip->linktype == 'a' && linkbuf[0] != '/')      /* Not absolute path - must fix */
          {
@@ -569,18 +636,18 @@ if (lstat(destfile,&deststatbuf) == -1)
          case 's':
 	           if (*linkbuf == '.')
 		      {
-		      RelativeLink(destfile,linkbuf,NULL,NULL,NULL);
+		      RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
 		      }
 		   else
 		      {
-                      LinkFiles(destfile,linkbuf,NULL,NULL,NULL);
+                      LinkFiles(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
 		      }
                    break;
          case 'r':
-                   RelativeLink(destfile,linkbuf,NULL,NULL,NULL);
+                   RelativeLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
 	           break;
          case 'a':
-                   AbsoluteLink(destfile,linkbuf,NULL,NULL,NULL);
+                   AbsoluteLink(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
                    break;
          default:
                    printf("cfengine: internal error, link type was [%c] in ImageCopy\n",ip->linktype);
@@ -600,21 +667,24 @@ else
    	       {
                case '<':  if (sourcestatbuf.st_size > ip->size)
 	                     {
-	   	             Silent("%s: source file %s is > %d bytes in copy\n",VPREFIX,sourcefile,ip->size);
+	   	             sprintf(OUTPUT,"Source file %s is > %d bytes in copy\n",sourcefile,ip->size);
+			     CfLog(cfsilent,OUTPUT,"");
 		             return;
 	                     }
 	                  break;
 		    
    	       case '=':  if (sourcestatbuf.st_size != ip->size)
 	                     {
-		             Silent("%s: source file %s is not %d bytes in copy\n",VPREFIX,sourcefile,ip->size);
+		             sprintf(OUTPUT,"Source file %s is not %d bytes in copy\n",sourcefile,ip->size);
+			     CfLog(cfsilent,OUTPUT,"");
 		             return;
 	                     }
 	                  break;
 		       
 	       default:  if (sourcestatbuf.st_size < ip->size)
 	                    {
-		            Silent("%s: source file %s is < %d bytes in copy\n",VPREFIX,sourcefile,ip->size);
+		            Silent(OUTPUT,"Source file %s is < %d bytes in copy\n",sourcefile,ip->size);
+			    CfLog(cfsilent,OUTPUT,"");
 		            return;
 	                    }
 	                  break;;
@@ -629,15 +699,15 @@ else
 		      }	     
 	           else
 	              {
-		      Verbose("cfengine: checksum comparison replaced by ctime: files not regular\n");
-		      Verbose("          %s -> %s\n",sourcefile,destfile);
+		      Verbose("%s: Checksum comparison replaced by ctime: files not regular\n",VPREFIX);
+		      Verbose("%s: %s -> %s\n",VPREFIX,sourcefile,destfile);
 		      ok_to_copy = (deststatbuf.st_ctime < sourcestatbuf.st_ctime);
 	              }
 
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      Verbose("cfengine: image file %s has a wrong MD5 checksum\n",destfile);
-                      Verbose("          (should be copy of %s)\n", sourcefile);
+                      Verbose("%s: Image file %s has a wrong MD5 checksum\n",VPREFIX,destfile);
+                      Verbose("%s: (should be copy of %s)\n",VPREFIX,sourcefile);
                       return;
                       }
 	           break;
@@ -648,15 +718,15 @@ else
 		      }	     
 	           else
 	              {
-		      Verbose("cfengine: checksum comparison replaced by ctime: files not regular\n");
-		      Verbose("          %s -> %s\n",sourcefile,destfile);
+		      Verbose("%s: Checksum comparison replaced by ctime: files not regular\n",VPREFIX);
+		      Verbose("%s: %s -> %s\n",VPREFIX,sourcefile,destfile);
 		      ok_to_copy = (deststatbuf.st_ctime < sourcestatbuf.st_ctime);
 	              }
 
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      Verbose("cfengine: image file %s has a wrong binary checksum\n",destfile);
-                      Verbose("          (should be copy of %s)\n", sourcefile);
+                      Verbose("%s: Image file %s has a wrong binary checksum\n",VPREFIX,destfile);
+                      Verbose("%s: (should be copy of %s)\n",VPREFIX,sourcefile);
                       return;
                       }
 	           break;
@@ -665,8 +735,8 @@ else
 	     
                    if (ok_to_copy && strcmp(ip->action,"warn") == 0)
                       { 
-                      Verbose("cfengine: image file %s out of date\n",destfile);
-                      Verbose("          (should be copy of %s)\n", sourcefile);
+                      Verbose("%s: Image file %s out of date\n",VPREFIX,destfile);
+                      Verbose("%s: (should be copy of %s)\n", VPREFIX,sourcefile);
                       return;
                       }
 	           break;
@@ -679,15 +749,15 @@ else
       {
       if (S_ISREG(srcmode))
          {
-         Verbose("cfengine: updating image %s from master %s\n",destfile,sourcefile);
+         sprintf(OUTPUT,"Updating image %s from master %s on %s\n",destfile,sourcefile,ip->server);
+	 CfLog(cfinform,OUTPUT,"");
 
 	 AddCopyClasses(ip);
 
          if (CopyReg(sourcefile,destfile,sourcestatbuf,deststatbuf,ip))
             {
-            chmod(destfile,srcmode & 07777);
             stat(destfile,&deststatbuf);
-            CheckExistingFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,NULL);
+            CheckCopiedFile(destfile,ip->plus,ip->minus,fixall,ip->uid,ip->gid,&deststatbuf,&sourcestatbuf,NULL,ip->acl_aliases);
             }
 
          return;
@@ -697,14 +767,15 @@ else
          {
          if (cfreadlink(sourcefile,linkbuf,bufsize,ip) == -1)
             {
-            printf("%s:Can't readlink %s\n",VPREFIX,sourcefile);
-	    Debug2("Leaving ImageCopy\n");
+            sprintf(OUTPUT,"Can't readlink %s\n",sourcefile);
+	    CfLog(cferror,OUTPUT,"");
             return;
             }
 
-         Verbose("cfengine: imaging link from %s to %s\n",destfile,linkbuf);
+         sprintf(OUTPUT,"Checkinging link from %s to %s\n",destfile,linkbuf);
+	 CfLog(cfverbose,OUTPUT,"");
 
-         LinkFiles(destfile,linkbuf,NULL,NULL,NULL);
+         LinkFiles(destfile,linkbuf,NULL,NULL,NULL,0,&empty);
          }
       }
    else
@@ -808,7 +879,6 @@ for (sp = ip->cache; sp != NULL; sp=sp->next)
 	    }
 	 else
 	    {
-	    Debug("cfreadlink returning %s\n",sp->cf_readlink);
 	    bzero(linkbuf,buffsize);
 	    strcpy(linkbuf,sp->cf_readlink);
 	    return 0;
@@ -939,7 +1009,7 @@ if (sstat.st_nlink > 1)  /* Preserve hard links, if possible */
       silent = SILENT;
       SILENT = true;
       
-      DoHardLink(dest,linkable);
+      DoHardLink(dest,linkable,NULL);
       
       SILENT = silent;
       return true;
@@ -992,13 +1062,14 @@ stat(new,&dstat);
 
 if ((dstat.st_size == 0) && (sstat.st_size != 0))
    {
-   Silent("%s: WARNING: new file was empty %s (old one wasn't)\n",VPREFIX,new);
+   sprintf(OUTPUT,"WARNING: new file was empty %s (old one wasn't)\n",new);
+   CfLog(cfsilent,OUTPUT,"");
    }
 
 if (rename(new,dest) == -1)
    {
-   printf("%s: SERIOUS PROBLEM! Could not install copy file as %s\n",VPREFIX,dest);
-   perror("rename");
+   sprintf(OUTPUT,"SERIOUS PROBLEM! Could not install copy file as %s\n",dest);
+   CfLog(cferror,OUTPUT,"rename");
    rename(backup,dest);
    return false;
    }
