@@ -39,19 +39,24 @@
 
 /*******************************************************************/
 
-InstallLocalInfo (value)
+InstallLocalInfo (varvalue)
 
-char *value;
+char *varvalue;
 
 { int number = -1;
   char buffer[maxvarsize], *sp;
-
+  char value[bufsize];
+  
 if ( ! IsInstallable(CLASSBUFF))
    {
    Debug1("Not installing %s, no match\n",value);
    return;
    }
 
+value[0] = '\0';
+
+ExpandVarstring(varvalue,value,NULL);
+ 
 Debug1("(Action is control, storing variable [%s=%s])\n",CURRENTITEM,value);
 
 switch (ScanVariable(CURRENTITEM))
@@ -372,6 +377,10 @@ switch(GetCommAttribute(item))
                    break;
    case cfinclude: PrependItem(&VINCLUDEPARSE,value,"any");
                    break;
+   case cfacl:     PrependItem(&VACLBUILD,value,"any");
+                   break;
+   case cfdefine:  HandleDefine(value);
+                   break;
    default:        yyerror("Illegal file attribute");
    }
 }
@@ -383,7 +392,7 @@ HandleOptionalImageAttribute(item)
 
 char *item;
 
-{ char value[maxvarsize];
+{ char value[bufsize];
 
 VBUFF[0] = value[0] = '\0';
 
@@ -393,7 +402,7 @@ sscanf(VBUFF,"%*[^=]=%s",value);
 
 if (value[0] == '\0')
    {
-   yyerror("Image attribute with no value");
+   yyerror("Copy attribute with no value");
    }
 
 Debug1("HandleOptionalImageAttribute(%s)\n",value);
@@ -432,6 +441,11 @@ switch(GetCommAttribute(item))
                    break;
    case cfsize:    HandleCopySize(value);
                    break;
+   case cfacl:     PrependItem(&VACLBUILD,value,"any");
+                   break;
+   case cfpurge:   HandlePurge(value);
+                   break;
+
    default:        yyerror("Illegal copy attribute");
    }
 }
@@ -520,6 +534,9 @@ switch(GetCommAttribute(item))
    case cfrmdirs:  HandleTidyRmdirs(value);
                    break;
 
+   case cfdefine:  HandleDefine(value);
+                   break;
+
    default:        yyerror("Illegal tidy attribute");
    }
 }
@@ -553,6 +570,8 @@ switch(GetCommAttribute(item))
                    break;
    case cfgroup:   strcpy(VGIDNAME,value);
                    break;
+   case cfdefine:  HandleDefine(value);
+                   break;
    default:        yyerror("Illegal directory attribute");
    }
 }
@@ -589,6 +608,8 @@ switch(GetCommAttribute(item))
 		   
    case cfsize:    HandleDisableSize(value);
                    break;
+   case cfdefine:  HandleDefine(value);
+                   break;
 
    default:        yyerror("Illegal disable attribute");
    }
@@ -632,6 +653,10 @@ switch(GetCommAttribute(item))
                    break;
    case cfcptype:  HandleCopyType(value);
                    break;
+   case cfnofile:  HandleDeadLinks(value);
+                   break;
+   case cfdefine:  HandleDefine(value);
+                   break;
    default:        yyerror("Illegal link attribute");
    }
 }
@@ -681,6 +706,10 @@ switch(GetCommAttribute(item))
                    break;
    case cfsignal:  HandleProcessSignal(value);
                    break;
+   case cfdefine:  HandleDefine(value);
+                   break;
+   case cfuseshell:HandleUseShell(value);
+                   break;
    default:        yyerror("Illegal process attribute");
    }
 }
@@ -708,9 +737,11 @@ Debug1("HandleOptionalLinkAttribute(%s)\n",value);
 
 switch(GetCommAttribute(item))
    {
-   case cftimeout: HandleTimeOut(value);
-                   break;
-   default:        yyerror("Illegal shellcommand attribute");
+   case cftimeout:  HandleTimeOut(value);
+                    break;
+   case cfuseshell: HandleUseShell(value);
+                    break;
+   default:         yyerror("Illegal shellcommand attribute");
    }
 
 }
@@ -1038,14 +1069,14 @@ Debug1("Installing mailserver (%s) for group (%s)",path,GROUPBUFF);
 
 /*******************************************************************/
 
-AppendScript(item,timeout)
+AppendScript(item,timeout,useshell)
 
-char *item;
+char *item, useshell;
 int timeout;
 
 { struct TwoDimList *tp = NULL;
   struct ShellComm *ptr;
-  char *sp, *spe, *spn;
+  char *sp;
 
 Debug1("Installing item (%s) in the script list\n",item);
 
@@ -1073,12 +1104,12 @@ for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
       FatalError("Memory Allocation failed for AppendScript() #1");
       }
 
-   if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+   if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
       {
       FatalError("Memory Allocation failed for Appendscript() #2");
       }
 
-   if ((spn = malloc(strlen(sp)+1)) == NULL)
+   if ((ptr->name = strdup(sp)) == NULL)
       {
       FatalError("Memory Allocation failed for Appendscript() #3");
       }
@@ -1092,12 +1123,8 @@ for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
       VSCRIPTTOP->next = ptr;
       }
 
-   strcpy (spn,sp);
-   strcpy (spe,CLASSBUFF);
-
    ptr->timeout = timeout;
-   ptr->name = spn;
-   ptr->classes = spe;
+   ptr->useshell = useshell;
    ptr->next = NULL;
    
    VSCRIPTTOP = ptr;
@@ -1113,8 +1140,7 @@ InstallLinkItem (from,to)
 char *from, *to;
 
 { struct Link *ptr;
-  char *spfrom,*spto,*spe;
-  char buffer[bufsize];
+  char buffer[bufsize], buffer2[bufsize];
   
 Debug1("Storing Link: %s -> %s\n",from,to);
 
@@ -1130,23 +1156,36 @@ ExpandVarstring(from,VBUFF,"");
 bzero(buffer,bufsize);
 ExpandVarstring(to,buffer,"");
 
+bzero(buffer2,bufsize);
+ExpandVarstring(ALLCLASSBUFFER,buffer2,""); 
+
 if ((ptr = (struct Link *)malloc(sizeof(struct Link))) == NULL)
    {
    FatalError("Memory Allocation failed for InstallListItem() #1");
    }
-if ((spfrom = malloc(strlen(VBUFF)+1)) == NULL)
+ 
+if ((ptr->from = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallListItem() #2");
    }
-if ((spto = malloc(strlen(buffer)+1)) == NULL)
+
+if ((ptr->to = strdup(buffer)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallListItem() #3");
    }
-if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallListItem() #4");
    }
 
+if ((ptr->defines = strdup(buffer2)) == NULL)
+   {
+   FatalError("Memory Allocation failed for InstallListItem() #4a");
+   }
+
+AddInstallable(ptr->defines);
+ 
 if (VLINKTOP == NULL)                 /* First element in the list */
    {
    VLINK = ptr;
@@ -1156,23 +1195,16 @@ else
    VLINKTOP->next = ptr;
    }
 
-strcpy (spfrom,VBUFF);
-strcpy (spto,buffer);
-strcpy (spe,CLASSBUFF);
-
-if (strlen(spfrom) > 1)
+if (strlen(ptr->from) > 1)
    {
-   DeleteSlash(spfrom);
+   DeleteSlash(ptr->from);
    }
 
-if (strlen(spto) > 1)
+if (strlen(ptr->to) > 1)
    {
-   DeleteSlash(spto);
+   DeleteSlash(ptr->to);
    }
 
-ptr->classes = spe;
-ptr->from = spfrom;
-ptr->to = spto;
 ptr->force = FORCELINK;
 ptr->silent = LINKSILENT;
 ptr->type = LINKTYPE;
@@ -1182,6 +1214,8 @@ ptr->copy = VCPLNPARSE;
 ptr->exclusions = VEXCLUDEPARSE;
 ptr->inclusions = VINCLUDEPARSE;
 ptr->recurse = VRECURSE;
+ptr->nofile = DEADLINKS;
+ 
 VLINKTOP = ptr;
 
 if (ptr->recurse != 0)
@@ -1199,7 +1233,7 @@ InstallLinkChildrenItem (from,to)
 char *from, *to;
 
 { struct Link *ptr;
-  char *spfrom,*spto, *spe, *sp;
+  char *sp, buffer[bufsize];
   struct TwoDimList *tp = NULL;
 
 Debug1("Storing Linkchildren item: %s -> %s\n",from,to);
@@ -1213,6 +1247,9 @@ if ( ! IsInstallable(CLASSBUFF))
 VBUFF[0]='\0';                                /* Expand any variables */
 ExpandVarstring(from,VBUFF,"");
 
+bzero(buffer,bufsize);
+ExpandVarstring(ALLCLASSBUFFER,buffer,""); 
+
 Build2DListFromVarstring(&tp,to,'/');
     
 Set2DList(tp);
@@ -1224,22 +1261,29 @@ for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
       FatalError("Memory Allocation failed for InstallListChildrenItem() #1");
       }
 
-   if ((spfrom = malloc(strlen(VBUFF)+1)) == NULL)
+   if ((ptr->from = strdup(VBUFF)) == NULL)
       {
       FatalError("Memory Allocation failed for InstallLinkchildrenItem() #2");
       }
 
-   if ((spto = malloc(strlen(sp)+1)) == NULL)
+   if ((ptr->to = strdup(sp)) == NULL)
       {
       FatalError("Memory Allocation failed for InstallLinkChildrenItem() #3");
       }
 
-   if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+   if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
       {
       FatalError("Memory Allocation failed for InstallLinkChildrenItem() #3");
       }
 
-   if (VCHLINKTOP == NULL)                 /* First element in the list */
+if ((ptr->defines = strdup(buffer)) == NULL)
+   {
+   FatalError("Memory Allocation failed for InstallListItem() #4a");
+   }
+
+AddInstallable(ptr->defines);
+
+ if (VCHLINKTOP == NULL)                 /* First element in the list */
       {
       VCHLINK = ptr;
       }
@@ -1248,13 +1292,6 @@ for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
       VCHLINKTOP->next = ptr;
       }
 
-   strcpy (spfrom,VBUFF);
-   strcpy (spto,sp);
-   strcpy (spe,CLASSBUFF);
-
-   ptr->classes = spe;
-   ptr->from = spfrom;
-   ptr->to = spto;
    ptr->force = FORCELINK;
    ptr->silent = LINKSILENT;
    ptr->type = LINKTYPE;
@@ -1265,7 +1302,7 @@ for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
    ptr->recurse = VRECURSE;
    VCHLINKTOP = ptr;
 
-   if (ptr->recurse != 0 && strcmp(spto,"linkchildren") == 0)
+   if (ptr->recurse != 0 && strcmp(ptr->to,"linkchildren") == 0)
       {
       yyerror("Sorry don't know how to recurse with linkchildren keyword");
       }
@@ -1285,7 +1322,7 @@ char *path, *defines;
 int freespace;
 
 { struct Disk *ptr;
-  char *spname, *spclass, *spdef, buffer[bufsize];
+  char buffer[bufsize];
 
 Debug1("Installing item (%s) in the required list\n",path);
 
@@ -1309,17 +1346,17 @@ if ((ptr = (struct Disk *)malloc(sizeof(struct Disk))) == NULL)
    FatalError("Memory Allocation failed for InstallRequired() #1");
    }
 
-if ((spname = malloc(strlen(VBUFF)+1)) == NULL)
+if ((ptr->name = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallRequired() #2");
    }
 
-if ((spclass = malloc(strlen(buffer)+1)) == NULL)
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallRequired() #2");
    }
 
-if ((spdef = malloc(strlen(defines)+1)) == NULL)
+if ((ptr->define = strdup(buffer)) == NULL)
    {
    FatalError("Memory Allocation failed for Installrequied() #2");
    }
@@ -1333,13 +1370,7 @@ else
    VREQUIREDTOP->next = ptr;
    }
 
-strcpy(spclass,CLASSBUFF);
-strcpy(spdef,buffer);
-strcpy(spname,VBUFF);
-
-ptr->name = spname;
-ptr->classes = spclass;
-ptr->define = spdef;
+AddInstallable(ptr->define);
 ptr->freespace = freespace;
 ptr->next = NULL;
 VREQUIREDTOP = ptr;
@@ -1383,7 +1414,6 @@ AppendMiscMount(from,onto,perm)
 char *from, *onto, *perm;
 
 { struct MiscMount *ptr;
-  char *sp1,*sp2,*sp3, *spe;
 
 Debug1("Adding misc mountable %s %s (%s) to list\n",from,onto,perm);
 
@@ -1398,31 +1428,26 @@ if ((ptr = (struct MiscMount *)malloc(sizeof(struct MiscMount))) == NULL)
    FatalError("Memory Allocation failed for AppendMiscMount #1");
    }
 
-if ((sp1 = malloc(strlen(from)+1)) == NULL)
+if ((ptr->from = strdup(from)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendMiscMount() #2");
    }
 
-if ((sp2 = malloc(strlen(onto)+1)) == NULL)
+if ((ptr->onto = strdup(onto)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendMiscMount() #3");
    }
 
-if ((sp3 = malloc(strlen(perm)+1)) == NULL)
+if ((ptr->options = strdup(perm)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendMiscMount() #4");
    }
 
-if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendMiscMount() #5");
    }
 
-strcpy(sp1,from);
-strcpy(sp2,onto);
-strcpy(sp3,perm);
-strcpy(spe,CLASSBUFF);
-
 if (VMISCMOUNTTOP == NULL)                 /* First element in the list */
    {
    VMISCMOUNT = ptr;
@@ -1432,10 +1457,6 @@ else
    VMISCMOUNTTOP->next = ptr;
    }
 
-ptr->classes = CLASSBUFF;
-ptr->from = sp1;
-ptr->onto = sp2;
-ptr->options  = sp3;
 ptr->next = NULL;
 VMISCMOUNTTOP = ptr;
 }
@@ -1491,16 +1512,17 @@ switch (action)
    {
    case files:
                   InstallFileListItem(CURRENTPATH,PLUSMASK,MINUSMASK,FILEACTION,
-				      VUIDNAME,VGIDNAME,VRECURSE,(char)PTRAVLINKS);
+				      VUIDNAME,VGIDNAME,VRECURSE,(char)PTRAVLINKS,ALLCLASSBUFFER);
                   break;
 
    case processes: InstallProcessItem(EXPR,RESTART,PROMATCHES,PROCOMP,
-				      PROSIGNAL,PROACTION,CLASSBUFF);
+				      PROSIGNAL,PROACTION,CLASSBUFF,ALLCLASSBUFFER,USESHELL);
                    break;
    case image:
                   InstallImageItem(CURRENTPATH,PLUSMASK,MINUSMASK,DESTINATION,
 				   IMAGEACTION,VUIDNAME,VGIDNAME,FORCECOPY,IMGSIZE,IMGCOMP,
-				   IMAGEBACKUP,VRECURSE,COPYTYPE,LINKTYPE,CFSERVER,ALLCLASSBUFFER);
+				   IMAGEBACKUP,VRECURSE,COPYTYPE,LINKTYPE,CFSERVER,
+				   ALLCLASSBUFFER,PURGE);
                   break;
 
    case tidy:     if (VAGE >= 99999)
@@ -1509,17 +1531,17 @@ switch (action)
                      return;
                      }
                   InstallTidyItem(CURRENTPATH,CURRENTITEM,VRECURSE,VAGE,(char)PTRAVLINKS,
-				  TIDYSIZE,AGETYPE,LINKDIRS,TIDYDIRS,CLASSBUFF);
+				  TIDYSIZE,AGETYPE,LINKDIRS,TIDYDIRS,CLASSBUFF,ALLCLASSBUFFER);
                   break;
 
-   case makepath: InstallMakePath(CURRENTPATH,PLUSMASK,MINUSMASK,VUIDNAME,VGIDNAME);
+   case makepath: InstallMakePath(CURRENTPATH,PLUSMASK,MINUSMASK,VUIDNAME,VGIDNAME,ALLCLASSBUFFER);
                   break;
 
-   case disable:  AppendDisable(CURRENTPATH,CURRENTITEM,ROTATE,DISCOMP,DISABLESIZE);
+   case disable:  AppendDisable(CURRENTPATH,CURRENTITEM,ROTATE,DISCOMP,DISABLESIZE,ALLCLASSBUFFER);
                   break;
 
    case shellcommands:
-                  AppendScript(CURRENTPATH,VTIMEOUT);
+                  AppendScript(CURRENTPATH,VTIMEOUT,USESHELL);
 		  break;
 
    case required:
@@ -1590,8 +1612,7 @@ InstallEditFile(file,edit,data)
 
 char *file,*edit,*data;
 
-{ char *filename,*datstring;
-  struct Edit *ptr;
+{ struct Edit *ptr;
 
 if (data == NULL)
    {
@@ -1604,11 +1625,12 @@ else
 
 if ( ! IsInstallable(CLASSBUFF))
    {
-   if (DEBUG || D1) printf("Not installing Edit no match\n");
+   Debug1("Not installing Edit no match\n");
    return;
    }
 
-VBUFF[0]='\0';                         /* Expand any variables */
+bzero(VBUFF,bufsize);
+
 ExpandVarstring(file,VBUFF,"");
 
 if ((ptr = (struct Edit *)malloc(sizeof(struct Edit))) == NULL)
@@ -1616,7 +1638,7 @@ if ((ptr = (struct Edit *)malloc(sizeof(struct Edit))) == NULL)
    FatalError("Memory Allocation failed for InstallEditFile() #1");
    }
 
-if ((filename = malloc(strlen(VBUFF)+1)) == NULL)
+if ((ptr->fname = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallEditFile() #2");
    }
@@ -1635,9 +1657,6 @@ if (strncmp(VBUFF,"home",4) && strlen(VBUFF) < 6)
    yyerror("Can't edit home directories: missing a filename after home");
    }
 
-strcpy (filename,VBUFF);
-
-ptr->fname=filename;
 ptr->next = NULL;
 ptr->actions = NULL;
 VEDITLISTTOP = ptr;
@@ -1653,7 +1672,6 @@ char *file,*edit,*data;
 { struct Edit *ptr;
   struct Edlist *top,*new;
   char varbuff[bufsize];
-  char *datstring, *spe;
 
 if (data == NULL)
    {
@@ -1703,25 +1721,19 @@ for (ptr = VEDITLIST; ptr != NULL; ptr=ptr->next)
          VBUFF[0]='\0';                         /* Expand any variables */
          ExpandVarstring(data,VBUFF,"");
 
-         if ((datstring = malloc(strlen(VBUFF)+1)) == NULL)
+         if ((new->data = strdup(VBUFF)) == NULL)
             {
             FatalError("Memory Allocation failed for AddEditAction() #1");
             }
-
-         strcpy(datstring,VBUFF);
-         new->data = datstring;
          }
 
       new->next = NULL;
 
-      if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+      if ((new->classes = strdup(CLASSBUFF)) == NULL)
          {
          FatalError("Memory Allocation failed for InstallEditFile() #3");
          }
 
-      strcpy (spe,CLASSBUFF);
-      new->classes=spe;
-
       if ((new->code = EditActionsToCode(edit)) == NoEdit)
          {
          yyerror("Unknown edit action");
@@ -1788,6 +1800,8 @@ for (ptr = VEDITLIST; ptr != NULL; ptr=ptr->next)
 		   {
 		   yyerror("Argument must be an integer, greater than zero");
 		   }
+	 case DefineClasses:
+	           AddInstallable(new->data);
          }
 
       return;
@@ -1821,17 +1835,14 @@ return (NoEdit);
 
 /********************************************************************/
 
+AppendDisable(path,type,rotate,comp,size,defines)
 
-AppendDisable(path,type,rotate,comp,size)
-
-char *path, *type, comp;
+char *path, *type, comp, *defines;
 short rotate;
 int size;
 
-{ char *sp, *spe, *sp2;
-  char buf[bufsize];
+{ char buf1[bufsize], buf2[bufsize];
   struct Disable *ptr;
-  int i;
 
 Debug1("Installing item (%s) in the disable list\n",path);
 
@@ -1847,26 +1858,31 @@ if (strlen(type) > 0 && strcmp(type,"plain") != 0 && strcmp(type,"file") !=0 &&
    yyerror("Invalid file type in Disable");
    }
 
-for (i=0; i<bufsize; i++)
-   {
-   buf[i] = VBUFF[i] = '\0';
-   }
-
-ExpandVarstring(path,buf,"");
+bzero(VBUFF,bufsize);
+bzero(buf1,bufsize);
+bzero(buf2,bufsize); 
 
+ExpandVarstring(path,buf1,"");
+ExpandVarstring(defines,buf2,"");
+ 
 if ((ptr = (struct Disable *)malloc(sizeof(struct Disable))) == NULL)
    {
    FatalError("Memory Allocation failed for AppendDisable() #1");
    }
 
-if ((sp = malloc(strlen(buf)+1)) == NULL)
+if ((ptr->name = strdup(buf1)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendDisable() #2");
    }
 
-if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+if ((ptr->defines = strdup(buf2)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendDisable() #3");
+   } 
+
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
+   {
+   FatalError("Memory Allocation failed for AppendDisable() #4");
    }
 
 if (strlen(type) == 0)
@@ -1878,11 +1894,16 @@ else
    sprintf(VBUFF,"%s",type);
    }
 
-if ((sp2 = malloc(strlen(VBUFF)+1)) == NULL)
+if ((ptr->type = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for AppendDisable() #4");
    }
 
+if (! IsItemIn(VALLADDCLASSES,buf2))
+   {
+   AppendItem(&VALLADDCLASSES,buf2,NULL);
+   }
+ 
 if (VDISABLETOP == NULL)                 /* First element in the list */
    {
    VDISABLELIST = ptr;
@@ -1892,29 +1913,23 @@ else
    VDISABLETOP->next = ptr;
    }
 
-strcpy (sp,buf);
-strcpy (spe,CLASSBUFF);
-strcpy (sp2,VBUFF);
-
-ptr->name = sp;
-ptr->type = sp2;
-ptr->classes = spe;
 ptr->rotate = rotate;
 ptr->comp = comp;
 ptr->size = size;
 ptr->next = NULL;
 VDISABLETOP = ptr;
 InitializeAction();
+AddInstallable(ptr->defines); 
 }
 
 /*******************************************************************/
 
-InstallTidyItem (path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes)
+InstallTidyItem (path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines)
 
 char *wild, *path;
 short age,tidysize,tidydirs;
 int rec, travlinks;
-char type, ldirs, *classes;
+char type, ldirs, *classes, *defines;
 
 { struct TwoDimList *tp = NULL;
   char *sp;
@@ -1929,11 +1944,11 @@ for (sp = Get2DListEnt(tp); sp != NULL; sp = Get2DListEnt(tp))
    {
    if (TidyPathExists(sp))
       {
-      AddTidyItem(sp,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes);
+      AddTidyItem(sp,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines);
       }
    else
       {
-      InstallTidyPath(sp,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes);
+      InstallTidyPath(sp,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines);
       }
    }
 
@@ -1943,16 +1958,15 @@ Delete2DList(tp);
 
 /*******************************************************************/
 
-InstallMakePath(path,plus,minus,uidnames,gidnames)
+InstallMakePath(path,plus,minus,uidnames,gidnames,defines)
 
-char *path;
+char *path, *defines;
 mode_t plus,minus;
 char *uidnames;
 char *gidnames;
 
 { struct File *ptr;
-  char *sp, *spe;
-
+  char buffer[bufsize]; 
 
 Debug1("InstallMakePath (%s) (+%o)(-%o)(%s)(%s)\n",path,plus,minus,uidnames,gidnames);
 
@@ -1965,21 +1979,31 @@ if ( ! IsInstallable(CLASSBUFF))
 VBUFF[0]='\0';                                /* Expand any variables */
 ExpandVarstring(path,VBUFF,"");
 
+bzero(buffer,bufsize);
+ExpandVarstring(ALLCLASSBUFFER,buffer,""); 
+
 if ((ptr = (struct File *)malloc(sizeof(struct File))) == NULL)
    {
    FatalError("Memory Allocation failed for InstallMakepath() #1");
    }
 
-if ((sp = malloc(strlen(VBUFF)+1)) == NULL)
+if ((ptr->path = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallMakepath() #2");
    }
 
-if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallMakepath() #3");
    }
+ 
+if ((ptr->defines = strdup(buffer)) == NULL)
+   {
+   FatalError("Memory Allocation failed for InstallMakepath() #3a");
+   }
 
+AddInstallable(ptr->defines); 
+ 
 if (VMAKEPATHTOP == NULL)                 /* First element in the list */
    {
    VMAKEPATH = ptr;
@@ -1989,17 +2013,15 @@ else
    VMAKEPATHTOP->next = ptr;
    }
 
-strcpy (sp,VBUFF);
-strcpy (spe,CLASSBUFF);
-
-ptr->classes = spe;
-ptr->path = sp;
 ptr->plus = plus;
 ptr->minus = minus;
 ptr->recurse = 0;
 ptr->action = fixdirs;
 ptr->uid = MakeUidList(uidnames);
 ptr->gid = MakeGidList(gidnames);
+ptr->inclusions = NULL;
+ptr->exclusions = NULL;
+ptr->acl_aliases= VACLBUILD; 
 ptr->next = NULL;
 VMAKEPATHTOP = ptr;
 InitializeAction();
@@ -2168,6 +2190,30 @@ switch (units)
 
 /*******************************************************************/
 
+HandlePurge(value)
+
+char *value;
+
+ {
+  Debug1("HandlePurge(%s)\n",value);
+  
+  if (strcmp(value,"true") == 0)
+    {
+    PURGE = 'y';
+    return;
+    }
+
+ if (strcmp(value,"false") == 0)
+    {
+    PURGE = 'n';
+    return;
+    }
+ 
+ yyerror("Illegal purge value");
+ }
+
+/*******************************************************************/
+
 HandleRequiredSize(value)
 
 char *value;
@@ -2321,7 +2367,7 @@ if (strcmp(value,"false") == 0 || strcmp(value,"off") == 0)
    return;
    }
 
-yyerror("Illegal copy attribute for force= ");
+yyerror("Illegal copy attribute for backup= ");
 }
 
 /*******************************************************************/
@@ -2343,6 +2389,28 @@ VTIMEOUT = num;
 }
 
 
+/*******************************************************************/
+
+HandleUseShell(value)
+
+char *value;
+
+{
+ if (strcmp(value,"true") == 0)
+   {
+   USESHELL = true;
+   return;
+   }
+
+if (strcmp(value,"false") == 0)
+   {
+   USESHELL = false;
+   return;
+   }
+
+yyerror("Illegal attribute for useshell= ");
+}
+
 /*******************************************************************/
 
 GetFileAction(action)
@@ -2366,9 +2434,9 @@ return (int) warnall;
 
 /*******************************************************************/
 
-InstallFileListItem(path,plus,minus,action,uidnames,gidnames,recurse,travlinks)
+InstallFileListItem(path,plus,minus,action,uidnames,gidnames,recurse,travlinks,defines)
 
-char *path;
+char *path, *defines;
 mode_t plus,minus;
 enum fileactions action;
 char *uidnames;
@@ -2377,7 +2445,7 @@ int recurse;
 char travlinks;
 
 { struct File *ptr;
-  char *sp, *spe, *spl;
+  char *spl;
   struct TwoDimList *tp = NULL;
 
 Debug1("InstallFileaction (%s) (+%o)(-%o) (%s) (%d) (%c)\n",path,plus,minus,FILEACTIONTEXT[action],action,travlinks);
@@ -2388,6 +2456,9 @@ if ( ! IsInstallable(CLASSBUFF))
    return;
    }
 
+bzero(VBUFF,bufsize);
+ExpandVarstring(ALLCLASSBUFFER,VBUFF,""); 
+ 
 Build2DListFromVarstring(&tp,path,'/');
     
 Set2DList(tp);
@@ -2399,15 +2470,22 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
       FatalError("Memory Allocation failed for InstallFileListItem() #1");
       }
 
-   if ((sp = malloc(strlen(spl)+1)) == NULL)
+   if ((ptr->path = strdup(spl)) == NULL)
       {
       FatalError("Memory Allocation failed for InstallFileListItem() #2");
       }
 
-   if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+   if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
       {
       FatalError("Memory Allocation failed for InstallFileListItem() #3");
-     }
+      }
+
+   if ((ptr->defines = strdup(VBUFF)) == NULL)
+      {
+      FatalError("Memory Allocation failed for InstallFileListItem() #3");
+      }
+
+   AddInstallable(ptr->defines);
 
    if (VFILETOP == NULL)                 /* First element in the list */
       {
@@ -2418,11 +2496,6 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
       VFILETOP->next = ptr;
       }
 
-   strcpy (sp,spl);
-   strcpy (spe,CLASSBUFF);
-
-   ptr->classes = spe;
-   ptr->path = sp;
    ptr->action = action;
    ptr->plus = plus;
    ptr->minus = minus;
@@ -2432,6 +2505,7 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
    ptr->exclusions = VEXCLUDEPARSE;
    ptr->inclusions = VINCLUDEPARSE;
    ptr->travlinks = travlinks;
+   ptr->acl_aliases  = VACLBUILD;
    ptr->next = NULL;
    VFILETOP = ptr;
    }
@@ -2444,15 +2518,14 @@ InitializeAction();
 
 /*******************************************************************/
 
-InstallProcessItem(expr,restart,matches,comp,signal,action,classes)
+InstallProcessItem(expr,restart,matches,comp,signal,action,classes,defines,useshell)
 
-char *expr, *restart, *classes;
+char *expr, *restart, *classes, *defines;
 short matches, signal;
-char action, comp;
+char action, comp, useshell;
 
 { struct Process *ptr;
-  char *sp1, *sp2, *spe;
-  char buf1[bufsize], buf2[bufsize];
+  char buf1[bufsize], buf2[bufsize], buf3[bufsize];
 
 if (comp == 't')
    {
@@ -2463,9 +2536,11 @@ Debug1("InstallProcessItem(%s,%s,%d,%d,%c)\n",expr,restart,matches,signal,action
 
 bzero(buf1,bufsize);
 bzero(buf2,bufsize);
+bzero(buf3,bufsize); 
 
 ExpandVarstring(expr,buf1,"");
 ExpandVarstring(restart,buf2,"");
+ExpandVarstring(defines,buf3,""); 
 
 if ( ! IsInstallable(CLASSBUFF))
    {
@@ -2478,21 +2553,25 @@ if ((ptr = (struct Process *)malloc(sizeof(struct Process))) == NULL)
    FatalError("Memory Allocation failed for InstallProcItem() #1");
    }
 
-if ((sp1 = malloc(strlen(buf1)+1)) == NULL)
+if ((ptr->expr = strdup(buf1)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallProcItem() #2");
    }
 
-if ((sp2 = malloc(strlen(buf2)+1)) == NULL)
+if ((ptr->restart = strdup(buf2)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallProcItem() #3");
    }
 
-if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+if ((ptr->defines = strdup(buf3)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallProcItem() #4");
    }
 
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
+   {
+   FatalError("Memory Allocation failed for InstallProcItem() #5");
+   }
 
 if (VPROCTOP == NULL)                 /* First element in the list */
    {
@@ -2503,26 +2582,26 @@ else
    VPROCTOP->next = ptr;
    }
 
-strcpy (sp1,buf1);
-strcpy (sp2,buf2);
-strcpy (spe,CLASSBUFF);
-
-ptr->classes = spe;
-ptr->expr = sp1;
-ptr->restart = sp2;
+if (! IsItemIn(VALLADDCLASSES,buf3))
+   {
+   AppendItem(&VALLADDCLASSES,buf3,NULL);
+   }
+ 
 ptr->matches = matches;
 ptr->comp = comp;
 ptr->signal = signal;
 ptr->action = action;
+ptr->useshell = useshell; 
 ptr->next = NULL;
 VPROCTOP = ptr;
 InitializeAction();
+AddInstallable(ptr->defines); 
 }
 
 /*******************************************************************/
 
 InstallImageItem(path,plus,minus,destination,action,uidnames,gidnames,
-		 force,size,comp,backup,rec,type,lntype,server,classbuffer)
+		 force,size,comp,backup,rec,type,lntype,server,classbuffer,purge)
 
 char *path, *destination, *action, *server, *classbuffer;
 mode_t plus,minus;
@@ -2530,23 +2609,23 @@ char *uidnames;
 char *gidnames;
 short force;
 short backup;
-char type, lntype, comp;
+char type, lntype, comp, purge;
 int rec, size;
 
 { struct Image *ptr;
-  char *sp, *spe, *spd, *spa, *spl, *sps, *spcl = NULL, *spserv;
+  char *sp, *spl; 
   char buf1[bufsize], buf2[bufsize], buf3[bufsize], buf4[bufsize];
   struct TwoDimList *tp = NULL, *tpserv = NULL;
   int i;
   struct hostent *hp;
-
+  
 if ( ! IsInstallable(CLASSBUFF))
    {
    Debug1("Not installing copy item, no match\n");
    return;
    }
   
-Debug1("InstallImageItem (%s) (+%o)(-%o) (%s)\n",path,plus,minus,destination);
+Debug1("InstallImageItem (%s) (+%o)(-%o) (%s), server=%s\n",path,plus,minus,destination,server);
 
 if (strlen(destination) == 0)
    {
@@ -2619,27 +2698,27 @@ if ((ptr = (struct Image *)malloc(sizeof(struct Image))) == NULL)
    FatalError("Memory Allocation failed for InstallImageItem() #1");
    }
 
-if ((spe = malloc(strlen(CLASSBUFF)+1)) == NULL)
+if ((ptr->classes = strdup(CLASSBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallImageItem() #3");
    }
 
-if ((spd = malloc(strlen(buf2)+1)) == NULL)
+if ((ptr->destination = strdup(buf2)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallImageItem() #4");
    }
 
-if ((sps = malloc(strlen(buf3)+1)) == NULL)
+if ((ptr->server = strdup(buf3)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallImageItem() #5");
    }
 
-if ((spa = malloc(strlen(action)+1)) == NULL)
+if ((ptr->action = strdup(action)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallImageItem() #6");
    }
 
-if ((spcl = malloc(strlen(buf4)+1)) == NULL)
+if ((ptr->defines = strdup(buf4)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallImageItem() #7");
    }
@@ -2652,7 +2731,7 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
       return;
       }
 
-   if ((sp = malloc(strlen(spl)+1)) == NULL)
+   if ((ptr->path = strdup(spl)) == NULL)
       {
       FatalError("Memory Allocation failed for InstallImageItem() #2");
       }
@@ -2666,18 +2745,6 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
       VIMAGETOP->next = ptr;
       }
 
-   strcpy (sp,spl);
-   strcpy (spe,CLASSBUFF);
-   strcpy (spd,buf2);
-   strcpy (spa,action);
-   strcpy (sps,buf3);
-   strcpy (spcl,buf4);
-
-   ptr->classes = spe;
-   ptr->server = sps;
-   ptr->path = sp;
-   ptr->destination = spd;
-   ptr->action=spa;
    ptr->plus = plus;
    ptr->minus = minus;
    ptr->uid = MakeUidList(uidnames);
@@ -2693,8 +2760,9 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
    ptr->symlink = VCPLNPARSE;
    ptr->exclusions = VEXCLUDEPARSE;
    ptr->inclusions = VINCLUDEPARSE;
-   ptr->defines = spcl;
    ptr->cache = NULL;
+   ptr->purge = purge;
+   ptr->acl_aliases = VACLBUILD;
    
    if (strcmp(buf3,"localhost") != 0)
       {
@@ -2709,26 +2777,20 @@ for (spl = Get2DListEnt(tp); spl != NULL; spl = Get2DListEnt(tp))
 	 exit(1);
 	 }
       
-      ptr->dns = (struct hostent *) malloc(sizeof(struct hostent));
+      ptr->dns = (struct in_addr *) malloc(sizeof(struct in_addr));
       
-      bcopy(hp,ptr->dns,sizeof(struct hostent));
+      bcopy((hp->h_addr),(ptr->dns),sizeof(struct in_addr));
+      }
+   else
+      {
+      ptr->dns = NULL;
       }
    
    ptr->inode_cache = NULL;
    
    VIMAGETOP = ptr;
 
-   for (sp = ptr->defines; *sp != '\0'; sp++)
-      {
-      bzero(buf1,bufsize);
-      sscanf(sp,"%[^,:.]",buf1);
-      sp += strlen(buf1);
-      
-      if (! IsItemIn(VALLADDCLASSES,buf1))
-	 {
-	 AppendItem(&VALLADDCLASSES,buf1,NULL);
-	 }
-      }
+   AddInstallable(ptr->defines);
    }
 
 /* Add to possible classes so actions will be installed */
@@ -2936,7 +2998,7 @@ HandleProcessMatches(value)
 
 char *value;
 
-{ int i = 0;
+{ int i = -1;
 
 switch (*value)
    {
@@ -3091,6 +3153,28 @@ yyerror("Invalid link action");
 
 /*******************************************************************/
 
+HandleDeadLinks(value)
+
+char *value;
+
+{
+if (strcmp(value,"kill") == 0)
+   {
+   DEADLINKS = true;
+   return;
+   }
+
+if (strcmp(value,"force") == 0)
+   {
+   DEADLINKS = false;
+   return;
+   }
+ 
+yyerror("Invalid deadlink option");
+}
+
+/*******************************************************************/
+
 HandleLinkType(value)
 
 char *value;
@@ -3149,8 +3233,9 @@ HandleDefine(value)
 
 char *value;
 
-{
-Debug("Additional classes on copying: %s\n",value);
+{ char *sp;
+ 
+Debug("Define response classes: %s\n",value);
 
 if (strlen(value) > bufsize)
    {
@@ -3159,9 +3244,17 @@ if (strlen(value) > bufsize)
 
 strcpy(ALLCLASSBUFFER,value);
 
-if (! IsItemIn(VALLADDCLASSES,value))
+for (sp = value; *sp != '\0'; sp++)
    {
-   AppendItem(&VALLADDCLASSES,value,NULL);
+   if (*sp == ':' || *sp == ',' || *sp == '.')
+      {
+      continue;
+      }
+   
+   if (! isalnum(*sp))
+      {
+      yyerror("illegal class list in define=");
+      }
    }
 }
 
@@ -3336,16 +3429,15 @@ return(gidlist);
 
 /*******************************************************************/
 
+InstallTidyPath(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines)
 
-InstallTidyPath(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes)
-
-char *wild, *path;
+char *wild, *path, *defines;
 short age,tidydirs;
 int rec, travlinks,tidysize;
 char type, ldirs, *classes;
 
 { struct Tidy *ptr;
-  char *pp, *sp;
+  char *sp;
   int no_of_links = 0;
 
 if (!IsInstallable(classes))
@@ -3367,7 +3459,7 @@ if ((ptr = (struct Tidy *)malloc(sizeof(struct Tidy))) == NULL)
    FatalError("Memory Allocation failed for InstallTidyItem() #1");
    }
 
-if ((pp = malloc(strlen(VBUFF)+1)) == NULL)
+if ((ptr->path = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallTidyItem() #3");
    }
@@ -3381,10 +3473,10 @@ else
    VTIDYTOP->next = ptr;
    }
 
-if (rec != INFINITERECURSE && strncmp("home/",pp,5) == 0)     /* Is this a subdir of home wildcard? */
+if (rec != INFINITERECURSE && strncmp("home/",ptr->path,5) == 0) /* Is this a subdir of home wildcard? */
    {
-   for (sp = pp; *sp != '\0'; sp++)                     /* Need to make recursion relative to start */
-      {                                                    /* of the search, not relative to home */
+   for (sp = ptr->path; *sp != '\0'; sp++)                     /* Need to make recursion relative to start */
+      {                                                        /* of the search, not relative to home */
       if (*sp == '/')
          {
          no_of_links++;
@@ -3392,24 +3484,21 @@ if (rec != INFINITERECURSE && strncmp("home/",pp,5) == 0)     /* Is this a subdi
       }
    }
 
-strcpy (pp,VBUFF);
-
 ptr->tidylist = NULL;
-ptr->path = pp;
 ptr->recurse = rec + no_of_links;
 ptr->next = NULL;
 VTIDYTOP = ptr;
 
-AddTidyItem(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes);
+AddTidyItem(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines);
 
 InitializeAction();
 }
 
 /*********************************************************************/
 
-AddTidyItem(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes)
+AddTidyItem(path,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines)
 
-char *wild, *path;
+char *wild, *path, *defines;
 short age,tidysize,tidydirs;
 int rec, travlinks;
 char type, ldirs,*classes;
@@ -3432,7 +3521,7 @@ for (ptr = VTIDY; ptr != NULL; ptr=ptr->next)
 
    if (strcmp(ptr->path,varbuff) == 0)
       {
-      PrependTidy(&(ptr->tidylist),wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes);
+      PrependTidy(&(ptr->tidylist),wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines);
       
       /* Must have the maximum recursion level here */
       
@@ -3540,7 +3629,6 @@ char *path, *hostname, *classes;
 struct Auth **list, **listtop;
 
 { struct Auth *ptr;
-  char *pp;
 
 if (!IsInstallable(classes))
    {
@@ -3563,7 +3651,7 @@ if ((ptr = (struct Auth *)malloc(sizeof(struct Auth))) == NULL)
    FatalError("Memory Allocation failed for InstallAuthPath() #1");
    }
 
-if ((pp = malloc(strlen(VBUFF)+1)) == NULL)
+if ((ptr->path = strdup(VBUFF)) == NULL)
    {
    FatalError("Memory Allocation failed for InstallAuthPath() #3");
    }
@@ -3577,14 +3665,11 @@ else
    (*listtop)->next = ptr;
    }
 
-strcpy (pp,VBUFF);
-
 ptr->accesslist = NULL;
-ptr->path = pp;
 ptr->next = NULL;
 *listtop = ptr;
 
-AddAuthHostItem(pp,hostname,classes,list);
+AddAuthHostItem(ptr->path,hostname,classes,list);
 }
 
 /***********************************************************************/
@@ -3652,16 +3737,16 @@ return false;
 
 /*********************************************************************/
 
-PrependTidy(list,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes)
+PrependTidy(list,wild,rec,age,travlinks,tidysize,type,ldirs,tidydirs,classes,defines)
 
 struct TidyPattern **list;
 char *wild;
 short age,tidysize,tidydirs;
 int rec, travlinks;
-char type, ldirs,*classes;
+char type, ldirs, *classes, *defines;
 
 { struct TidyPattern *tp;
-  char *spe,*sp;
+  char *spe,*sp, buffer[bufsize];
 
 if ((tp = (struct TidyPattern *)malloc(sizeof(struct TidyPattern))) == NULL)
    {
@@ -3669,19 +3754,27 @@ if ((tp = (struct TidyPattern *)malloc(sizeof(struct TidyPattern))) == NULL)
    FatalError("");
    }
 
-if ((sp = malloc(strlen(wild)+1)) == NULL)
+if ((sp = strdup(wild)) == NULL)
    {
    FatalError("Memory Allocation failed for PrependTidy() #2");
    }
 
+bzero(buffer,bufsize);
+ExpandVarstring(defines,buffer,""); 
+
+if ((tp->defines = strdup(buffer)) == NULL)
+   {
+   FatalError("Memory Allocation failed for PrependTidy() #2a");
+   }
+
+AddInstallable(tp->defines); 
+ 
 if ((classes!= NULL) && (spe = malloc(strlen(classes)+2)) == NULL)
    {
    perror("Can't allocate memory in PrependItem()");
    FatalError("");
    }
 
-strcpy(sp,wild);
-
 tp->size = tidysize;
 tp->recurse = rec;
 tp->age = age;
