@@ -69,17 +69,17 @@ char *string;
 { struct Item *ip;
   char *sp;
 
-EditVerbose("Edit: Inserting %s \n",string);
+EditVerbose("Inserting %s \n",string);
 
 if ((ip = (struct Item *)malloc(sizeof(struct Item))) == NULL)
    {
-   perror("Can't allocate memory in InsertItemAfter()");
+   CfLog(cferror,"","Can't allocate memory in InsertItemAfter()");
    FatalError("");
    }
 
 if ((sp = malloc(strlen(string)+1)) == NULL)
    {
-   perror("Can't allocate memory in InsertItemAfter()");
+   CfLog(cferror,"","Can't allocate memory in InsertItemAfter()");
    FatalError("");
    }
 
@@ -135,31 +135,23 @@ EditVerbose("Edit: Inserting file %s \n",string);
 
 if ((fp=fopen(string,"r")) == NULL)
    {
-   Verbose("Edit: could not open file %s\n",string);
+   Verbose("Could not open file %s\n",string);
    return;
    }
 
 while(!feof(fp))
    {
-   bzero(linebuf,bufsize);   
-   fgets(linebuf,bufsize,fp);
-   
-   if (strlen(linebuf) == 0)
-      {
-      break;
-      }
-   
-   Chop(linebuf);
+   ReadLine(linebuf,bufsize,fp);
    
    if ((ip = (struct Item *)malloc(sizeof(struct Item))) == NULL)
       {
-      perror("Can't allocate memory in InsertItemAfter()");
+      CfLog(cferror,"","Can't allocate memory in InsertItemAfter()");
       FatalError("");
       }
    
    if ((sp = malloc(strlen(linebuf)+1)) == NULL)
       {
-      perror("Can't allocate memory in InsertItemAfter()");
+      CfLog(cferror,"","Can't allocate memory in InsertItemAfter()");
       FatalError("");
       }
 
@@ -197,6 +189,7 @@ while(!feof(fp))
 NUMBEROFEDITS++;
 
 fclose(fp);
+ 
 return;
 }
 
@@ -214,7 +207,7 @@ for (ip = list; ip != NULL; ip=ip->next)
    {
    if (EDABORTMODE && ItemMatchesRegEx(ip->name,VEDITABORT))
       {
-      Verbose("Aborting search, regex %s matches line\n",VEDITABORT);
+      EditVerbose("Aborting search, regex %s matches line\n",VEDITABORT);
       return NULL;
       }
    
@@ -248,8 +241,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s:  %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   CfLog(cferror,OUTPUT,"");
+   sprintf("%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return NULL;
    }
 
@@ -269,7 +264,8 @@ for (ip = list; (ip != NULL); ip=ip->next)
       }
    else if (match < -1)
       {
-      printf("%s: Regular expression internal error [%s]\n",VPREFIX,string); 
+      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
+      CfLog(cferror,OUTPUT,"");
       }
    }
 
@@ -313,14 +309,11 @@ char *string;
   char fastmap[(1 << BYTEWIDTH)];
   struct re_pattern_buffer buf;
   struct re_registers regs;
-  int match, line = 1;
+  int match, line = CURRENTLINENUMBER;
   char *err;
 
-if (DEBUG || D2)
-   {
-   printf("LocateItemMatchingRegExp(list,%s)\n",string);
-   }
-
+Debug2("LocateItemMatchingRexExp(%s,%s)\n",list->name,string);
+  
 buf.allocated = 0;
 buf.buffer = NULL;
 buf.fastmap = fastmap;
@@ -328,8 +321,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s:%s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return NULL;
    }
 
@@ -345,14 +340,16 @@ for (ip = list; (ip != NULL); ip=ip->next, line++)
       
    if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
       {
-      Debug2("REGEXP matched [%s] to [%s]\n",ip->name,string);
       EditVerbose("Edit: Search ended at line %d\n",line);
+      EditVerbose("Edit: (Found %s)\n",ip->name);
       CURRENTLINENUMBER = line;
+      CURRENTLINEPTR = ip;
       return ip;
       }
    else if (match < -1)
       {
-      printf("%s: Regular expression internal error [%s]\n",VPREFIX,string); 
+      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
+      CfLog(cferror,OUTPUT,"");
       }
    }
 
@@ -374,8 +371,6 @@ char *string;
   int match, line = 1;
   char *err;
 
-Debug2("LocateItemMatchingRegExp(list,%s)\n",string);
-
 buf.allocated = 0;
 buf.buffer = NULL;
 buf.fastmap = fastmap;
@@ -383,8 +378,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s:  %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return NULL;
    }
 
@@ -400,18 +397,19 @@ for (ip = list; (ip != NULL); ip=ip->next)
       
    if ((match = re_search(&buf,ip->name,strlen(ip->name),0,strlen(ip->name),&regs)) >= 0)
       {
-      Debug2("REGEXP found [%s] in [%s]\n",string,ip->name);
-      EditVerbose("Edit: Search ended at line %d\n",line);
+      EditVerbose("Search ended at line %d\n",line);
       CURRENTLINENUMBER = line;
+      CURRENTLINEPTR = ip;
       return ip;
       }
    else if (match < 0)
       {
-      printf ("%s: ERROR in regular expression [%s]!\n",VPREFIX,string);
+      sprintf(OUTPUT,"ERROR in regular expression [%s]!\n",string);
+      CfLog(cferror,OUTPUT,"");
       }
    }
 
-EditVerbose("Edit: Search for %s failed. Current line still %d\n",string,CURRENTLINENUMBER);
+EditVerbose("Search for %s failed. Current line still %d\n",string,CURRENTLINENUMBER);
 
 return NULL;
 }
@@ -436,7 +434,7 @@ Debug2("DeleteToRegExp(list,%s)\n",string);
 
 if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    {
-   printf("%s: File line-pointer undefined during editfile action\n",VPREFIX);
+   CfLog(cferror,"File line-pointer undefined during editfile action\n","");
    return true;
    }
 
@@ -447,8 +445,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
@@ -462,16 +462,14 @@ for (ip = CURRENTLINEPTR; (ip != NULL); ip=ip->next)
       
    if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
       {
-      Debug2("REGEXP matched [%s] to [%s]\n",ip->name,string);
-      Debug2("cfengine: (Found a match in file)\n");
-
       linefound = true;
       ip_end = ip;
       break;
       }
    else if (match < -1)
       {
-      printf("cfengine: Regular expression internal error [%s]\n",string); 
+      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
+      CfLog(cferror,OUTPUT,"");
       }
    }
 
@@ -554,7 +552,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
          *list = ip->next;
          free((char *)ip);
 
-         EditVerbose("Edit: Deleted item %s\n",ip->name);
+         EditVerbose("Deleted item %s\n",ip->name);
          NUMBEROFEDITS++;
          return true;
          }
@@ -568,7 +566,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
 	    }
          free((char *)ip);
 
-         EditVerbose("Edit: Deleted item %s\n",string);
+         EditVerbose("Deleted item %s\n",string);
          NUMBEROFEDITS++;
          return true;
          }
@@ -601,8 +599,8 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   sprintf(OUTPUT,"%s\n",err);
    return false;
    }
 
@@ -626,7 +624,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
          *list = ip->next;
          free((char *)ip);
 
-         EditVerbose("Edit: Deleted item %s\n",string);
+         EditVerbose("Deleted item %s\n",string);
          NUMBEROFEDITS++;
          return true;
          }
@@ -640,7 +638,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
 	    }
          free((char *)ip);
 
-         EditVerbose("Edit: Deleted item %s\n",string);
+         EditVerbose("Deleted item %s\n",string);
          NUMBEROFEDITS++;
          return true;
          }
@@ -681,7 +679,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
 	    }
          free((char *)ip);
 
-         EditVerbose("Edit: Deleted item %s\n",ip->name);
+         EditVerbose("Deleted item %s\n",ip->name);
          NUMBEROFEDITS++;
          return true;
          }
@@ -695,7 +693,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
 	    }
          free((char *)ip);
 
-         EditVerbose("Edit: Deleted item %s\n",string);
+         EditVerbose("Deleted item %s\n",string);
          NUMBEROFEDITS++;
          return true;
          }
@@ -730,7 +728,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
       {
       if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
          {
-         printf("%s: bufsize overflow while commenting line - abort\n",VPREFIX);
+         CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
          return false;
          }
 
@@ -739,15 +737,15 @@ for (ip = *list; ip != NULL; ip=ip->next)
          continue;
          }
 
-      EditVerbose("Edit: Commenting %s%s%s\n",comm,ip->name,end);
+      EditVerbose("Commenting %s%s%s\n",comm,ip->name,end);
 
       sprintf(buff,"%s%s%s",comm,ip->name,end);
       free(ip->name);
 
       if ((ip->name = malloc(strlen(buff)+1)) == NULL)
          {
-         printf("cfengine: malloc in CommentItemStarting\n ");
-         exit(1);
+         CfLog("malloc in CommentItemStarting\n","");
+         FatalError("");;
          }
 
       strcpy(ip->name,buff);
@@ -782,7 +780,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
       {
       if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
          {
-         printf("%s: bufsize overflow while commenting line - abort\n",VPREFIX);
+         CfLog("Bufsize overflow while commenting line - abort\n","");
          return false;
          }
 
@@ -791,15 +789,15 @@ for (ip = *list; ip != NULL; ip=ip->next)
          continue;
          }
 
-      EditVerbose("Edit: Commenting %s%s%s\n",comm,ip->name,end);
+      EditVerbose("Commenting %s%s%s\n",comm,ip->name,end);
 
       sprintf(buff,"%s%s%s",comm,ip->name,end);
       free(ip->name);
 
       if ((ip->name = malloc(strlen(buff)+1)) == NULL)
          {
-         printf("cfengine: malloc in CommentItemContaining\n ");
-         exit(1);
+         CfLog(cferror,"malloc in CommentItemContaining\n","");
+         FatalError("");;
          }
 
       strcpy(ip->name,buff);
@@ -834,8 +832,8 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   sprintf(OUTPUT,"%s\n",err);
    return false;
    }
 
@@ -851,7 +849,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
       {
       if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
          {
-         printf("cfengine: bufsize overflow while commenting line - abort\n");
+         CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
          return false;
          }
 
@@ -860,15 +858,15 @@ for (ip = *list; ip != NULL; ip=ip->next)
          continue;
          }
 
-      EditVerbose("Edit: Commenting %s%s%s\n",comm,ip->name,end);
+      EditVerbose("Commenting %s%s%s\n",comm,ip->name,end);
 
       sprintf(buff,"%s%s%s",comm,ip->name,end);
       free(ip->name);
 
       if ((ip->name = malloc(strlen(buff)+1)) == NULL)
          {
-         printf("cfengine: malloc in CommentItemContaining\n ");
-         exit(1);
+         CfLog(cferror,"malloc in CommentItemContaining\n ","");
+         FatalError("");;
          }
 
       strcpy(ip->name,buff);
@@ -903,8 +901,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");   
    return false;
    }
 
@@ -914,7 +914,7 @@ for (ip = *list; ip != NULL; ip=ip->next)
       {
       if (strlen(ip->name)+strlen(comm)+strlen(end)+2 > bufsize)
          {
-         printf("%s: bufsize overflow while commenting line - abort\n",VPREFIX);
+         CfLog(cferror,"Bufsize overflow while commenting line - abort\n","");
          return false;
          }
 
@@ -924,13 +924,12 @@ for (ip = *list; ip != NULL; ip=ip->next)
          continue;
          }
 
-      EditVerbose("Edit: uncomment line %s\n",ip->name);
+      EditVerbose("Uncomment line %s\n",ip->name);
       CURRENTLINEPTR = ip->next;
 
       if ((sp = malloc(strlen(ip->name)+2)) == NULL)
          {
-         printf("cfengine: couldn't allocate memory in UnCommentNLines\n");
-         perror("malloc");
+         CfLog(cferror,"No Memory in UnCommentNLines\n","malloc");
          return false;
          }
 
@@ -996,13 +995,12 @@ for (ip = *list; ip != NULL; ip=ip->next)
          continue;
          }
 
-      EditVerbose("Edit: uncomment line %s\n",ip->name);
+      EditVerbose("Uncomment line %s\n",ip->name);
       CURRENTLINEPTR = ip->next;
 
       if ((sp = malloc(strlen(ip->name)+2)) == NULL)
          {
-         printf("cfengine: couldn't allocate memory in UnCommentNLines\n");
-         perror("malloc");
+         CfLog(cferror,"No memory in UnCommentNLines\n","malloc");
          return false;
          }
 
@@ -1068,7 +1066,7 @@ Debug2("CommentToRegExp(list,%s %s)\n",comm,string);
 
 if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    {
-   printf("cfengine: File line-pointer undefined during editfile action\n");
+   CfLog(cferror,"File line-pointer undefined during editfile action\n","");
    return true;
    }
 
@@ -1079,8 +1077,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(string,strlen(string),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,string);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",string);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");   
    return false;
    }
 
@@ -1094,15 +1094,14 @@ for (ip = CURRENTLINEPTR; (ip != NULL); ip=ip->next)
       
    if ((match = re_match(&buf,ip->name,strlen(ip->name),0,&regs)) == strlen(ip->name))
       {
-      Debug2("REGEXP matched [%s] to [%s]\n",ip->name,string);
-      Debug2("cfengine: (Found a match in file)\n");
       linefound = true;
       ip_end = ip;
       break;
       }
    else if (match < -1)
       {
-      printf("%s: Regular expression internal error [%s]\n",VPREFIX,string); 
+      sprintf(OUTPUT,"Regular expression internal error [%s]\n",string);
+      CfLog(cferror,OUTPUT,"");
       }
 
    }
@@ -1118,18 +1117,17 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
    {
    if (ip == ip_end)
       {
-      EditVerbose("Edit: terminating line: %s (Done)\n",ip->name);
+      EditVerbose("Terminating line: %s (Done)\n",ip->name);
       done = true;
       }
 
-   EditVerbose("Edit: comment line %s%s%s\n",comm,ip->name, end);
+   EditVerbose("Comment line %s%s%s\n",comm,ip->name, end);
    NUMBEROFEDITS++;
    CURRENTLINEPTR = ip->next;
 
    if ((sp = malloc(strlen(ip->name)+strlen(comm)+strlen(end)+2)) == NULL)
       {
-      printf("cfengine: couldn't allocate memory in CommentToRegExp\n");
-      perror("malloc");
+      CfLog(cferror,"No memory in CommentToRegExp\n","malloc");
       return false;
       }
 
@@ -1167,14 +1165,15 @@ sscanf(string,"%d", &N);
 
 if (N < 1)
    {
-   printf("%s: Illegal number value in DeleteNLines: %s\n",VPREFIX,string);
+   sprintf(OUTPUT,"Illegal number value in DeleteNLines: %s\n",string);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
 
 if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    {
-   printf("cfengine: File line-pointer undefined during editfile action\n");
+   CfLog(cferror,"File line-pointer undefined during editfile action\n","");
    return true;
    }
 
@@ -1194,11 +1193,11 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
       
    if (ctr == N)
       {
-      EditVerbose("Edit: terminating line: %s (Done)\n",ip->name);
+      EditVerbose("Terminating line: %s (Done)\n",ip->name);
       done = true;
       }
 
-   EditVerbose("Edit: delete line %s\n",ip->name);
+   EditVerbose("Delete line %s\n",ip->name);
    NUMBEROFEDITS++;
    CURRENTLINEPTR = ip->next;
 
@@ -1223,10 +1222,8 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
 
 if (ctr-1 < N)
    {
-   if (!SILENT)
-      {
-      printf("%s: DeleteNLines deleted only %d lines (not %d)\n",VPREFIX,ctr-1,N);
-      }
+   sprintf(OUTPUT,"DeleteNLines deleted only %d lines (not %d)\n",ctr-1,N);
+   CfLog(cfsilent,OUTPUT,"");
    }
 
 return true;
@@ -1250,7 +1247,6 @@ for (ip = list; ip != NULL && ip->next != NULL; ip=ip->next)
 
 CURRENTLINEPTR = ip;
 
-Debug2("Last line (%d): [%s]\n",CURRENTLINENUMBER,ip->name);
 return ip;
 }
 
@@ -1274,8 +1270,8 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(regexp,strlen(regexp),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,regexp);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",regexp);
+   sprintf(OUTPUT,"%s\n",err);
    return false;
    }
 
@@ -1287,7 +1283,8 @@ if ((match = re_match(&buf,line,strlen(line),0,&regs)) == strlen(line))
    }
 else if (match < -1)
    {
-   printf("cfengine: Regular expression internal error [%s]\n",regexp); 
+   sprintf(OUTPUT,"Regular expression internal error [%s]\n",regexp);
+   CfLog(cferror,OUTPUT,"");
    }
 
 return false;
@@ -1310,7 +1307,7 @@ char *search, *replace;
   int match;
   char *err;
   
-EditVerbose("Edit: checking for replace/%s/%s\n",search,replace);
+EditVerbose("Checking for replace/%s/%s\n",search,replace);
 buf.allocated = 0;
 buf.buffer = NULL;
 buf.fastmap = fastmap;
@@ -1318,8 +1315,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(search,strlen(search),&buf)) != 0)
    {
-   printf("%s: Regular expression error for %s\n",VPREFIX,search);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for %s\n",search);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
@@ -1337,7 +1336,8 @@ for (ip = *liststart; ip != NULL; ip=ip->next)
       }
    else if (match < -1)
       {
-      printf ("%s: Error in regular expression [%s]!\n",VPREFIX,search);
+      sprintf(OUTPUT,"Error in regular expression [%s]!\n",search);
+      CfLog(cferror,OUTPUT,"");
       }
 
    bzero(VBUFF,bufsize);
@@ -1367,7 +1367,8 @@ for (ip = *liststart; ip != NULL; ip=ip->next)
             }
           else if (match < -1)
             {
-            printf ("cfengine: ERROR in regular expression [%s]!\n",search);
+            sprintf(OUTPUT,"ERROR in regular expression [%s]!\n",search);
+	    CfLog(cferror,OUTPUT,"");
             }
          }
 
@@ -1405,14 +1406,16 @@ sscanf(string,"%d", &N);
 
 if (N < 1)
    {
-   printf("%s: Illegal number value in CommentNLines: %s\n",VPREFIX,string);
+   sprintf(OUTPUT,"Illegal number value in CommentNLines: %s\n",string);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
 
 if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    {
-   printf("%s: File line-pointer undefined during editfile action\n",VPREFIX);
+   sprintf(OUTPUT,"File line-pointer undefined during editfile action\n");
+   CfLog(cferror,OUTPUT,"");
    return true;
    }
 
@@ -1427,7 +1430,7 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
    
    if (ctr == N)
       {
-      EditVerbose("Edit: terminating line: %s (Done)\n",ip->name);
+      EditVerbose("Terminating line: %s (Done)\n",ip->name);
       done = true;
       }
 
@@ -1442,14 +1445,13 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
       continue;
       }
 
-   EditVerbose("Edit: comment line %s\n",ip->name);
+   EditVerbose("Comment line %s\n",ip->name);
    NUMBEROFEDITS++;
    CURRENTLINEPTR = ip->next;
 
    if ((sp = malloc(strlen(ip->name)+strlen(comm)+strlen(end)+2)) == NULL)
       {
-      printf("cfengine: couldn't allocate memory in CommentNLines\n");
-      perror("malloc");
+      CfLog(cferror,"No memory in CommentNLines\n","malloc");
       return false;
       }
 
@@ -1469,10 +1471,8 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
 
 if (ctr-1 < N)
    {
-   if (!SILENT)
-      {
-      printf("%s: CommentNLines commented only %d lines (not %d)\n",VPREFIX,ctr-1,N);
-      }
+   sprintf(OUTPUT,"CommentNLines commented only %d lines (not %d)\n",ctr-1,N);
+   CfLog(cfinform,OUTPUT,"");
    }
 
 return true;
@@ -1497,14 +1497,16 @@ sscanf(string,"%d", &N);
 
 if (N < 1)
    {
-   printf("%s: Illegal number value in CommentNLines: %s\n",VPREFIX,string);
+   sprintf(OUTPUT,"Illegal number value in CommentNLines: %s\n",string);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
 
 if (CURRENTLINEPTR == NULL)  /* Shouldn't happen */
    {
-   printf("%s: File line-pointer undefined during editfile action\n",VPREFIX);
+   sprintf(OUTPUT,"File line-pointer undefined during editfile action\n");
+   CfLog(cferror,OUTPUT,"");
    return true;
    }
 
@@ -1519,7 +1521,7 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
    
    if (ctr == N)
       {
-      EditVerbose("Edit: terminating line: %s (Done)\n",ip->name);
+      EditVerbose("Terminating line: %s (Done)\n",ip->name);
       done = true;
       }
 
@@ -1530,13 +1532,12 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
       continue;
       }
 
-   EditVerbose("Edit: uncomment line %s\n",ip->name);
+   EditVerbose("Uncomment line %s\n",ip->name);
    CURRENTLINEPTR = ip->next;
 
    if ((sp = malloc(strlen(ip->name)+2)) == NULL)
       {
-      printf("cfengine: couldn't allocate memory in UnCommentNLines\n");
-      perror("malloc");
+      CfLog("No memory in UnCommentNLines\n","malloc");
       return false;
       }
 
@@ -1585,10 +1586,8 @@ for (ip = CURRENTLINEPTR; ip != NULL; ip = CURRENTLINEPTR)
 
 if (ctr-1 < N)
    {
-   if (!SILENT)
-      {
-      printf("%s: CommentNLines commented only %d lines (not %d)\n",VPREFIX,ctr-1,N);
-      }
+   sprintf(OUTPUT,"CommentNLines commented only %d lines (not %d)\n",ctr-1,N);
+   CfLog(cfinform,OUTPUT,"");
    }
 
 return true;
@@ -1615,20 +1614,21 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(regex,strlen(regex),&buf)) != 0)
    {
-   printf("%s: Regular expression error for AbortAtLineMatching %s\n",VPREFIX,regex);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for AbortAtLineMatching %s\n",regex);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return true;
    }
 
 if ((match = re_match(&buf,item,strlen(item),0,&regs)) == strlen(item))
    {
-   Debug2("REGEXP matched [%s] to [%s]\n",item,regex);
-   Debug2("cfengine: (Found a match in file)\n");
    return true;
    }
 else if (match < -1)
    {
-   printf("%s: Regular expression internal error for AbortAtLineMatchings [%s]\n",VPREFIX,regex);
+   sprintf(OUTPUT,"Regular expression internal error for AbortAtLineMatchings [%s]\n",regex);
+   CfLog(cferror,OUTPUT,"");
    return true;  /* for safety */
    }
 
@@ -1651,7 +1651,7 @@ Debug("ReplaceWithFieldMatch(%s,%s,%c)\n",field,replace,split);
 
 if ((replace == NULL) || (strlen(replace) == 0))
    {
-   EditVerbose("Edit: Ignoring empty line which doing ReplaceLinesMatchingField\n");
+   EditVerbose("Ignoring empty line which doing ReplaceLinesMatchingField\n");
    return;
    }
   
@@ -1681,8 +1681,9 @@ for (sp = replace; *sp != '\0'; sp++)
 
 if (fcount < matching_field)
    {
-   Silent("%s: File formats did not completely match in ReplaceLinesMatchingField\n",VPREFIX);
-   Silent("%s: while editing %s\n",VPREFIX,filename);
+   CfLog(cfsilent,"File formats did not completely match in ReplaceLinesMatchingField\n","");
+   sprintf(OUTPUT,"while editing %s\n",filename);
+   CfLog(cfsilent,OUTPUT,"");
    return;
    }
 
@@ -1746,7 +1747,7 @@ for (ip = *filestart; ip != NULL; ip=ip->next, linenum++)
 
    if (strcmp(linefield,match) == 0)
       {
-      EditVerbose("Edit:   Replacing line %d (key %s)\n",linenum,match);
+      EditVerbose("Replacing line %d (key %s)\n",linenum,match);
 
       count++;
 
@@ -1757,7 +1758,8 @@ for (ip = *filestart; ip != NULL; ip=ip->next, linenum++)
 
       if (count > 1)
 	 {
-	 Silent("%s: Several lines in %s matched key %s\n",VPREFIX,filename,match);
+	 sprintf(OUTPUT,"Several lines in %s matched key %s\n",filename,match);
+	 CfLog(cfsilent,OUTPUT,"");
 	 }
 
       NUMBEROFEDITS++;
