@@ -46,25 +46,29 @@ char *file;
 
 if (stat(file,&statbuf) == -1)
    {
-   printf("cfengine: Couldn't stat %s\n",file);
+   sprintf(OUTPUT,"Couldn't stat %s\n",file);
+   CfLog(cferror,OUTPUT,"stat");
    return false;
    }
 
 if ((EDITFILESIZE != 0) &&(statbuf.st_size > EDITFILESIZE))
    {
-   printf("%s: File %s is bigger than the limit <editfilesize>\n",VPREFIX,file);
+   sprintf(OUTPUT,"File %s is bigger than the limit <editfilesize>\n",file);
+   CfLog(cfinform,OUTPUT,"");
    return(false);
    }
 
 if (! S_ISREG(statbuf.st_mode))
    {
-   printf("%s: %s is not a plain file\n",VPREFIX,file);
+   sprintf(OUTPUT,"%s is not a plain file\n",file);
+   CfLog(cfinform,OUTPUT,"");
    return false;
    }
 
 if ((fp = fopen(file,"r")) == NULL)
    {
-   printf("%s: Couldn't read file %s for editing\n",VPREFIX,file);
+   sprintf(OUTPUT,"Couldn't read file %s for editing\n",file);
+   CfLog(cferror,OUTPUT,"fopen");
    return false;
    }
 
@@ -76,12 +80,7 @@ for (i = 0; i < bufsize; i++)
 
 while(!feof(fp))
    {
-   fgets (VBUFF,bufsize,fp);
-
-   if(VBUFF[strlen(VBUFF)-1] == '\n')
-      {
-      VBUFF[strlen(VBUFF)-1] = '\0';        /* chop */
-      }
+   ReadLine(VBUFF,bufsize,fp);
 
    if (!feof(fp) || strlen(VBUFF) != 0) 
       {
@@ -108,8 +107,9 @@ char *file;
 
 if (stat(file,&statbuf) == -1)
    {
-   printf("%s: Couldn't stat %s, which needed editing!\n",VPREFIX,file);
-   printf("%s: Check definition in program - is file NFS mounted?\n\n",VPREFIX);
+   sprintf(OUTPUT,"Couldn't stat %s, which needed editing!\n",file);
+   CfLog(cferror,OUTPUT,"");
+   CfLog(cferror,"Check definition in program - is file NFS mounted?\n\n","");
    return false;
    }
 
@@ -120,8 +120,8 @@ if (! IsItemIn(VREPOSLIST,VBUFF))
    {
    if (rename(file,VBUFF) == -1)
       {
-      printf("cfengine: Error occurred while renaming %s\n",file);
-      perror("rename ");
+      sprintf(OUTPUT,"Error while renaming %s\n",file);
+      CfLog(cferror,OUTPUT,"rename ");
       return false;
       }
    else if (Repository(VBUFF))
@@ -132,8 +132,9 @@ if (! IsItemIn(VREPOSLIST,VBUFF))
 
 if ((fp = fopen(file,"w")) == NULL)
    {
-   printf("%s: Couldn't write file %s after editing\n",VPREFIX,file);
+   sprintf(OUTPUT,"Couldn't write file %s after editing\n",file);
    rename(VBUFF,file);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
@@ -142,7 +143,8 @@ for (ip = liststart; ip != NULL; ip=ip->next)
    fprintf(fp,"%s\n",ip->name);
    }
 
-printf("%s: Edited file %s \n",VPREFIX,file);
+sprintf(OUTPUT,"Edited file %s \n",file);
+CfLog(cfinform,OUTPUT,""); 
 
 fclose(fp);
 chmod(file,statbuf.st_mode);                    /* Restore file permissions etc */
