@@ -32,11 +32,13 @@
 /* TOOLKIT : links                                                   */
 /*********************************************************************/
 
-LinkChildFiles(from,to,type,inclusions,exclusions,copy)
+LinkChildFiles(from,to,type,inclusions,exclusions,copy,nofile,ptr)
 
 char *from, *to;
 char type;
 struct Item *inclusions, *exclusions, *copy;
+short nofile;
+struct Link *ptr;
 
 { DIR *dirh;
   struct dirent *dirp;
@@ -44,6 +46,8 @@ struct Item *inclusions, *exclusions, *copy;
   struct stat statbuf;
   int LinkFiles(), HardLinkFiles(), AbsoluteLink(), RelativeLink();
   int (*linkfiles)();
+
+Debug("LinkChildFiles(%s,%s)\n",from,to);
   
 if (stat(to,&statbuf) == -1)
    {
@@ -52,7 +56,8 @@ if (stat(to,&statbuf) == -1)
 
 if ((dirh = opendir(to)) == NULL)
    {
-   printf("%s: Can't open directory %s\n",VPREFIX,to);
+   sprintf(OUTPUT,"Can't open directory %s\n",to);
+   CfLog(cferror,OUTPUT,"opendir");
    return false;
    }
 
@@ -96,11 +101,11 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
                 linkfiles = HardLinkFiles;
                 break;
       default:
-                printf("cfengine: internal error, link type was [%c]\n",type);
+                printf("Internal error, link type was [%c]\n",type);
                 continue;
       }
    
-   (*linkfiles)(pcwdfrom,pcwdto,inclusions,exclusions,copy);
+   (*linkfiles)(pcwdfrom,pcwdto,inclusions,exclusions,copy,nofile,ptr);
    }
 
 closedir(dirh);
@@ -109,7 +114,7 @@ return true;
 
 /*********************************************************************/
 
-LinkChildren(path,type,rootstat,uid,gid,inclusions,exclusions,copy)
+LinkChildren(path,type,rootstat,uid,gid,inclusions,exclusions,copy,nofile,ptr)
 
 
 /* --------------------------------------------------------------------
@@ -130,6 +135,8 @@ struct stat *rootstat;
 uid_t uid;
 gid_t gid;
 struct Item *inclusions, *exclusions, *copy;
+short nofile;
+struct Link *ptr;
 
 { char *sp;
   char lastlink[bufsize],server[bufsize],from[bufsize],to[bufsize],relpath[bufsize];
@@ -141,10 +148,12 @@ struct Item *inclusions, *exclusions, *copy;
   int LinkFiles(), HardLinkFiles(), AbsoluteLink(), RelativeLink();
   int (*linkfiles)();
 
-
+Debug("LinkChildren(%s)\n",path);
+  
 if (! S_ISDIR(rootstat->st_mode))
    {
-   printf("%s: File %s is not a directory: it has no children to link!\n",VPREFIX,path);
+   sprintf(OUTPUT,"File %s is not a directory: it has no children to link!\n",path);
+   CfLog(cferror,OUTPUT,"");
    return;
    }
 
@@ -169,7 +178,8 @@ for (sp = path+strlen(path); sp != path-1; sp--)
 
          if ((dirh = opendir(odir)) == NULL)
             {
-            printf("%s: Can't open directory %s\n",VPREFIX,path);
+            sprintf(OUTPUT,"Can't open directory %s\n",path);
+	    CfLog(cferror,OUTPUT,"opendir");
             return;
             }
 
@@ -223,11 +233,12 @@ for (sp = path+strlen(path); sp != path-1; sp--)
                             linkfiles = HardLinkFiles;
                             break;
                   default:
-                            printf("cfengine: internal error, link type was [%c]\n",type);
+                            sprintf(OUTPUT,"Internal error, link type was [%c]\n",type);
+			    CfLog(cferror,OUTPUT,"");
                             continue;
                   }
 
-	       matched = (*linkfiles)(from,to,inclusions,exclusions,copy);
+	       matched = (*linkfiles)(from,to,inclusions,exclusions,copy,nofile,ptr);
 
                if (matched && !DONTDO)
 		 {
@@ -241,8 +252,10 @@ for (sp = path+strlen(path); sp != path-1; sp--)
       }
    }
 
-printf("%s: Couldn't link the children of %s to anything because no\n",VPREFIX,path);
-printf("%s: file system was found to mirror it in the defined binservers list.\n",VPREFIX);
+sprintf(OUTPUT,"Couldn't link the children of %s to anything because no\n",path);
+CfLog(cferror,OUTPUT,""); 
+sprintf(OUTPUT,"file system was found to mirror it in the defined binservers list.\n");
+CfLog(cferror,OUTPUT,""); 
 }
 
 /*********************************************************************/
@@ -270,7 +283,7 @@ if (maxrecurse == 0)  /* reached depth limit */
 
 if (IgnoreFile(to,""))
    {
-   Verbose("cfengine: Ignoring directory %s\n",from);
+   Verbose("%s: Ignoring directory %s\n",VPREFIX,from);
    return false;
    }
 
@@ -283,7 +296,8 @@ bug_check = lp->next;
 
 if ((dirh = opendir(to)) == NULL)
    {
-   printf("%s: Can't open directory [%s]\n",VPREFIX,from);
+   sprintf(OUTPUT,"Can't open directory [%s]\n",from);
+   CfLog(cferror,OUTPUT,"opendir");
    return false;
    }
 
@@ -313,7 +327,6 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
    if (BufferOverflow(newfrom,dirp->d_name))
       {
-      printf(" culprit: RecursiveLink\n");
       closedir(dirh);
       return true;
       }
@@ -322,7 +335,6 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
 
    if (BufferOverflow(newto,dirp->d_name))
       {
-      printf(" culprit: RecursiveLink\n");
       closedir(dirh);
       return true;
       }
@@ -333,7 +345,8 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
       if (stat(newto,&statbuf) == -1)
          {
-         Verbose("cfengine: Can't stat %s\n",newto);
+         sprintf(OUTPUT,"Can't stat %s\n",newto);
+	 CfLog(cfverbose,OUTPUT,"");
          continue;
          }
       }
@@ -341,11 +354,13 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       {
       if (lstat(newto,&statbuf) == -1)
          {
-         Verbose("cfengine: Can't stat %s\n",newto);
+         sprintf(OUTPUT,"Can't stat %s\n",newto);
+	 CfLog(cfverbose,OUTPUT,"");
+	 
 	 bzero(VBUFF,bufsize);
          if (readlink(newto,VBUFF,bufsize) != -1)
             {
-            Verbose("          File is link to -> %s\n",VBUFF);
+            Verbose("File is link to -> %s\n",VBUFF);
             }
          continue;
          }
@@ -376,7 +391,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
                    continue;
 	 }
 
-      (*linkfiles)(newfrom,newto,lp->inclusions,lp->exclusions,lp->copy);
+      (*linkfiles)(newfrom,newto,lp->inclusions,lp->exclusions,lp->copy,lp->nofile,lp);
       }
    }
 
@@ -386,10 +401,12 @@ return true;
 
 /*********************************************************************/
 
-LinkFiles(from,to_tmp,inclusions,exclusions,copy)  /* should return true if 'to' found */
+LinkFiles(from,to_tmp,inclusions,exclusions,copy,nofile,ptr)  /* should return true if 'to' found */
 
 char *from, *to_tmp;
 struct Item *inclusions, *exclusions, *copy;
+short nofile;
+struct Link *ptr;
 
 { struct stat buf;
   char to[bufsize],linkbuf[bufsize], saved[bufsize], absto[bufsize], *sp, *lastnode;
@@ -405,7 +422,7 @@ if ((*to_tmp != '/') && (*to_tmp != '.'))  /* links without a directory referenc
 
 if (strlen(to_tmp)+3 > bufsize)
    {
-   printf("cfengine:%s: bufsize boundaries exceeded in LinkFiles(%s->%s)\n",VPREFIX,from,to_tmp);
+   printf("%s: bufsize boundaries exceeded in LinkFiles(%s->%s)\n",VPREFIX,from,to_tmp);
    return false;
    }
 
@@ -421,7 +438,7 @@ lastnode++;
 
 if (IgnoredOrExcluded(links,lastnode,inclusions,exclusions))
    {
-   Verbose("cfengine: skipping non-included pattern %s\n",from);
+   Verbose("%s: Skipping non-included pattern %s\n",VPREFIX,from);
    return true;
    }
 
@@ -441,23 +458,17 @@ if (IsWildItemIn(VCOPYLINKS,lastnode) || IsWildItemIn(copy,lastnode))
    ip.inclusions = NULL;
    ip.symlink = NULL;
    ip.classes = NULL;
-   Verbose("cfengine: link item %s marked for copying instead\n",from);
+   Verbose("%s: Link item %s marked for copying instead\n",VPREFIX,from);
    MakeDirectoriesFor(to);
    CheckImage(to,from,&ip);
    return true;
    }
 
-if (*to == '.')         /* relative path, must still check if exists */
+if (*to != '/')         /* relative path, must still check if exists */
    {
-   strcpy(absto,from);
-   ChopLastNode(absto);   /* Get the path where WE want to make link */
-   AddSlash(absto);
-
-   if (BufferOverflow(absto,to))
-      {
-      FatalError("culprit: LinkFiles()");
-      }
-   strcat(absto,to);
+   Debug("Relative link destination detected: %s\n",to);
+   strcpy(absto,AbsLinkPath(from,to));
+   Debug("Absolute path to relative link = %s, from %s\n",absto,from);
    }
 else
    {
@@ -466,7 +477,14 @@ else
 
 if (stat(absto,&buf) == -1)
    {
-   return(false);  /* no error warning, since the higher level routine uses this */
+   if (nofile)
+      {
+      Verbose("%s: (Force non-existent file link)\n",VPREFIX);
+      }
+   else
+      {
+      return(false);  /* no error warning, since the higher level routine uses this */
+      }
    }
 
 Debug2("Trying to link %s -> %s (%s)\n",from,to,absto);
@@ -475,14 +493,17 @@ if (lstat(from,&buf) == 0)
    {
    if (! S_ISLNK(buf.st_mode) && ! ENFORCELINKS)
       {
-      Silent("%s: Error linking %s -> %s\n",VPREFIX,from,to);
-      Silent("%s: Cannot make link: %s exists and is not a link! (uid %d)\n",VPREFIX,from,buf.st_uid);
+      Silent(OUTPUT,"Error linking %s -> %s\n",from,to);
+      CfLog(cfsilent,OUTPUT,"");
+      Silent(OUTPUT,"Cannot make link: %s exists and is not a link! (uid %d)\n",from,buf.st_uid);
+      CfLog(cfsilent,OUTPUT,"");
       return(true);
       }
 
    if (S_ISREG(buf.st_mode) && ENFORCELINKS)
       {
-      Silent("%s: moving %s to %s.%s\n",VPREFIX,from,from,CF_SAVED);
+      sprintf(OUTPUT,"Moving %s to %s.%s\n",from,from,CF_SAVED);
+      CfLog(cfsilent,OUTPUT,"");
 
       if (DONTDO)
          {
@@ -495,7 +516,8 @@ if (lstat(from,&buf) == 0)
 
       if (rename(from,saved) == -1)
          {
-         perror("rename");
+         sprintf(OUTPUT,"Can't rename %s to %s\n",from,saved);
+	 CfLog(cferror,OUTPUT,"rename");
          return(true);
          }
 
@@ -512,18 +534,32 @@ if (readlink(from,linkbuf,bufsize) == -1)
    {
    if (! MakeDirectoriesFor(from))                  /* link doesn't exist */
       {
-      Silent("%s: Couldn't build directory tree up to %s!\n",VPREFIX,from);
-      Silent("%s: One element was a plain file, not a directory!\n",VPREFIX);
+      sprintf(OUTPUT,"Couldn't build directory tree up to %s!\n",from);
+      CfLog(cfsilent,OUTPUT,"");
+      sprintf(OUTPUT,"One element was a plain file, not a directory!\n");
+      CfLog(cfsilent,OUTPUT,"");      
       return(true);
       }
    }
 else
-   {
-   if (strcmp(linkbuf,to) != 0)
+   { int off1 = 0, off2 = 0;
+   
+   if (strncmp(linkbuf,"./",2) == 0)   /* Ignore ./ at beginning */
+      {
+      off1 = 2;
+      }
+
+   if (strncmp(to,"./",2) == 0)
+      {
+      off2 = 2;
+      }
+   
+   if (strcmp(linkbuf+off1,to+off2) != 0)
       {
       if (ENFORCELINKS)
          {
-         Silent("%s: remove link %s\n",VPREFIX,from);
+         sprintf(OUTPUT,"Removing link %s\n",from);
+	 CfLog(cfsilent,OUTPUT,"");
 
          if (!DONTDO)
             {
@@ -533,35 +569,44 @@ else
                return true;
                }
 
-            DoLink(from,to);
+            DoLink(from,to,ptr->defines);
             return true;
             }
          }
       else
          {
-         Silent("%s: Warning old link %s points somewhere else. Doing nothing!\n",VPREFIX,from);
-         Silent("%s: (Link points to %s not %s)\n\n",VPREFIX,linkbuf,to);
+         sprintf(OUTPUT,"Old link %s points somewhere else. Doing nothing!\n",from);
+	 CfLog(cfsilent,OUTPUT,"");
+         sprintf(OUTPUT,"(Link points to %s not %s)\n\n",linkbuf,to);
+	 CfLog(cfsilent,OUTPUT,"");	 
          return(true);
          }
       }
    else
       {
-      Verbose("cfengine: Link (%s->%s) exists and is okay.\n",from,to_tmp);
-      KillOldLink(from);             /* Check whether link points somewhere */
+      sprintf(OUTPUT,"Link (%s->%s) exists.\n",from,to_tmp);
+      CfLog(cfverbose,OUTPUT,"");
+
+      if (!nofile)
+	 {
+	 KillOldLink(from);             /* Check whether link points somewhere */
+	 }
       return(true);
       }
    }
 
-DoLink(from,to);
+DoLink(from,to,ptr->defines);
 return(true);
 }
 
 /*********************************************************************/
 
-RelativeLink(from,to,inclusions,exclusions,copy)
+RelativeLink(from,to,inclusions,exclusions,copy,nofile,ptr)
 
 char *from, *to;
 struct Item *inclusions, *exclusions,*copy;
+short nofile;
+struct Link *ptr;
 /* global char LINKTO[] */
 
 { char *sp, *commonto, *commonfrom, *lastnode;
@@ -572,12 +617,13 @@ Debug2("RelativeLink(%s,%s)\n",from,to);
 
 if (*to == '.')
    {
-   return LinkFiles(from,to,inclusions,exclusions,copy);
+   return LinkFiles(from,to,inclusions,exclusions,copy,nofile,ptr);
    }
 
 if (!CompressPath(LINKTO,to))
    {
-   printf("%s: failed to link %s to %s\n",VPREFIX,from,to);
+   sprintf(OUTPUT,"Failed to link %s to %s\n",from,to);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
@@ -586,8 +632,9 @@ commonfrom = from;
 
 if (strcmp(commonto,commonfrom) == 0)
    {
-   printf("%s: can't link file to itself!\n",VPREFIX);
-   printf("%s: (%s -> %s)\n",VPREFIX,from,to);
+   CfLog(cferror,"Can't link file to itself!\n","");
+   sprintf(OUTPUT,"(%s -> %s)\n",from,to);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 
@@ -597,6 +644,16 @@ while (*commonto == *commonfrom)
    commonfrom++;
    }
 
+while (!((*commonto == '/') && (*commonfrom == '/')))
+   {
+   commonto--;
+   commonfrom--;
+   }
+
+commonto++; 
+
+Debug("Commonto = %s, common from = %s\n",commonto,commonfrom); 
+
 for (sp = commonfrom; *sp != '\0'; sp++)
    {
    if (*sp == '/')
@@ -605,37 +662,40 @@ for (sp = commonfrom; *sp != '\0'; sp++)
        }
    }
 
+Debug("LEVELS = %d\n",levels);
+ 
 bzero(buff,bufsize);
 
 strcat(buff,"./");
 
-while(levels-- > 0)
+while(--levels > 0)
    {
    if (BufferOverflow(buff,"../"))
       {
-      printf("cfengine: culprit RelativeLink()\n");
       return false;
       }
+   
    strcat(buff,"../");
    }
 
 if (BufferOverflow(buff,commonto))
    {
-   printf("cfengine: culprit RelativeLink()\n");
    return false;
    }
 
 strcat(buff,commonto);
-
-return LinkFiles(from,buff,inclusions,exclusions,copy);
+ 
+return LinkFiles(from,buff,inclusions,exclusions,copy,nofile,ptr);
 }
 
 /*********************************************************************/
 
-AbsoluteLink(from,to,inclusions,exclusions,copy)
+AbsoluteLink(from,to,inclusions,exclusions,copy,nofile,ptr)
 
 char *from, *to;
 struct Item *inclusions,*exclusions,*copy;
+short nofile;
+struct Link *ptr;
 /* global LINKTO */
 
 { char absto[bufsize];
@@ -661,8 +721,9 @@ expand[0] = '\0';
 
 if (!ExpandLinks(expand,absto,0))  /* begin at level 1 and beam out at 15 */
    {
-   printf("%s: failed to make absolute link in\n",VPREFIX);
-   printf("%s: %s -> %s\n",VPREFIX,from,to);
+   CfLog(cferror,"Failed to make absolute link in\n","");
+   sprintf("%s: %s -> %s\n",from,to);
+   CfLog(cferror,OUTPUT,"");
    return false;
    }
 else
@@ -672,14 +733,14 @@ else
 
 CompressPath(LINKTO,expand);
 
-return LinkFiles(from,LINKTO,inclusions,exclusions,copy);
+return LinkFiles(from,LINKTO,inclusions,exclusions,copy,nofile,ptr);
 }
 
 /*********************************************************************/
 
-DoLink (from,to)
+DoLink (from,to,defines)
 
-char *from, *to;
+char *from, *to, *defines;
 
 {
 if (DONTDO)
@@ -688,18 +749,23 @@ if (DONTDO)
    }
 else
    {
-   Silent("%s: Linking files %s -> %s\n\n",VPREFIX,from,to);
+   sprintf(OUTPUT,"Linking files %s -> %s\n\n",from,to);
+   CfLog(cfsilent,OUTPUT,"");
 
    if (symlink(to,from) == -1)
       {
       perror("symlink");
       }
+   else
+      {
+      AddMultipleClasses(defines);
+      }
    }
 }
 
 /*********************************************************************/
 
-KillOldLink (name)
+KillOldLink(name)
 
 char *name;
 
@@ -708,14 +774,14 @@ char *name;
   struct stat statbuf;
   short i;
 
-for (i = 0; i < bufsize; i++)
-   {
-   linkbuf[i] = linkpath[i] = '\0';
-   }
+Debug("KillOldLink(%s)\n",name);
+bzero(linkbuf,bufsize);
+bzero(linkpath,bufsize); 
 
 if (readlink(name,linkbuf,bufsize) == -1)
    {
-   printf("cfengine: KillOldLink() contradiction! Can't read existing link!\n");
+   sprintf(OUTPUT,"(Can't read link %s while checking for deadlinks)\n",name);
+   CfLog(cfverbose,OUTPUT,"");
    return;
    }
 
@@ -730,18 +796,20 @@ if (linkbuf[0] != '/')
    }
 
 strcat(linkpath,linkbuf);
-
-if (stat(linkpath,&statbuf) == -1)               /* link points nowhere */
+CompressPath(VBUFF,linkpath); 
+ 
+if (stat(VBUFF,&statbuf) == -1)               /* link points nowhere */
    {
    if (KILLOLDLINKS || DEBUG || D2)
       {
-      Silent("%s: %s is a link which points to %s\n",VPREFIX,name,linkpath);
-      Silent("%s: but that file doesn't exist!\n",VPREFIX);
+      sprintf(OUTPUT,"%s is a link which points to %s, but that file doesn't seem to exist\n",name,VBUFF);
+      CfLog(cfsilent,OUTPUT,"");
       }
 
-   if (KILLOLDLINKS || VERBOSE || DEBUG || D2)
+   if (KILLOLDLINKS)
       {
-      printf("%s: removing dead link %s\n",VPREFIX,name);
+      sprintf(OUTPUT,"Removing dead link %s\n",name);
+      CfLog(cfsilent,OUTPUT,"");
 
       if (! DONTDO)
          {
@@ -753,10 +821,11 @@ if (stat(linkpath,&statbuf) == -1)               /* link points nowhere */
 
 /*********************************************************************/
 
-HardLinkFiles(from,to,inclusions,exclusions,copy)  /* should return true if 'to' found */
+HardLinkFiles(from,to,inclusions,exclusions,copy,nofile,ptr)  /* should return true if 'to' found */
 
 char *from, *to;
 struct Item *inclusions,*exclusions,*copy;
+struct Link *ptr;
 
 { struct stat frombuf,tobuf;
   char saved[bufsize], *lastnode;
@@ -771,13 +840,13 @@ lastnode++;
 
 if (inclusions != NULL && !IsWildItemIn(inclusions,lastnode))
    {
-   Verbose("cfengine: skipping non-included pattern %s\n",from);
+   Verbose("%s: Skipping non-included pattern %s\n",VPREFIX,from);
    return true;
    }
 
 if (IsWildItemIn(VEXCLUDELINK,lastnode) || IsWildItemIn(exclusions,lastnode))
    {
-   Verbose("cfengine: skipping excluded pattern %s\n",from);
+   Verbose("%s: Skipping excluded pattern %s\n",VPREFIX,from);
    return true;
    }
 
@@ -795,7 +864,7 @@ if (IsWildItemIn(VCOPYLINKS,lastnode) || IsWildItemIn(copy,lastnode))
    ip.backup = true;
    ip.exclusions = NULL;
    ip.symlink = NULL;
-   Verbose("cfengine: link item %s marked for copying instead\n",from);
+   Verbose("%s: Link item %s marked for copying instead\n",VPREFIX,from);
    CheckImage(to,from,&ip);
    return true;
    }
@@ -807,10 +876,8 @@ if (stat(to,&tobuf) == -1)
 
 if (! S_ISREG(tobuf.st_mode))
    {
-   if (! SILENT)
-      {
-      printf("%s: will only hard link regular files and %s is not regular\n",VPREFIX,to);
-      }
+   sprintf(OUTPUT,"%s: will only hard link regular files and %s is not regular\n",VPREFIX,to);
+   CfLog(cfsilent,OUTPUT,"");
    return true;
    }
 
@@ -818,7 +885,7 @@ Debug2("Trying to (hard) link %s -> %s\n",from,to);
 
 if (stat(from,&frombuf) == -1)
    {
-   DoHardLink(from,to);
+   DoHardLink(from,to,ptr->defines);
    return true;
    }
 
@@ -833,22 +900,26 @@ if (frombuf.st_ino != tobuf.st_ino && frombuf.st_dev != frombuf.st_dev)
 
    if (frombuf.st_mode == tobuf.st_mode && frombuf.st_size == tobuf.st_size)
       {
-      Verbose("Hard link (%s->%s) on different device APPEARS okay\n",from,to);
+      sprintf(OUTPUT,"Hard link (%s->%s) on different device APPEARS okay\n",from,to);
+      CfLog(cfverbose,OUTPUT,"");
       return true;
       }
    }
 
 if (frombuf.st_ino == tobuf.st_ino && frombuf.st_dev == frombuf.st_dev)
    {
-   Verbose("HARD link (%s=>%s) exists and is okay.\n",from,to);
+   sprintf(OUTPUT,"Hard link (%s->%s) exists and is okay.\n",from,to);
+   CfLog(cfverbose,OUTPUT,"");
    return true;
    }
 
-printf("%s: %s does not appear to be a hard link to %s\n",VPREFIX,from,to);
+sprintf(OUTPUT,"%s does not appear to be a hard link to %s\n",from,to);
+CfLog(cfinform,OUTPUT,""); 
 
 if (ENFORCELINKS)
    {
-   printf("%s: moving %s to %s.%s\n",VPREFIX,from,from,CF_SAVED);
+   sprintf(OUTPUT,"Moving %s to %s.%s\n",from,from,CF_SAVED);
+   CfLog(cfinform,OUTPUT,"");
 
    if (DONTDO)
       {
@@ -865,7 +936,7 @@ if (ENFORCELINKS)
       return(true);
       }
 
-   DoHardLink(from,to);
+   DoHardLink(from,to,ptr->defines);
    }
 
 return(true);
@@ -873,22 +944,27 @@ return(true);
 
 /*********************************************************************/
 
-DoHardLink (from,to)
+DoHardLink (from,to,defines)
 
-char *from, *to;
+char *from, *to, *defines;
 
 {
 if (DONTDO)
    {
-   Verbose("cfengine: Hardlink files %s -> %s\n\n",from,to);
+   printf("Hardlink files %s -> %s\n\n",from,to);
    }
 else
    {
-   Silent("%s: Hardlinking files %s -> %s\n\n",VPREFIX,from,to);
+   sprintf(OUTPUT,"Hardlinking files %s -> %s\n",from,to);
+   CfLog(cfsilent,OUTPUT,"");
 
    if (link(to,from) == -1)
       {
-      perror("link");
+      CfLog(cferror,"","link");
+      }
+   else
+      {
+      AddMultipleClasses(defines);
       }
    }
 }
@@ -914,7 +990,7 @@ Debug2("ExpandLinks(%s,%d)\n",from,level);
 
 if (level >= maxlinklevel)
    {
-   printf("%s: Too many levels of symbolic links to evaluate absolute path\n",VPREFIX);
+   CfLog(cferror,"Too many levels of symbolic links to evaluate absolute path\n");
    return false;
    }
 
@@ -956,8 +1032,8 @@ for (sp = from; *sp != '\0'; sp++)
 
    if (lstat(dest,&statbuf) == -1)  /* File doesn't exist so we can stop here */
       {
-      printf("%s: can't stat %s in ExpandLinks\n",VPREFIX,dest);
-      perror("stat");
+      sprintf(OUTPUT,"Can't stat %s in ExpandLinks\n",dest);
+      CfLog(cferror,OUTPUT,"stat");
       return false;
       }
 
@@ -967,8 +1043,8 @@ for (sp = from; *sp != '\0'; sp++)
       
       if (readlink(dest,buff,bufsize) == -1)
 	 {
-	 printf("cfengine: Expand links can't stat %s\n",dest);
-	 perror("readlink");
+	 sprintf(OUTPUT,"Expand links can't stat %s\n",dest);
+	 CfLog(cferror,OUTPUT,"readlink");
 	 return false;
 	 }
       else
@@ -979,7 +1055,6 @@ for (sp = from; *sp != '\0'; sp++)
 	    AddSlash(dest);
 	    if (BufferOverflow(dest,buff))
 	       {
-	       printf("cfengine: culprit ExpandLinks()\n");
 	       return false;
 	       }
 	    strcat(dest,buff);
@@ -1025,3 +1100,54 @@ for (sp = from; *sp != '\0'; sp++)
    }
 return true;
 }
+
+/*********************************************************************/
+
+char *
+AbsLinkPath (from,relto)
+
+char *from, *relto;
+
+/* Take an abolute source and a relative destination object
+   and find the absolute name of the to object */
+
+{ char *sp;
+  int pop = 1;
+ 
+if (*relto == '/')
+   {
+   printf("Cfengine internal error: call to AbsLInkPath with absolute pathname\n");
+   FatalError("");
+   }
+
+strcpy(CURRENTPATH,from);  /* reuse to save stack space */
+ 
+for (sp = relto; *sp != '\0'; sp++)
+   {
+   if (strncmp(sp,"../",3) == 0)
+      {
+      pop++;
+      sp += 2;
+      continue;
+      }
+
+   if (strncmp(sp,"./",2) == 0)
+      {
+      sp += 1;
+      continue;
+      }
+
+   break; /* real link */
+   }
+
+while (pop > 0)
+    {
+    ChopLastNode(CURRENTPATH);
+    pop--;
+    }
+
+AddSlash(CURRENTPATH);
+strcat(CURRENTPATH,sp);
+Debug("Reconstructed absolute linkname = %s\n",CURRENTPATH);
+return CURRENTPATH; 
+}
