@@ -81,11 +81,13 @@ void HandleSignal(signum)
 int signum;
  
 {
-printf("%s: Received signal %s\n",VPREFIX,SIGNALS[signum]);
+sprintf(OUTPUT,"Received signal %s\n",SIGNALS[signum]);
+CfLog(cfinform,OUTPUT,""); 
 
 if (signum == SIGTERM || signum == SIGINT || signum == SIGHUP)
    {
    ReleaseCurrentLock();
+   closelog();
    exit(0);
    }
 }
@@ -116,12 +118,22 @@ if (now == 0)
    return true;
    }
 
-Debug("GetLock(%s,%s,time=%d)\n",operator,operand,now);
+Debug("GetLock(%s,%s,time=%d), ExpireAfter=%d, IfElapsed=%d\n",operator,operand,now,expireafter,ifelapsed);
 
 sprintf(CFLOG,"%s/cfengine.%s.runlog",VLOGDIR,host);
 sprintf(CFLOCK,"%s/lock.%s.%s.%s.%s",VLOCKDIR,VCANONICALFILE,host,operator,operand);
 sprintf(CFLAST,"%s/last.%s.%s.%s.%s",VLOCKDIR,VCANONICALFILE,host,operator,operand);
 
+if (strlen(CFLOCK) > 254)
+   {
+   CFLOCK[253] = '\0';  /* most nodenames are 255 chars or less */
+   }
+
+if (strlen(CFLAST) > 254)
+   {
+   CFLAST[253] = '\0';  /* most nodenames are 255 chars or less */
+   }
+
 /* Look for non-existent (old) processes */
 
 lastcompleted = GetLastLock();
@@ -129,13 +141,15 @@ elapsedtime = (time_t)(now-lastcompleted) / 60;
 
 if (elapsedtime < 0)
    {
-   Verbose("Another cfengine seems to have done %s.%s since I started\n",operator, operand);
+   sprintf(OUTPUT,"Another cfengine seems to have done %s.%s since I started\n",operator, operand);
+   CfLog(cfverbose,OUTPUT,"");
    return false;
    }
 
 if (elapsedtime < ifelapsed)
    {
-   Verbose("Too soon since last run with %s.%s (%u/%u minutes)\n",operator,operand,elapsedtime,ifelapsed);
+   sprintf(OUTPUT,"Too soon since last run with %s.%s (%u/%u minutes)\n",operator,operand,elapsedtime,ifelapsed);
+   CfLog(cfverbose,OUTPUT,"");
    return false;
    }
 
@@ -148,13 +162,15 @@ if (lastcompleted != 0)
    {
    if (elapsedtime >= expireafter)
       {
-      Verbose("\n%s: atom lock %s expired...(after %u/%u minutes)\n",VPREFIX,CFLOCK,elapsedtime,expireafter);
+      sprintf(OUTPUT,"Lock %s expired...(after %u/%u minutes)\n",CFLOCK,elapsedtime,expireafter);
+      CfLog(cfinform,OUTPUT,"");
       
       pid = GetLockPid();
 
       if (pid == -1)
 	 {
-	 printf("%s: illegal pid in corrupt lock %s - ignoring lock\n",VPREFIX,CFLOCK);
+	 sprintf(OUTPUT,"Illegal pid in corrupt lock %s - ignoring lock\n",CFLOCK);
+	 CfLog(cferror,OUTPUT,"");
 	 }
       else
 	 {
@@ -170,8 +186,9 @@ if (lastcompleted != 0)
 
 	 if (kill(pid,SIGTERM) == ESRCH)
 	    {
-	    printf("%s: unable to kill expired process %d, exiting this time..\n",VPREFIX,pid);
-	    exit(1);
+	    sprintf(OUTPUT,"Unable to kill expired process %d, exiting this time..\n",pid);
+	    CfLog(cferror,OUTPUT,"");
+	    FatalError("");;
 	    }
 	 
 	 LockLog(pid,"Lock expired, process killed",operator,operand);
@@ -181,7 +198,7 @@ if (lastcompleted != 0)
       }
    else
       {
-      printf("Couldn't obtain lock for %s (already running!)\n",CFLOCK);
+      Verbose("Couldn't obtain lock for %s (already running!)\n",CFLOCK);
       return false;
       }
    }
@@ -205,16 +222,16 @@ Debug("ReleaseCurrentLock(%s)\n",CFLOCK);
 
 if (unlink(CFLOCK) == -1)
    {
-   Verbose("%s: unable to remove lock %s\n",VPREFIX,CFLOCK);
-   /* perror("unlink"); */
-   exit(1);
+   sprintf(OUTPUT,"Unable to remove lock %s\n",CFLOCK);
+   CfLog(cflogonly,OUTPUT,"unlink");
+   return;
    }
 
 if ((fd = creat(CFLAST,0644)) == -1)
    {
-   printf("%s: unable to create %s\n",VPREFIX,CFLAST);
-   perror("creat");
-   exit(1);
+   sprintf(OUTPUT,"Unable to create %s\n",CFLAST);
+   CfLog(cferror,OUTPUT,"creat");
+   FatalError("");
    }
 else
    {
@@ -239,8 +256,8 @@ time_t now;
 
 if ((dirh = opendir(VLOCKDIR)) == NULL)
    {
-   printf("%s: Can't open directory %s\n",VPREFIX,VLOCKDIR);
-   perror("opendir");
+   sprintf(OUTPUT,"Can't open directory %s\n",VLOCKDIR);
+   CfLog(cferror,OUTPUT,"opendir");
    return 0;
    }
 
@@ -277,9 +294,9 @@ if (stat(CFLAST,&statbuf) == -1)
    
    if ((fd = creat(CFLAST,0644)) == -1)
       {
-      printf("%s: unable to create %s\n",VPREFIX,CFLAST);
-      perror("creat");
-      exit(1);
+      sprintf("Unable to create %s\n",CFLAST);
+      CfLog(cferror,OUTPUT,"creat");
+      FatalError("");
       }
    else
       {
@@ -322,16 +339,14 @@ GetLockPid()
 
 if ((fp = fopen(CFLOCK,"r")) == NULL)
    {
-   printf("%s: GetLock weird error, lock disappeared!\n",VPREFIX);
-   perror("fopen");
-   exit(1);
+   CfLog(cferror,"GetLock weird error, lock disappeared!\n","fopen");
+   FatalError("");
    }
 
 fscanf(fp,"%d",&pid);
 
 fclose(fp);
 
-Debug("GetLockPid return pid = %d\n",pid);
 return pid;
 }
 
@@ -345,9 +360,9 @@ Debug("SetLock(%s)\n",CFLOCK);
 
 if ((fp = fopen(CFLOCK,"w")) == NULL)
    {
-   printf("%s: GetLock: can't open new lock file %s\n",VPREFIX,CFLOCK);
-   perror("fopen");
-   exit(1);
+   sprintf(OUTPUT,"GetLock: can't open new lock file %s\n",CFLOCK);
+   CfLog(cferror,OUTPUT,"fopen");
+   FatalError("");;
    }
 
 fprintf(fp,"%d\n",getpid());
@@ -371,9 +386,9 @@ Debug("LockLog(%s)\n",str);
 
 if ((fp = fopen(CFLOG,"a")) == NULL)
    {
-   printf("%s: GetLock: can't open log file %s\n",VPREFIX,CFLOG);
-   perror("fopen");
-   exit(1);
+   sprintf("GetLock: can't open log file %s\n",CFLOG);
+   CfLog(cferror,OUTPUT,"fopen");
+   FatalError("");
    }
 
 if ((tim = time((time_t *)NULL)) == -1)
