@@ -27,10 +27,27 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+
+char *HASH[hashtablesize];
+
+
 /*******************************************************************/
 /* Macro substitution based on HASH-table                          */
 /*******************************************************************/
 
+InitHashTable()
+
+{ int i;
+
+for (i = 0; i < hashtablesize; i++)
+   {
+   HASH[i] = 0;
+   }
+}
+
+
+/*******************************************************************/
+
 Hash(name)
 
 char *name;
@@ -85,7 +102,7 @@ if (HASH[slot] != 0)
    {
    if (CompareMacro(name,HASH[slot]) == 0)
       {
-      sprintf(VBUFF,"Redefinition of macro %s",name);
+      sprintf(VBUFF,"Redefinition of macro %s=%s",name,value);
       Warning(VBUFF);
       free(HASH[slot]);
       HASH[slot] = sp;
@@ -164,7 +181,7 @@ RecordMacroId(name)
 char *name;
 
 {
-Debug1("RecordMacroId(%s)\n",name);
+Debug("RecordMacroId(%s)\n",name);
 strcpy(CURRENTITEM,name);
 }
 
