@@ -51,7 +51,7 @@ GetNameInfo()
 if (uname(&VSYSNAME) == -1)
    {
    perror("uname ");
-   FatalError("cfengine: uname couldn't get kernel name info!!\n");
+   FatalError("Uname couldn't get kernel name info!!\n");
    }
 
 for (sp = VSYSNAME.sysname; *sp != '\0'; sp++)
@@ -100,7 +100,8 @@ if ((sp = malloc(strlen(VSYSNAME.nodename)+1)) == NULL)
    }
 
 strcpy(sp,VSYSNAME.nodename);
-strcpy(VPREFIX,VSYSNAME.nodename); /* Temporary */
+ 
+strcpy(VPREFIX,VSYSNAME.nodename);
 
 for (sp2=sp; *sp2 != '\0'; sp2++)  /* Truncate fully qualified name */
    {
@@ -204,15 +205,15 @@ sprintf(VBUFF,"%s_%s_%s_%s",VSYSNAME.sysname,VSYSNAME.machine,VSYSNAME.release,V
 
 if (strlen(VBUFF) < maxvarsize-2)
    {
-   strcpy(VARCH,VBUFF);
+   strcpy(VARCH,CanonifyName(VBUFF));
    }
 else
    {
    Verbose("cfengine internal: $(arch) overflows maxvarsize! Truncating\n");
-   strcpy(VARCH,VSYSNAME.sysname);
+   strcpy(VARCH,CanonifyName(VSYSNAME.sysname));
    }
 
-AddClassToHeap(CanonifyName(VARCH));
+AddClassToHeap(VARCH);
 
 Verbose("Additional hard class defined as: %s\n",VARCH);
 
@@ -264,7 +265,8 @@ else
    {
    if ((fd = creat(name,000)) == -1)      /* dummy mode ignored */
       {
-      printf("cfengine: creat(%s) failed\n",name);
+      sprintf(OUTPUT,"creat(%s) failed\n",name);
+      CfLog(cferror,OUTPUT,"creat");
       }
    else
       {
@@ -309,19 +311,22 @@ switch(action)
    case image:
               if (IsWildItemIn(VEXCLUDECOPY,lastnode) || IsWildItemIn(exclusions,lastnode))
                  {
-                 Verbose("cfengine: skipping excluded pattern %s\n",file);
+                 sprintf(OUTPUT,"Skipping excluded pattern %s\n",file);
+		 CfLog(cfverbose,OUTPUT,"");
                  return true;
                  }
    case links:
               if (IsWildItemIn(VEXCLUDELINK,lastnode) || IsWildItemIn(exclusions,lastnode))
                  {
-                 Verbose("cfengine: skipping excluded pattern %s\n",file);
+                 sprintf(OUTPUT,"kipping excluded pattern %s\n",file);
+		 CfLog(cfverbose,OUTPUT,"");
                  return true;
                  }
    default:
               if (IsWildItemIn(exclusions,lastnode))
                  {
-                 Verbose("cfengine: skipping excluded pattern %s\n",file);
+                 sprintf(OUTPUT,"Skipping excluded pattern %s\n",file);
+		 CfLog(cfverbose,OUTPUT,"");
                  return true;
                  }       
    }
@@ -341,3 +346,4 @@ Verbose("---------------------------------------------------------------------\n
 Verbose("%s\n",string);
 Verbose("---------------------------------------------------------------------\n\n");
 }
+
