@@ -140,6 +140,8 @@ for (ptr = VIMPORT; ptr != NULL; ptr=ptr->next)
       {
       continue;
       }
+
+   Verbose("Import file %s\n",ptr->name);
    
    filename[0] = '\0';
 
@@ -297,7 +299,13 @@ switch(ACTION)   /* Check for IP names in cfd */
 		 break;
    case deny:
                  InstallAuthItem(CURRENTAUTHPATH,id,&VDENY,&VDENYTOP,CLASSBUFF);
-		 break;		    
+		 break;
+
+   case acls:
+                 Debug1("ACL-alias: %s\n",id);
+		 strcpy(CURRENTITEM,id);
+                 InstallACL(id,CLASSBUFF);
+		 break;
    }
 
 
@@ -472,7 +480,7 @@ else
                            }
                         else
                            {
-                           yyerror("misc_mounts: host:/frompath /mounton_path [ro|rw]\n");
+                           yyerror("miscmounts: host:/frompath /mounton_path ro|rw\n");
                            }
                         break;
 
@@ -658,7 +666,7 @@ else
                           if (MOUNT_ONTO)
 			    {
                             yyerror ("Path not expected");
-                            FatalError("misc_mount: syntax error");
+                            FatalError("miscmounts: syntax error");
                             }
                           MOUNT_ONTO = true;
                           strcpy(MOUNTONTO,CURRENTPATH);
@@ -928,6 +936,10 @@ switch (ACTION)
    case deny:
                  InstallAuthItem(CURRENTAUTHPATH,wildcard,&VDENY,&VDENYTOP,CLASSBUFF);
 		 break;
+
+   case acls:
+                 AddACE(CURRENTITEM,wildcard,CLASSBUFF);
+		 break;
    default:
                  yyerror("Wildcards cannot be used in this context:");
    }
@@ -987,6 +999,11 @@ else if (pid == 0)                     /* child */
    {
    argv = (char **) malloc((argc+1)*sizeof(char *));
 
+   if (argv == NULL)
+      {
+      FatalError("Out of memory");
+      }
+   
    for (i = 0; i < argc; i++)
       {
       argv[i] = arg[i];
@@ -1007,6 +1024,7 @@ else                                    /* parent */
       {
       printf("%s: Wait for child failed\n",VPREFIX);
       perror("wait");
+      free(argv);
       return false;
       }
    else
@@ -1014,131 +1032,92 @@ else                                    /* parent */
       if (WIFSIGNALED(status))
          {
          Debug("%s: script %s returned: %d\n",VPREFIX,comm,WTERMSIG(status));
+	 free(argv);
          return false;
          }
 
       if (! WIFEXITED(status))
          {
+	 free(argv);
          return false;
          }
 
       if (WEXITSTATUS(status) == 0)
          {
+	 free(argv);
          return true;
          }
       else
          {
+	 free(argv);
          return false;
          }
       }
    }
 
+free(argv);
 return false;
 }
 
 
-/*******************************************************************/
-
-SplitCommand(comm,arg)
-
-char *comm, arg[maxshellargs][bufsize];
-
-{ char *sp;
-  int i = 0, j;
-  char buff[bufsize];
-
-for (sp = comm; *sp != '\0'; sp++)
-   {
-   bzero(buff,bufsize);
-
-   if (i >= maxshellargs-1)
-      {
-      yyerror("Too many arguments in embedded script");
-      FatalError("Use a wrapper");
-      }
-
-   while (*sp == ' ' || *sp == '\t')
-      {
-      sp++;
-      }
-
-   switch (*sp)
-      {
-      case '\"': sscanf (++sp,"%[^\"]",buff);
-                 break;
-      case '\'': sscanf (++sp,"%[^\']",buff);
-                 break;
-      case '`':  sscanf (++sp,"%[^`]",buff);
-	         break;
-      default:   sscanf (sp,"%s",buff);
-                 break;
-      }
-
-   for (j = 0; j < bufsize; j++)
-      {
-      arg[i][j] = buff[j];
-      }
-
-   sp += strlen(arg[i]);
-   i++;
-   }
-return (i);
-}
-
-
 /*******************************************************************/
 /* Toolkits Misc                                                   */
 /*******************************************************************/
 
 InitializeAction()                                   /* Set defaults */
 
-{
-Debug1("InitializeAction()\n");
-
-PLUSMASK = 0;
-MINUSMASK = 0;
-VRECURSE = 0;
-VAGE = 99999;
-VUIDNAME[0] = '*';
-VUIDNAME[1] = '\0';
-VGIDNAME[0] = '*';
-VGIDNAME[1] = '\0';
-HAVE_RESTART = 0;
-FILEACTION=warnall;
-CURRENTPATH[0] = '\0';
-CURRENTITEM[0] = '\0';
-ACTIONPENDING = false;
-DESTINATION[0] = '\0';
-IMAGEACTION[0] = '\0';
-PTRAVLINKS = (short) '?';
-FORCECOPY = false;
-IMAGEBACKUP = true;
-ROTATE=0;
-TIDYSIZE=0;
-PROMATCHES=-1;
-AGETYPE='a';
-COPYTYPE = 't';
-LINKDIRS = 'k';
-TIDYDIRS = false;
-VEXCLUDEPARSE = NULL;
-VINCLUDEPARSE = NULL;
-VCPLNPARSE = NULL;
-VTIMEOUT=0;
-strcpy(ALLCLASSBUFFER,"");
-strcpy(CFSERVER,"localhost");
-DISCOMP='>';
-DISABLESIZE=cfnosize;
-
+ {
+ Debug1("InitializeAction()\n");
+ 
+ PLUSMASK = (mode_t)0;
+ MINUSMASK = (mode_t)0;
+ VRECURSE = 0;
+ VAGE = 99999;
+ VUIDNAME[0] = '*';
+ VUIDNAME[1] = '\0';
+ VGIDNAME[0] = '*';
+ VGIDNAME[1] = '\0';
+ HAVE_RESTART = 0;
+ FILEACTION=warnall;
+ CURRENTPATH[0] = '\0';
+ CURRENTITEM[0] = '\0';
+ ACTIONPENDING = false;
+ DESTINATION[0] = '\0';
+ IMAGEACTION[0] = '\0';
+ PTRAVLINKS = (short) '?';
+ FORCECOPY = false;
+ IMAGEBACKUP = true;
+ ROTATE=0;
+ TIDYSIZE=0;
+ PROMATCHES=-1;
+ PROSIGNAL=0; 
+ AGETYPE='a';
+ COPYTYPE = 't';
+ LINKDIRS = 'k';
+ USESHELL = 'y';
+ PURGE = 'n';
+ TIDYDIRS = false;
+ VEXCLUDEPARSE = NULL;
+ VINCLUDEPARSE = NULL;
+ VACLBUILD = NULL;
+ VCPLNPARSE = NULL;
+ VTIMEOUT=0;
+ strcpy(ALLCLASSBUFFER,"");
+ strcpy(CFSERVER,"localhost");
+ DISCOMP='>';
+ DISABLESIZE=cfnosize;
+ 
  /* Make sure we don't clean the buffer in the middle of a link! */
 
-if ( ! ACTION_IS_LINK && ! ACTION_IS_LINKCHILDREN)
-   {
-   LINKFROM[0] = '\0';
-   LINKTO[0] = '\0';
-   LINKSILENT = false;
-   LINKTYPE = 's';
-   }
-}
+ if ( ! ACTION_IS_LINK && ! ACTION_IS_LINKCHILDREN)
+    {
+    LINKFROM[0] = '\0';
+    LINKTO[0] = '\0';
+    LINKSILENT = false;
+    LINKTYPE = 's';
+    DEADLINKS = false;
+    }
+ }
 
 /*********************************************************************/
 
@@ -1146,16 +1125,16 @@ SetMountPath (value)
 
 char *value;
 
-{ char buff[bufsize];
+ { char buff[bufsize];
 
-bzero(buff,bufsize);
+ bzero(buff,bufsize);
 
-ExpandVarstring(value,buff,"");
-
-Debug("Appending [%s] to mountlist\n",buff);
-
-AppendItem(&VMOUNTLIST,buff,CLASSBUFF);
-}
+ ExpandVarstring(value,buff,"");
+ 
+ Debug("Appending [%s] to mountlist\n",buff);
+ 
+ AppendItem(&VMOUNTLIST,buff,CLASSBUFF);
+ }
 
 /*********************************************************************/
 
@@ -1163,18 +1142,18 @@ SetRepository (value)
 
 char *value;
 
-{
-if (*value != '/')
-   {
-   yyerror("File repository must be an absolute directory name");
-   }
+ {
+  if (*value != '/')
+     {
+     yyerror("File repository must be an absolute directory name");
+     }
+  
+  if (VREPOSITORY[0] != '\0')
+     {
+     yyerror("Redefinition of system variable repository");
+     }
 
-if (VREPOSITORY[0] != '\0')
-   {
-   yyerror("Redefinition of system variable repository");
-   }
-
-ExpandVarstring(value,VREPOSITORY,"");
-}
+  ExpandVarstring(value,VREPOSITORY,"");
+ }
 
 /* EOF */
