@@ -34,6 +34,8 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+int DEADLOCK = false;
+FILE *PIPE;
 
 /*******************************************************************/
 
@@ -45,22 +47,22 @@ char *psopts;
 { FILE *pp;
   char pscomm[maxlinksize];
 
-sprintf(pscomm,"%s %s 2<&1",VPSCOMM[VSYSTEMHARDCLASS],psopts);
+sprintf(pscomm,"%s %s",VPSCOMM[VSYSTEMHARDCLASS],psopts);
   
-if ((pp = popen(pscomm,"r")) == NULL)
+if ((pp = cfpopen(pscomm,"r")) == NULL)
    {
-   printf("%s: couldn't open the process list with command\n",VPREFIX);
-   printf("%s: %s\n",VPREFIX,pscomm);
+   sprintf(OUTPUT,"Couldn't open the process list with command %s\n",pscomm);
+   CfLog(cferror,OUTPUT,"popen");
    return false;
    }
 
 while (!feof(pp))
    {
-   fgets(VBUFF,bufsize,pp);
+   ReadLine(VBUFF,bufsize,pp);
    AppendItem(procdata,VBUFF,"");
    }
 
-pclose(pp);
+cfpclose(pp);
 return true;
 }
 
@@ -89,8 +91,10 @@ buf.translate = 0;
 
 if ((err = re_compile_pattern(pp->expr,strlen(pp->expr),&buf)) != 0)
    {
-   printf("%s: Regular expression error for process pattern %s\n",VPREFIX,pp->expr);
-   printf("%s: %s\n",VPREFIX,err);
+   sprintf(OUTPUT,"Regular expression error for process pattern %s\n",pp->expr);
+   CfLog(cferror,OUTPUT,"");
+   sprintf(OUTPUT,"%s\n",err);
+   CfLog(cferror,OUTPUT,"");
    return;
    }
 
@@ -119,7 +123,8 @@ for (ip = procdata; ip != NULL; ip=ip->next)
       
       if (pid == -1)
 	 {
-	 Verbose("cfengine: Unable to extract pid while looking for %s\n",pp->expr);
+	 sprintf(OUTPUT,"Unable to extract pid while looking for %s\n",pp->expr);
+	 CfLog(cfverbose,OUTPUT,"");
 	 continue;
 	 }
 
@@ -139,26 +144,30 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 
       if (pp->action == 'w')
 	 {
-	 printf("%s: Process warning, found: %s\n",VPREFIX,ip->name);
+	 sprintf(OUTPUT,"Process warning, found: %s\n",ip->name);
+	 CfLog(cfinform,OUTPUT,"");
 	 continue;
 	 }
 
       if (pp->signal != cfnosignal)
 	 {
-	 Verbose("%s: signalling process %d with %s\n",VPREFIX,pid,SIGNALS[pp->signal]);
+	 sprintf(OUTPUT,"Signalling process %d (%s) with %s\n",pid,pp->expr,SIGNALS[pp->signal]);
+	 CfLog(cfinform,OUTPUT,"");
 
 	 if (!DONTDO)
 	    {
 	    if ((ret = kill((pid_t)pid,pp->signal)) < 0)
 	       {
-	       Verbose("%s: couldn't send signal to pid %d\n",VPREFIX,pid);
-	       if (VERBOSE || DEBUG || D2) perror("kill");
+	       sprintf(OUTPUT,"Couldn't send signal to pid %d\n",pid);
+	       CfLog(cferror,OUTPUT,"kill");
+
 	       continue;
 	       }
 
 	    if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
 	       {
-	       printf("%s: killed: %s\n",VPREFIX,ip->name);
+	       sprintf(OUTPUT,"Killed: %s\n",ip->name);
+	       CfLog(cfinform,OUTPUT,"");
 	       }
 	    }
 	 }
@@ -166,30 +175,35 @@ for (ip = procdata; ip != NULL; ip=ip->next)
       }
    else if (match < -1)
       {
-      printf("%s: Regular expression internal error [%s]\n",VPREFIX,pp->expr); 
+      sprintf(OUTPUT,"Regular expression internal error [%s]\n",pp->expr);
+      CfLog(cferror,OUTPUT,"");
       }
    }
 
 if (pp->matches >= 0)
    {
+   AddMultipleClasses(pp->defines);
+
    switch (pp->comp)
       {
       case '=': if (matches != (int)pp->matches)
                    {
-	    	   printf("%s: %d processes matched %s (should be %d)\n",
-			  VPREFIX,matches,pp->expr,pp->matches);
+	    	   sprintf(OUTPUT,"%d processes matched %s (should be %d)\n",matches,pp->expr,pp->matches);
+		   CfLog(cfinform,OUTPUT,"");
 		   }
                 break;
+		
       case '>': if (matches <= (int)pp->matches)
 	           {
-	    	   printf("%s: %d processes matched %s (should be >%d)\n",
-			  VPREFIX,matches,pp->expr,pp->matches);
+	    	   sprintf(OUTPUT,"%d processes matched %s (should be >%d)\n",matches,pp->expr,pp->matches);
+		   CfLog(cfinform,OUTPUT,"");
 	           } 
                 break;
+		
       case '<': if (matches >= (int)pp->matches)
 	           {
-	    	   printf("%s: %d processes matched %s (should be <%d)\n",
-			  VPREFIX,matches,pp->expr,pp->matches);
+	    	   sprintf(OUTPUT,"%d processes matched %s (should be <%d)\n",matches,pp->expr,pp->matches);
+		   CfLog(cfinform,OUTPUT,"");
 	           } 
       }
    }
@@ -202,28 +216,42 @@ if (strlen(pp->restart) != 0)
       return;
       }
 
-   Verbose("cfengine: Executing shell command: %s\n",pp->restart);
+   sprintf(OUTPUT,"Executing shell command: %s\n",pp->restart);
+   CfLog(cfinform,OUTPUT,"");
 
    if (DONTDO)
       {
       return;
       }
-   
-   if ((pipe = popen(pp->restart,"r")) == NULL)
-      {
-      printf ("%s: process restart execution failed on %s\n",VPREFIX,pp->restart);
-      perror("popen");
-      return;
-      }
-   
-   signal(SIGALRM,(void *)BreakPipe);
-   alarm(5);
-   
-   while (!feof(pipe))
-      {
-      fgets(line,bufsize,pipe);
 
-      if (feof(pipe) || ferror(pipe))
+   if (pp->useshell == 'y')
+      {
+      if ((PIPE = popen(pp->restart,"r")) == NULL)
+	 {
+	 sprintf(OUTPUT,"Process restart execution failed on %s\n",pp->restart);
+	 CfLog(cferror,OUTPUT,"popen");
+	 return;
+	 }
+      }
+   else
+      {
+      if ((PIPE = cfpopen(pp->restart,"r")) == NULL)
+	 {
+	 sprintf(OUTPUT,"Process restart execution failed on %s\n",pp->restart);
+	 CfLog(cferror,OUTPUT,"popen");
+	 return;
+	 }
+      }
+
+   DEADLOCK = false;   
+   signal(SIGALRM,(void *)BreakPipe);
+   alarm(30);
+   
+   while (!feof(PIPE))
+      {
+      ReadLine(line,bufsize,PIPE);
+
+      if (feof(PIPE) || ferror(PIPE))
 	 {
 	 break;
 	 }
@@ -232,15 +260,33 @@ if (strlen(pp->restart) != 0)
 	 {
 	 break;
 	 }
+
+      /* patch for ERESTARTSYSTEM bug in popen */
+
+      if (strstr(line,"cfd: start") || DEADLOCK)
+	 {
+         printf("");
+	 break;
+	 }
       
-      Verbose("restart: %s",line);
+      sprintf("Restart: %s",line);
+      CfLog(cfinform,OUTPUT,"");
       }
 
    alarm(0);
    signal(SIGALRM,SIG_DFL);
+
+   if (pp->useshell == 'y')
+      {
+      pclose(PIPE);
+      }
+   else
+      {
+      cfpclose(PIPE);
+      }
    
-   pclose(pipe);
-   Verbose("(Done with %s)\n",pp->restart);
+   sprintf(OUTPUT,"(Done with %s)\n",pp->restart);
+   CfLog(cfinform,OUTPUT,"");
    }
 }
 
@@ -249,4 +295,8 @@ if (strlen(pp->restart) != 0)
 BreakPipe()
 
 {
+Verbose("%s: Workaroud on ERESTARTSYS popen() zombie deadlock bug\n",VPREFIX);
+ungetc('\n',PIPE);
+ungetc(EOF,PIPE);
+DEADLOCK = true;
 }
