@@ -52,8 +52,8 @@ sprintf(sendbuff,"AUTH %s %s",ip_string,fqname);
 
 if ((n = send(sd,sendbuff,bufsize,0)) != bufsize)
    {
-   printf("socket verify only sent %d bytes\n",n);
-   perror("send");
+   sprintf(OUTPUT,"socket verify only sent %d bytes\n",n);
+   CfLog(cferror,OUTPUT,"send");
    return false;
    }
 
