@@ -68,11 +68,11 @@ char *VVNAMES[] =
    "quote",
    "dollar",
    "repchar",
-   "split",
+   "Split",
    "underscoreclasses",
    "interfacename",
-   "ExpireAfter",
-   "IfElapsed",
+   "expireafter",
+   "ifelapsed",
    NULL
    };
 
@@ -147,6 +147,11 @@ char *string, *buffer, *bserver;
   char currentitem[bufsize], scanstr[6];
   int len;
 
+if (string == 0 || strlen(string) == 0)
+   {
+   return false;
+   }
+  
 buffer[0] = '\0';
 
 for (sp = string; /* No exit */ ; sp++)       /* check for varitems */
@@ -204,11 +209,22 @@ for (sp = string; /* No exit */ ; sp++)       /* check for varitems */
 
 
          case cfhost:
-	     	    if (BufferOverflow(buffer,VDEFAULTBINSERVER.name))
-		      {
-		      FatalError("Can't expand varstring");
-		      }
-                    strcat(buffer,VUQNAME);
+	            if (strlen(VUQNAME) == 0)
+		       {
+		       if (BufferOverflow(buffer,VDEFAULTBINSERVER.name))
+			  {
+			  FatalError("Can't expand varstring");
+			  }
+		       strcat(buffer,VDEFAULTBINSERVER.name);
+		       }
+		    else
+		       {
+		       if (BufferOverflow(buffer,VUQNAME))
+			  {
+			  FatalError("Can't expand varstring");
+			  }
+		       strcat(buffer,VUQNAME);
+		       }
                     break;
 
 	 case cffqhost:
@@ -497,7 +513,7 @@ char *name;
 
 for (i = 0; VVNAMES[i] != '\0'; i++)
    {
-   if (strcmp(VVNAMES[i],name) == 0)
+   if (strcmp(VVNAMES[i],ToLowerStr(name)) == 0)
       {
       return (enum vnames) i;
       }
