@@ -95,7 +95,8 @@ if (! IsWildCard(extract))
    strcat(buffer,"/");
    if (BufferOverflow(buffer,extract))
        {
-       printf("cfengine:%s: culprit %s\n",VPREFIX,extract);
+       sprintf(OUTPUT,"Culprit %s\n",extract);
+       CfLog(cferror,OUTPUT,"");
        }
    strcat(buffer,extract);
    ExpandWildCardsAndDo(rest,buffer,function,argptr);
@@ -107,8 +108,8 @@ else
    
    if ((dirh=opendir(buffer)) == NULL)
       {
-      printf("Can't open dir: %s\n",buffer);
-      perror("opendir");
+      sprintf(OUTPUT,"Can't open dir: %s\n",buffer);
+      CfLog(cferror,OUTPUT,"opendir");
       return;
       }
 
@@ -128,8 +129,8 @@ else
 
       if (stat(buffer,&statbuf) == -1)
          {
-         printf("cfengine:%s: Can't stat %s\n\n",VPREFIX,buffer);
-         perror("stat");
+         sprintf(OUTPUT,"Can't stat %s\n\n",buffer);
+         CfLog(cferror,OUTPUT,"stat");
          continue;
          }
  
@@ -142,7 +143,8 @@ else
  
    if (count == 0)
       {
-      Verbose("cfengine: No directories matching %s in %s\n",extract,buffer);
+      sprintf(OUTPUT,"No directories matching %s in %s\n",extract,buffer);
+      CfLog(cfinform,OUTPUT,"");
       return;
       }
    closedir(dirh);
